@@ -36,13 +36,13 @@ import ir
 class account_invoice(osv.osv):
 	def _amount_untaxed(self, cr, uid, ids, prop, unknow_none,unknow_dict):
 		id_set=",".join(map(str,ids))
-		cr.execute("SELECT s.id,COALESCE(SUM(l.price_unit*l.quantity),0) AS amount FROM account_invoice s LEFT OUTER JOIN account_invoice_line l ON (s.id=l.invoice_id) WHERE s.id IN ("+id_set+") GROUP BY s.id ")
+		cr.execute("SELECT s.id,COALESCE(SUM(l.price_unit*l.quantity),0)::decimal(16,2) AS amount FROM account_invoice s LEFT OUTER JOIN account_invoice_line l ON (s.id=l.invoice_id) WHERE s.id IN ("+id_set+") GROUP BY s.id ")
 		res=dict(cr.fetchall())
 		return res
 
 	def _amount_tax(self, cr, uid, ids, prop, unknow_none,unknow_dict):
 		id_set=",".join(map(str,ids))
-		cr.execute("SELECT s.id,COALESCE(SUM(l.amount),0) AS amount FROM account_invoice s LEFT OUTER JOIN account_invoice_tax l ON (s.id=l.invoice_id) WHERE s.id IN ("+id_set+") GROUP BY s.id ")
+		cr.execute("SELECT s.id,COALESCE(SUM(l.amount),0)::decimal(16,2) AS amount FROM account_invoice s LEFT OUTER JOIN account_invoice_tax l ON (s.id=l.invoice_id) WHERE s.id IN ("+id_set+") GROUP BY s.id ")
 		res=dict(cr.fetchall())
 		return res
 
@@ -59,6 +59,11 @@ class account_invoice(osv.osv):
 		type2journal = {'out_invoice': 'sale', 'in_invoice': 'purchase', 'out_refund': 'sale', 'in_refund': 'purchase'}
 		cr.execute("select id from account_journal where type=%s limit 1", (type2journal.get(type_inv, 'sale'),))
 		return cr.fetchone()[0]
+	
+	def _get_currency(self, cr, uid, context):
+		cr.execute("select id from res_currency where rate=1.0")
+		res = cr.fetchone()
+		return res and res[0] or False
 
 	def _get_journal_analytic(self, cr, uid, type_inv, context={}):
 		type2journal = {'out_invoice': 'sale', 'in_invoice': 'purchase', 'out_refund': 'sale', 'in_refund': 'purchase'}
@@ -115,14 +120,15 @@ class account_invoice(osv.osv):
 		'amount_untaxed': fields.function(_amount_untaxed, method=True, string='Untaxed Amount'),
 		'amount_tax': fields.function(_amount_tax, method=True, string='Tax Amount'),
 		'amount_total': fields.function(_amount_total, method=True, string='Total Amount'),
-		'currency_id' : fields.many2one('res.currency', 'Currency', required=True),
-		'journal_id' : fields.many2one('account.journal', 'Journal', required=True),
+		'currency_id': fields.many2one('res.currency', 'Currency', required=True),
+		'journal_id': fields.many2one('account.journal', 'Journal', required=True),
 	}
 	_defaults = {
 		'type': lambda *a: 'out_invoice',
 		'date_invoice': lambda *a: time.strftime('%Y-%m-%d'),
 		'state': lambda *a: 'draft',
 		'journal_id': _get_journal,
+		'currency_id': _get_currency,
 	}
 	
 	def unlink(self, cr, uid, ids):
@@ -140,39 +146,29 @@ class account_invoice(osv.osv):
 #		res = self.pool.get('res.partner').address_get(cr, uid, [part], ['invoice'])
 #		return [{}]
 
-	def onchange_partner_id(self, cr, uid, ids, type, part):
+	def onchange_partner_id(self, cr, uid, ids, type, partner_id):
 		invoice_addr_id = False
 		contact_addr_id = False
-		opt=[('uid',str(uid))]
-		if part:
-			opt.insert(0,('id',part))
-			res = self.pool.get('res.partner').address_get(cr, uid, [part], ['contact','invoice'])
+		acc_id = False
+		opt = [('uid', str(uid))]
+		if partner_id:
+			opt.insert(0, ('id', partner_id))
+			res = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['contact', 'invoice'])
 			contact_addr_id = res['contact']
 			invoice_addr_id = res['invoice']
-		if type in ('out_invoice','out_refund'):
-			acc_id=ir.ir_get(cr,uid,'meta','account.receivable', [('res.partner',part)])[0][2]
-		else:
-			acc_id=ir.ir_get(cr,uid,'meta','account.payable', [('res.partner',part)])[0][2]
-		result = {'value':{'address_contact_id': contact_addr_id, 'address_invoice_id': invoice_addr_id, 'account_id':acc_id }}
-		if part:
-			pt = self.pool.get('res.partner').browse(cr, uid, part).payment_term
-			if pt:
-				result['value']['payment_term'] = pt.id
+			p = self.pool.get('res.partner').browse(cr, uid, partner_id)
+			if type in ('out_invoice', 'out_refund'):
+				acc_id = p.property_account_receivable
+			else:
+				acc_id = p.property_account_payable
+		result = {'value': {'address_contact_id': contact_addr_id, 'address_invoice_id': invoice_addr_id, 'account_id': acc_id}}
+		if partner_id and p.property_payment_term:
+			result['value']['payment_term'] = p.property_payment_term
 		return result
 
 	def onchange_currency_id(self, cr, uid, ids, curr_id):
-# Uncomment this when ids will really contain the list of ids
-#		print str(ids)
-#		if curr_id:
-#			ilines = self.pool.get('account.invoice.line')
-#			currency = self.pool.get('res.currency')
-#			for id in ids:
-#				old_curr = self.read(cr, uid, [id], ['currency_id'])[0]['currencyid'][0]
-#				for iline_id in ilines.search(cr, uid, [('invoice_id', '=' , id),]):
-#					old_price = ilines.read(cr, uid, [iline_id], ['price_unit'])[0]['priceunit']
-#					ilines.write(cr, uid, [iline_id], {'price_unit': currency.compute(cr, uid, old_curr, curr_id, old_price),})
 		return {}
-	
+
 	# go from canceled state to draft state
 	def action_cancel_draft(self, cr, uid, ids, *args):
 		self.write(cr, uid, ids, {'state':'draft'})
@@ -190,23 +186,26 @@ class account_invoice(osv.osv):
 		ml = self.pool.get('account.move.line')
 		for inv in self.read(cr, uid, ids, ['move_id','account_id']):
 			if inv['move_id']:
-				move_line_id = ml.search(cr, uid, [('move_id','=',inv['move_id'][0])])
-				for line in ml.read(cr, uid, move_line_id, ['account_id']):
+				move_line_ids = ml.search(cr, uid, [('move_id', '=', inv['move_id'][0])])
+				for line in ml.read(cr, uid, move_line_ids, ['account_id']):
 					if line['account_id']==inv['account_id']:
 						return [line['id']]
-		return False
+		return []
 
-	def copy(self, cr, uid, id, default=None):
+	def copy(self, cr, uid, id, default=None, context={}):
 		if not default: default = {}
-		default.update( {'state':'draft', 'number':False, 'move_id':False} )
-		return super(account_invoice, self).copy(cr, uid, id, default)
+		default.update({'state':'draft', 'number':False, 'move_id':False})
+		return super(account_invoice, self).copy(cr, uid, id, default, context)
 
 	def test_paid(self, cr, uid, ids, *args):
-		res = self.move_line_id_payment_get(cr, uid, ids)[0]
+		res = self.move_line_id_payment_get(cr, uid, ids)
 		if not res:
 			return False
-		cr.execute('select reconcile_id from account_move_line where id=%d', (res,))
-		return bool(cr.fetchone()[0])
+		ok = True
+		for id in res:
+			cr.execute('select reconcile_id from account_move_line where id=%d', (id,))
+			ok = ok and  bool(cr.fetchone()[0])
+		return ok
 
 	def button_compute(self, cr, uid, ids, context={}):
 		for id in ids:
@@ -219,8 +218,8 @@ class account_invoice(osv.osv):
 		cr.execute('SELECT * FROM account_invoice WHERE move_id IS NULL AND id IN ('+id_set+')')
 		for inv in cr.dictfetchall():
 			# create the analytical lines
-			invoice_lines = self.read(cr, uid, [inv['id']], ['invoice_line'])[0]['invoice_line']
-			ils = self.pool.get('account.invoice.line').read(cr, uid, invoice_lines)
+			line_ids = self.read(cr, uid, [inv['id']], ['invoice_line'])[0]['invoice_line']
+			ils = self.pool.get('account.invoice.line').read(cr, uid, line_ids)
 			if inv['type'] in ('out_invoice', 'in_refund'):
 				sign = 1
 			else:
@@ -228,9 +227,10 @@ class account_invoice(osv.osv):
 			if inv['project_id']:
 				for il in ils:
 					self.pool.get('account.analytic.line').create(cr, uid, {
-						'name': il['name'],
+						'name': 'Inv.'+ str(inv['number'])+': '+il['name'],
 						'date': time.strftime('%Y-%m-%d'),
 						'account_id': inv['project_id'],
+						'unit_amount': il['quantity'],
 						'amount': il['price_subtotal'] * sign,
 						'general_account_id': il['account_id'][0],
 						'journal_id': self._get_journal_analytic(cr, uid, inv['type'])
@@ -246,13 +246,14 @@ class account_invoice(osv.osv):
 			total = 0
 			total_currency = 0
 			for i in iml:
-				if inv['currency_id']<>company_currency:
+				if inv['currency_id'] != company_currency:
 					i['currency_id'] = inv['currency_id']
 					i['amount_currency'] = i['price']
 					i['price'] = self.pool.get('res.currency').compute(cr, uid, inv['currency_id'], company_currency, i['price'])
 				else:
 					i['amount_currency'] = False
 					i['currency_id'] = False
+				i['ref'] = inv['number']
 				if inv['type'] in ('out_invoice','in_refund'):
 					total += i['price']
 					total_currency += i['amount_currency'] or i['price']
@@ -262,40 +263,39 @@ class account_invoice(osv.osv):
 					total_currency -= i['amount_currency'] or i['price']
 			acc_id = inv['account_id']
 
+			name = inv['name']
 			if inv['payment_term']:
 				totlines = self.pool.get('account.payment.term').compute(cr, uid, inv['payment_term'], total)
 				for t in totlines:
-					iml.append({ 'type':'dest', 'name':inv['name'], 'price': t[1], 'date_maturity': t[0], 'account_id':acc_id })
+					iml.append({ 'type':'dest', 'name':name, 'price': t[1], 'date_maturity': t[0], 'account_id':acc_id, 'ref': inv['number']})
 			else:
-				iml.append({ 'type':'dest', 'name':inv['name'], 'price': total, 'account_id':acc_id, 'date_maturity' : inv['date_due'] or False, 'amount_currency': total_currency, 'currency_id': inv['currency_id']})
+				iml.append({ 'type':'dest', 'name':name, 'price': total, 'account_id':acc_id, 'date_maturity' : inv['date_due'] or False, 'amount_currency': total_currency, 'currency_id': inv['currency_id'], 'ref': inv['number']})
 
 			date = inv['date_invoice']
 			part = inv['partner_id']
-			line = map(lambda x:(0,0,{ 'date':date, 'date_maturity': x.get('date_maturity', False), 'partner_id':part, 'name':x['name'], 'debit':x['price']>0 and x['price'], 'credit':x['price']<0 and -x['price'], 'account_id':x['account_id'], 'amount_currency':x.get('amount_currency', False), 'currency_id':x.get('currency_id', False) }) ,iml)
+			line = map(lambda x:(0,0,{ 'date':date, 'date_maturity': x.get('date_maturity', False), 'partner_id':part, 'name':x['name'], 'debit':x['price']>0 and x['price'], 'credit':x['price']<0 and -x['price'], 'account_id':x['account_id'], 'amount_currency':x.get('amount_currency', False), 'currency_id':x.get('currency_id', False), 'ref':x.get('ref',False) }) ,iml)
 
-			# create the move and all its lines
-			invtype = dict(self._columns['type'].selection)[inv['type']]
-			name = '%s, %s' % (invtype, inv['name'])
-			move = { 'name': name, 'line_id': line, 'journal_id' : self._get_journal(cr, uid, {'type' : inv['type']})}
+			journal_id = inv['journal_id'] #self._get_journal(cr, uid, {'type': inv['type']})
+			journal = self.pool.get('account.journal').browse(cr, uid, journal_id)
+			if journal.sequence_id:
+				name = self.pool.get('ir.sequence').get_id(cr, uid, journal.sequence_id.id)
+
+			move = {'name': name, 'line_id': line, 'journal_id': journal_id}
 			move_id = self.pool.get('account.move').create(cr, uid, move)
 
 			# make the invoice point to that move
-			self.write(cr, uid, [inv['id']], {'move_id':move_id})
+			self.write(cr, uid, [inv['id']], {'move_id': move_id})
 
 		self._log_event(cr, uid, ids)
 		return True
 
 	def action_number(self, cr, uid, ids, *args):
-		cr.execute('SELECT id,type,move_id,number,name FROM account_invoice WHERE id IN ('+','.join(map(str,ids))+')')
-		for (id,type,move_id,number,name) in cr.fetchall():
+		cr.execute('SELECT id, type, number, move_id FROM account_invoice WHERE id IN ('+','.join(map(str,ids))+')')
+		for (id, invtype, number, move_id) in cr.fetchall():
 			if not number:
-				number = self.pool.get('ir.sequence').get(cr, uid, 'account.invoice.'+type)
-				cr.execute('UPDATE account_invoice SET number=%s WHERE id=%d', (number,id))
-			if move_id:
-				invtype = dict(self._columns['type'].selection)[type]
-				name = '%s %s, %s' % (invtype, number or '', name)
-				name = name[:60]
-				cr.execute('UPDATE account_move SET name=%s WHERE id=%d', (name, move_id))
+				number = self.pool.get('ir.sequence').get(cr, uid, 'account.invoice.'+invtype)
+				cr.execute('UPDATE account_invoice SET number=%s WHERE id=%d', (number, id))
+				cr.execute('UPDATE account_move_line SET ref=%s WHERE move_id=%d and ref is null', (number, move_id))
 		return True
 
 	def action_cancel(self, cr, uid, ids, *args):
@@ -303,7 +303,8 @@ class account_invoice(osv.osv):
 		for i in invoices:
 			if i['move_id']:
 				# delete the move this invoice was pointing to
-				# Note that the corresponding move_lines and move_reconciles will be automatically deleted too
+				# Note that the corresponding move_lines and move_reconciles 
+				# will be automatically deleted too
 				self.pool.get('account.move').unlink(cr, uid, [i['move_id'][0]])
 		self.write(cr, uid, ids, {'state':'cancel', 'move_id':False})
 		self._log_event(cr, uid, ids,-1.0, 'Cancel Invoice')
@@ -317,7 +318,7 @@ class account_invoice(osv.osv):
 		for inv in invoices:
 			for tax in inv.tax_line:
 				if not tax['name'] in taxes:
-					taxes[tax['name']] = {'name':tax['name']}
+					taxes[tax['name']] = {'name': tax['name']}
 		return taxes.values()
 
 	def _log_event(self, cr, uid, ids, factor=1.0, name='Open Invoice'):
@@ -366,7 +367,7 @@ class account_invoice(osv.osv):
 					del line['id']
 					del line['invoice_id']
 					line['account_id'] = line['account_id'] and line['account_id'][0]
-					line['invoice_line_tax_id'] = [(6,0, line['invoice_line_tax_id']) ]
+					line['invoice_line_tax_id'] = [(6,0, line.get('invoice_line_tax_id', [])) ]
 				return map(lambda x: (0,0,x), lines)
 				
 			invoice_lines = self.pool.get('account.invoice.line').read(cr, uid, invoice['invoice_line'])
@@ -393,6 +394,46 @@ class account_invoice(osv.osv):
 			# create the new invoice
 			new_ids.append(self.create(cr, uid, invoice))
 		return new_ids
+		
+	def pay_and_reconcile(self, cr, uid, ids, pay_amount, pay_account_id, pay_journal_id, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context={}):
+		assert len(ids)==1, "Can only pay one invoice at a time"
+		invoice = self.browse(cr, uid, ids[0])
+		src_account_id = invoice.account_id.id
+		journal = self.pool.get('account.journal').browse(cr, uid, pay_journal_id)
+		if journal.sequence_id:
+			name = self.pool.get('ir.sequence').get_id(cr, uid, journal.sequence_id.id)
+		else:
+			raise osv.except_osv('No piece number !', 'Can not create an automatic sequence for this piece !\n\nPut a sequence in the journal definition for automatic numbering or create a sequence manually for this piece.')
+		types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
+		direction = types[invoice.type]
+		l1 = {
+			'name': name,
+			'debit': direction == 1 and pay_amount,
+			'credit': direction == -1 and pay_amount,
+			'account_id': src_account_id,
+			'partner_id': invoice.partner_id.id
+		}
+		l2 = {
+			'name':name,
+			'debit': direction == -1 and pay_amount,
+			'credit': direction == 1 and pay_amount,
+			'account_id': pay_account_id,
+			'partner_id': invoice.partner_id.id
+		}
+		lines = [(0, 0, l1), (0, 0, l2)]
+		move = {'name': name, 'line_id': lines, 'journal_id': pay_journal_id}
+		move_id = self.pool.get('account.move').create(cr, uid, move)
+		
+		line_ids = []
+		line = self.pool.get('account.move.line')
+		cr.execute('select id from account_move_line where move_id in ('+str(move_id)+','+str(invoice.move_id.id)+')')
+		lines = line.browse(cr, uid, map(lambda x: x[0], cr.fetchall()) )
+		for l in lines:
+			if l.account_id.id==src_account_id:
+				line_ids.append(l.id)
+
+		self.pool.get('account.move.line').reconcile(cr, uid, line_ids, 'manual', writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context)
+		return True
 account_invoice()
 
 class account_invoice_line(osv.osv):
@@ -404,23 +445,25 @@ class account_invoice_line(osv.osv):
 	_name = "account.invoice.line"
 	_description = "Invoice line"
 	_columns = {
-		'name': fields.char('Description', size=64, required=True),
+		'name': fields.char('Description', size=256, required=True),
 		'invoice_id': fields.many2one('account.invoice', 'Invoice Ref', ondelete='cascade'),
-		'uos_id': fields.many2one('account.uos', 'Unit of Sale', ondelete='set null'),
-		'account_id':fields.many2one('account.account', 'Source Account', required=True, domain=[('type','<>','view')]),
+		'uos_id': fields.many2one('account.uos', 'Unit', ondelete='set null'),
+		'account_id': fields.many2one('account.account', 'Source Account', required=True, domain=[('type','<>','view')]),
 		'price_unit': fields.float('Unit Price', required=True),
 		'price_subtotal': fields.function(_amount_line, method=True, string='Subtotal'),
 		'quantity': fields.float('Quantity', required=True),
 		'invoice_line_tax_id': fields.many2many('account.tax', 'account_invoice_line_tax', 'invoice_line_id', 'tax_id', 'Taxes', domain=[('parent_id','=',False)]),
+		'note': fields.text('Notes'),
 	}
 	_defaults = {
 		'quantity': lambda *a: 1
 	}
-
 	def move_line_get(self, cr, uid, invoice_id):
-		type = self.pool.get('account.invoice').read(cr, uid, [invoice_id],['type'])[0]['type']
+		invoice = self.pool.get('account.invoice').browse(cr, uid, invoice_id)
 		res = []
 		tax_grouped = {}
+		tax_obj = self.pool.get('account.tax')
+#TODO: rewrite using browse instead of the manual SQL queries
 		cr.execute('SELECT * FROM account_invoice_line WHERE invoice_id=%d', (invoice_id,))
 		for line in cr.dictfetchall():
 			res.append({'type':'src', 'name':line['name'], 'price_unit':line['price_unit'], 'quantity':line['quantity'], 'price':round(line['quantity']*line['price_unit'],2), 'account_id':line['account_id'], })
@@ -428,25 +471,27 @@ class account_invoice_line(osv.osv):
 			for (tax_id,) in cr.fetchall():
 				# even though we pass only one tax id at a time to compute, it can return several results
 				# in case a tax has a parent tax
-				sequence = self.pool.get('account.tax').read(cr, uid, [tax_id])[0]['sequence']
-				for tax in self.pool.get('account.tax').compute(cr, uid, [tax_id], line['price_unit'],line['quantity']):
+				sequence = tax_obj.read(cr, uid, [tax_id], ['sequence'])[0]['sequence']
+				for tax in tax_obj.compute(cr, uid, [tax_id], line['price_unit'], line['quantity'], invoice.address_invoice_id.id):
 					tax['sequence'] = sequence
-					if type in ('out_invoice','in_refund'):
+					if invoice.type in ('out_invoice','in_refund'):
 						tax['account_id'] = tax['account_collected_id']
 					else:
 						tax['account_id'] = tax['account_paid_id']
 					key = tax['account_id']
 					if not key in tax_grouped:
 						tax_grouped[key] = tax
+						tax_grouped[key]['base'] = line['price_unit'] * line['quantity']
 					else:
 						tax_grouped[key]['amount'] += tax['amount']
+						tax_grouped[key]['base'] += line['price_unit'] * line['quantity']
 		# delete automatic tax lines for this invoice
 		cr.execute("DELETE FROM account_invoice_tax WHERE NOT manual AND invoice_id=%d", (invoice_id,))
 		
 		# (re)create them
 		ait = self.pool.get('account.invoice.tax')
 		for t in tax_grouped.values():
-			ait.create(cr, uid, {'invoice_id':invoice_id, 'name':t['name'], 'account_id':t['account_id'], 'amount':t['amount'], 'manual':False, 'sequence':t['sequence']})
+			ait.create(cr, uid, {'invoice_id':invoice_id, 'name':t['name'], 'account_id':t['account_id'], 'amount':t['amount'], 'manual':False, 'sequence':t['sequence'], 'base': t['base']})
 		return res
 account_invoice_line()
 
@@ -456,8 +501,9 @@ class account_invoice_tax(osv.osv):
 	_columns = {
 		'invoice_id': fields.many2one('account.invoice', 'Invoice Line', ondelete='cascade'),
 		'name': fields.char('Tax Description', size=64, required=True),
-		'account_id': fields.many2one('account.account', 'Taxe Account', required=True, domain=[('type','<>','view'),('type','<>','income')]),
+		'account_id': fields.many2one('account.account', 'Tax Account', required=True, domain=[('type','<>','view'),('type','<>','income')]),
 		'amount': fields.float('Amount', digits=(16,2)),
+		'base': fields.float('Base', digits=(16,4)),
 		'manual': fields.boolean('Manual'),
 		'sequence': fields.integer('Sequence')
 	}
@@ -472,3 +518,5 @@ class account_invoice_tax(osv.osv):
 			res.append({'type':'tax','name':t['name'],'price_unit': t['amount'],'quantity': 1,'price': t['amount'] or 0.0,'account_id': t['account_id'], })
 		return res
 account_invoice_tax()
+
+
