@@ -35,12 +35,13 @@ class res_partner(osv.osv):
 	_name = 'res.partner'
 	_inherit = 'res.partner'
 	_description = 'Partner'
-	def _credit_get(self, cr, uid, ids, prop, unknow_none, unknow_dict):
+	def _credit_get(self, cr, uid, ids, prop, unknow_none, context):
 		res={}
 		try:
-			for id in ids:
-				acc = ir.ir_get(cr, uid, 'meta', 'account.receivable', [('res.partner',id)])[0][2]
-				cr.execute('select sum(debit-credit) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null', (acc, id))
+			for partner in self.browse(cr, uid, ids, context):
+				id = partner.id
+				acc = partner.property_account_receivable[0]
+				cr.execute("select sum(debit-credit) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null and state<>'draft'", (acc, id))
 				res[id]=cr.fetchone()[0] or 0.0
 		except:
 			for id in ids:
@@ -50,9 +51,10 @@ class res_partner(osv.osv):
 	def _debit_get(self, cr, uid, ids, prop, unknow_none, unknow_dict):
 		res={}
 		try:
-			for id in ids:
-				acc = ir.ir_get(cr, uid, 'meta', 'account.payable', [('res.partner',id)])[0][2]
-				cr.execute('select sum(debit-credit) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null', (acc, id))
+			for partner in self.browse(cr, uid, ids, context):
+				id = partner.id
+				acc = partner.property_account_payable[0]
+				cr.execute("select sum(debit-credit) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null and state<>'draft'", (acc, id))
 				res[id]=cr.fetchone()[0] or 0.0
 		except:
 			for id in ids:
@@ -63,7 +65,7 @@ class res_partner(osv.osv):
 		if not len(args):
 			return []
 		where = ' and '.join(map(lambda x: '(sum(debit-credit)'+x[1]+str(x[2])+')',args))
-		cr.execute('select partner_id from account_move_line where account_id in (select id from account_account where type=%s) and reconcile_id is null group by partner_id having '+where, ('receivable',) )
+		cr.execute('select partner_id from account_move_line where account_id in (select id from account_account where type=%s) and reconcile_id is null and state<>\'draft\' group by partner_id having '+where, ('receivable',) )
 		res = cr.fetchall()
 		if not len(res):
 			return [('id','=','0')]
@@ -73,7 +75,7 @@ class res_partner(osv.osv):
 		if not len(args):
 			return []
 		where = ' and '.join(map(lambda x: '(sum(debit-credit)'+x[1]+str(x[2])+')',args))
-		cr.execute('select partner_id from account_move_line where account_id in (select id from account_account where type=%s) and reconcile_id is null group by partner_id having '+where, ('payable',) )
+		cr.execute('select partner_id from account_move_line where account_id in (select id from account_account where type=%s) and reconcile_id is null and state<>\'draft\' group by partner_id having '+where, ('payable',) )
 		res = cr.fetchall()
 		if not len(res):
 			return [('id','=','0')]
@@ -84,7 +86,33 @@ class res_partner(osv.osv):
 		'debit': fields.function(_debit_get, fnct_search=_debit_search, method=True, string='Payable'),
 		'debit_limit': fields.float('Debit Limit'),
 		'credit_limit': fields.float('Credit Limit'),
-		'payment_term': fields.many2one('account.payment.term', 'Payment Term'),
+		'property_account_payable': fields.property(
+		  'account.account',
+		  type='many2one', 
+		  relation='account.account', 
+		  string="Account Payable", 
+		  method=True,
+		  view_load=True,
+		  group_name="Accounting Properties",
+		  help="This account will be used, instead of the default one, as the payable account for the current partner"),
+		'property_account_receivable': fields.property(
+		  'account.account',
+		  type='many2one', 
+		  relation='account.account', 
+		  string="Account Receivable", 
+		  method=True,
+		  view_load=True,
+		  group_name="Accounting Properties",
+		  help="This account will be used, instead of the default one, as the receivable account for the current partner"),
+		'property_payment_term': fields.property(
+		  'account.payment.term',
+		  type='many2one', 
+		  relation='account.payment.term', 
+		  string ='Payment Term',
+		  method=True,
+		  view_load=True,
+		  group_name="Accounting Properties",
+		  help="This payment term will be used, instead of the default one, for the current partner"),
 	}
 res_partner()
 
