@@ -46,6 +46,18 @@ class account_analytic_account(osv.osv):
 			node_balance = reduce(operator.add, [line.amount for line in account.line_ids], 0)
 			child_balance = reduce(operator.add, [child.balance for child in account.child_ids], 0)
 			res[account.id] = node_balance + child_balance
+		for id in ids:
+			res[id] = round(res.get(id, 0.0),2)
+		return res
+
+	def _quantity_calc(self, cr, uid, ids, name, arg, context={}):
+		res = {}
+		for account in self.browse(cr, uid, ids):
+			node_balance = reduce(operator.add, [line.unit_amount for line in account.line_ids], 0)
+			child_balance = reduce(operator.add, [child.quantity for child in account.child_ids], 0)
+			res[account.id] = node_balance + child_balance
+		for id in ids:
+			res[id] = round(res.get(id, 0.0),2)
 		return res
 
 	def name_get(self, cr, uid, ids, context={}):
@@ -70,24 +82,40 @@ class account_analytic_account(osv.osv):
 		'complete_name': fields.function(_complete_name_calc, method=True, type='char', string='Account Name'),
 		'code' : fields.char('Account code', size=8),
 		'active' : fields.boolean('Active'),
+		'type': fields.selection([('view','View'), ('normal','Normal')], 'type'),
 		'description' : fields.text('Description'),
 		'parent_id': fields.many2one('account.analytic.account', 'Parent Cost account'),
 		'child_ids': fields.one2many('account.analytic.account', 'parent_id', 'Childs Accounts'),
-		'line_ids': fields.one2many('account.analytic.line', 'account_id', 'Analytic moves'),
+		'line_ids': fields.one2many('account.analytic.line', 'account_id', 'Analytic entries'),
 		'balance' : fields.function(_balance_calc, method=True, type='float', string='Balance'),
+		'quantity': fields.function(_quantity_calc, method=True, type='float', string='Quantity'),
+		'quantity_max': fields.float('Maximal quantity'),
 		'partner_id' : fields.many2one('res.partner', 'Associated partner'),
 		'contact_id' : fields.many2one('res.partner.address', 'Contact'),
+		'date_start': fields.date('Date Start'),
+		'date': fields.date('Date End'),
 	}
 
 	_defaults = {
 		'active' : lambda *a : True,
+		'type' : lambda *a : 'normal',
 	}
 
 	_order = 'parent_id desc,code'
 
+	def create(self, cr, uid, vals, ctx={}):
+		parent_id = vals.get('parent_id', 0)
+		if ('code' not in vals or not vals['code']) and not parent_id:
+			vals['code'] = self.pool.get('ir.sequence').get(cr, uid, 'account.analytic.account')
+		elif parent_id:
+			parent = self.read(cr, uid, [parent_id], ['parent_id'])[0]
+			vals['code'] = '%03d' % (len(self.search(cr, uid, [('parent_id', '=', parent_id)])) + 1,)
+		return super(account_analytic_account, self).create(cr, uid, vals, ctx)
+
+
 	def on_change_parent(self, cr, uid, id, parent_id):
 		if not parent_id:
-			return {}
+			return {'value': {'code': False}}
 		parent = self.read(cr, uid, [parent_id], ['partner_id'])[0]
 		numchild = len(self.search(cr, uid, [('parent_id', '=', parent_id)]))
 		if parent['partner_id']:
@@ -96,29 +124,17 @@ class account_analytic_account(osv.osv):
 			partner = False
 		return {'value' : {'code' : '%03d' % (numchild + 1,), 'partner_id' : partner}}
 
-	def name_get2(self, cr, uid, ids, context={}):
-		if not len(ids):
-			return []
-		res = []
-		for account in self.browse(cr, uid, ids, context):
-			if account.parent_id:
-				name = '[%s.%s] %s' % (account.parent_id.code, account.code, account.name)
-			else:
-				name = '[%s] %s' % (account.code, account.name)
-			res.append((account.id, name))
-		return res
-
 	def name_search(self, cr, uid, name, args=[], operator='ilike', context={}):
 		codes = name.split('.')
 		codes.reverse()
 		parent_code = False
 		while codes:
 			current_code = codes.pop()
-			account = self.search(cr, uid, [('parent_id', '=', parent_code), ('code', '=', current_code)])
+			account = self.search(cr, uid, [('parent_id', '=', parent_code), ('code', '=', current_code)]+args)
 			if account:
 				parent_code = account[0]
 			else:
-				account = self.search(cr, uid, [('name', 'ilike', '%%%s%%' % name)])
+				account = self.search(cr, uid, [('name', 'ilike', '%%%s%%' % name)]+args)
 				break
 		return self.name_get(cr, uid, account)
 
@@ -131,7 +147,7 @@ class account_analytic_journal(osv.osv):
 		'name' : fields.char('Journal name', size=64, required=True),
 		'code' : fields.char('Journal code', size=8),
 		'active' : fields.boolean('Active'),
-		'type': fields.selection([('sale','Sale'), ('purchase','Purchase'), ('cash','Cash'), ('general','General'), ('situation','Situation')], 'Type', size=32, required=True),
+		'type': fields.selection([('sale','Sale'), ('purchase','Purchase'), ('cash','Cash'), ('general','General'), ('situation','Situation')], 'Type', size=32, required=True, help="Gives the type of the analytic journal. When a document (eg: an invoice) needs to create analytic entries, Tiny ERP will look for a matching journal of the same type."),
 		'line_ids' : fields.one2many('account.analytic.line', 'journal_id', 'Lines'),
 	}
 	_defaults = {
@@ -146,7 +162,7 @@ class account_analytic_unittype(osv.osv):
 		'name': fields.char('Unit type', size=64, required=True),
 		'rate': fields.float('Amount', required=True),
 		'factor': fields.float('Factor', required=True),
-		'line_id': fields.one2many('account.analytic.line', 'unit_id', 'Analytic moves'),
+		'line_id': fields.one2many('account.analytic.line', 'unit_id', 'Analytic entries'),
 		'general_account_id' : fields.many2one('account.account', 'General account', required=True),
 		'journal_id' : fields.many2one('account.analytic.journal', 'Analytic journal', required=True),
 	}
@@ -163,10 +179,10 @@ class account_analytic_line(osv.osv):
 		'amount' : fields.float('Amount', required=True),
 		'unit_amount' : fields.float('Quantity'),
 		'unit_id' : fields.many2one('account.analytic.unittype', 'Unit type'),
-		'account_id' : fields.many2one('account.analytic.account', 'Analytic Account', required=True),
-		'general_account_id' : fields.many2one('account.account', 'General account', required=True),
-		'move_id' : fields.many2one('account.move.line', 'General move'),
-		'journal_id' : fields.many2one('account.analytic.journal', 'Analytic journal', required=True),
+		'account_id' : fields.many2one('account.analytic.account', 'Analytic Account', required=True, ondelete='cascade'),
+		'general_account_id' : fields.many2one('account.account', 'General account', required=True, ondelete='cascade'),
+		'move_id' : fields.many2one('account.move.line', 'General entry', ondelete='cascade'),
+		'journal_id' : fields.many2one('account.analytic.journal', 'Analytic journal', required=True, ondelete='cascade'),
 		'code' : fields.char('Code', size=8),
 	}
 		
