@@ -59,6 +59,7 @@ class account_balance(report_sxw.rml_parse):
 			'SELECT DISTINCT account_id ' \
 			'FROM account_move_line ' \
 			'WHERE date>=%s AND date<=%s ' \
+			'AND state<>\'draft\' ' \
 			'AND account_id IN (' + ','.join(map(str,newids)) + ')',
 			(data['form']['date1'], data['form']['date2']))
 		newids = [id for (id,) in self.cr.fetchall()]
@@ -74,6 +75,7 @@ class account_balance(report_sxw.rml_parse):
 			'from account_move_line l left join account_account a on (l.account_id=a.id) ' \
 			'where account_id in (' + ','.join(map(str, self.ids)) + ') ' \
 			'and date>=%s and date<=%s ' \
+			'AND l.state<>\'draft\' ' \
 			'group by a.id, a.code, a.name order by a.code',
 			(self.datas['form']['date1'], self.datas['form']['date2']))
 		res = self.cr.dictfetchall()
@@ -86,14 +88,14 @@ class account_balance(report_sxw.rml_parse):
 		if not self.ids:
 			return 0.0
 
-		self.cr.execute('select sum(debit) from account_move_line where account_id in (' + ','.join(map(str, self.ids)) + ') and date>=%s and date<=%s', (self.datas['form']['date1'], self.datas['form']['date2']))
+		self.cr.execute('select sum(debit) from account_move_line where account_id in (' + ','.join(map(str, self.ids)) + ') and date>=%s and date<=%s and state<>\'draft\'', (self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_credit(self):
 		if not self.ids:
 			return 0.0
 
-		self.cr.execute('select sum(credit) from account_move_line where account_id in (' + ','.join(map(str, self.ids)) + ') and date>=%s and date<=%s', (self.datas['form']['date1'], self.datas['form']['date2']))
+		self.cr.execute('select sum(credit) from account_move_line where account_id in (' + ','.join(map(str, self.ids)) + ') and date>=%s and date<=%s and state<>\'draft\'', (self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 	
 	def _sum_sdebit(self):
