@@ -31,20 +31,20 @@ import time
 from report import report_sxw
 
 #
-# Use period and Journal for selection or ressources
+# Use period and Journal for selection or resources
 #
 class journal_print(report_sxw.rml_parse):
 	def lines(self, period_id, journal_id, *args):
 		self.cr.execute('update account_journal_period set state=%s where journal_id=%d and period_id=%d and state=%s', ('printed',journal_id,period_id,'draft'))
 		self.cr.commit()
-		self.cr.execute('select id from account_move_line where period_id=%d and journal_id=%d order by date,id', (period_id, journal_id))
+		self.cr.execute('select id from account_move_line where period_id=%d and journal_id=%d and state<>\'draft\' order by date,id', (period_id, journal_id))
 		ids = map(lambda x: x[0], self.cr.fetchall())
 		return self.pool.get('account.move.line').browse(self.cr, self.uid, ids)
 	def _sum_debit(self, period_id, journal_id):
-		self.cr.execute('select sum(debit) from account_move_line where period_id=%d and journal_id=%d', (period_id, journal_id))
+		self.cr.execute('select sum(debit) from account_move_line where period_id=%d and journal_id=%d and state<>\'draft\'', (period_id, journal_id))
 		return self.cr.fetchone()[0] or 0.0
 	def _sum_credit(self, period_id, journal_id):
-		self.cr.execute('select sum(credit) from account_move_line where period_id=%d and journal_id=%d', (period_id, journal_id))
+		self.cr.execute('select sum(credit) from account_move_line where period_id=%d and journal_id=%d and state<>\'draft\'', (period_id, journal_id))
 		return self.cr.fetchone()[0] or 0.0
 	def __init__(self, name, table, rml):
 		super(journal_print, self).__init__(name, table, rml)
