@@ -1,7 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
-#                    Fabien Pinckaers <fp@tiny.Be>
+# Copyright (c) 2005-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -64,7 +63,7 @@ class budget_report(report_sxw.rml_parse):
 	def lines(self, post_obj, date1, date2):
 		res = []
 		for a in post_obj.account_ids:
-	 		self.cr.execute("SELECT COALESCE(SUM(debit-credit), 0) FROM account_move_line WHERE account_id=%d AND date>=%s AND date<=%s", (a.id, date1, date2))
+	 		self.cr.execute("SELECT COALESCE(SUM(debit-credit), 0) FROM account_move_line WHERE account_id=%d AND date>=%s AND date<=%s and state<>'draft'", (a.id, date1, date2))
 			achievements = float(self.cr.fetchone()[0]) * (post_obj.sens=='produit' and -1 or 1)
 			res.append({'name': a.name, 'code': a.code, 'achievements': achievements})
 		return res
