@@ -31,7 +31,7 @@ import time
 from report import report_sxw
 
 #
-# Use period and Journal for selection or ressources
+# Use period and Journal for selection or resources
 #
 class journal_print(report_sxw.rml_parse):
 	def __init__(self, name, table, rml):
@@ -44,15 +44,15 @@ class journal_print(report_sxw.rml_parse):
 		})
 		
 	def lines(self, period_id, journal_id, *args):
-		self.cr.execute('select a.code, a.name, sum(debit) as debit, sum(credit) as credit from account_move_line l left join account_account a on (l.account_id=a.id) where l.period_id=%d and l.journal_id=%d group by a.id, a.code, a.name', (period_id, journal_id))
+		self.cr.execute('select a.code, a.name, sum(debit) as debit, sum(credit) as credit from account_move_line l left join account_account a on (l.account_id=a.id) where l.period_id=%d and l.journal_id=%d and l.state<>\'draft\' group by a.id, a.code, a.name', (period_id, journal_id))
 		return self.cr.dictfetchall()
 		
 	def _sum_debit(self, period_id, journal_id):
-		self.cr.execute('select sum(debit) from account_move_line where period_id=%d and journal_id=%d', (period_id, journal_id))
+		self.cr.execute('select sum(debit) from account_move_line where period_id=%d and journal_id=%d and state<>\'draft\'', (period_id, journal_id))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_credit(self, period_id, journal_id):
-		self.cr.execute('select sum(credit) from account_move_line where period_id=%d and journal_id=%d', (period_id, journal_id))
+		self.cr.execute('select sum(credit) from account_move_line where period_id=%d and journal_id=%d and state<>\'draft\'', (period_id, journal_id))
 		return self.cr.fetchone()[0] or 0.0
 report_sxw.report_sxw('report.account.central.journal', 'account.journal.period', 'addons/account/report/central_journal.rml',parser=journal_print)
 
