@@ -31,7 +31,7 @@ import time
 from report import report_sxw
 
 #
-# Use period and Journal for selection or ressources
+# Use period and Journal for selection or resources
 #
 class journal_print(report_sxw.rml_parse):
 	def __init__(self, name, table, rml):
@@ -68,31 +68,31 @@ class journal_print(report_sxw.rml_parse):
 	def lines(self, period_id):
 		if not self.journal_ids:
 			return []
-		self.cr.execute('select j.code, j.name, sum(l.debit) as debit, sum(l.credit) as credit from account_move_line l left join account_journal j on (l.journal_id=j.id) where period_id=%d and journal_id in (' + self.journal_ids + ') group by j.id, j.code, j.name', (period_id,))
+		self.cr.execute('select j.code, j.name, sum(l.debit) as debit, sum(l.credit) as credit from account_move_line l left join account_journal j on (l.journal_id=j.id) where period_id=%d and journal_id in (' + self.journal_ids + ') and l.state<>\'draft\' group by j.id, j.code, j.name', (period_id,))
 		return self.cr.dictfetchall()
 		
 	def _sum_debit_period(self, period_id):
 		if not self.journal_ids:
 			return 0.0
-		self.cr.execute('select sum(debit) from account_move_line where period_id=%d and journal_id in (' + self.journal_ids + ')', (period_id,))
+		self.cr.execute('select sum(debit) from account_move_line where period_id=%d and journal_id in (' + self.journal_ids + ') and state<>\'draft\'', (period_id,))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_credit_period(self, period_id):
 		if not self.journal_ids:
 			return 0.0
-		self.cr.execute('select sum(credit) from account_move_line where period_id=%d and journal_id in (' + self.journal_ids + ')', (period_id,))
+		self.cr.execute('select sum(credit) from account_move_line where period_id=%d and journal_id in (' + self.journal_ids + ') and state<>\'draft\'', (period_id,))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_debit(self):
 		if not self.journal_ids or not self.period_ids:
 			return 0.0
-		self.cr.execute('select sum(debit) from account_move_line where period_id in (' + self.period_ids + ') and journal_id in (' + self.journal_ids + ')')
+		self.cr.execute('select sum(debit) from account_move_line where period_id in (' + self.period_ids + ') and journal_id in (' + self.journal_ids + ') and state<>\'draft\'')
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_credit(self):
 		if not self.journal_ids or not self.period_ids:
 			return 0.0
-		self.cr.execute('select sum(credit) from account_move_line where period_id in (' + self.period_ids + ') and journal_id in (' + self.journal_ids + ')')
+		self.cr.execute('select sum(credit) from account_move_line where period_id in (' + self.period_ids + ') and journal_id in (' + self.journal_ids + ') and state<>\'draft\'')
 		return self.cr.fetchone()[0] or 0.0
 report_sxw.report_sxw('report.account.general.journal', 'account.journal.period', 'addons/account/report/general_journal.rml',parser=journal_print)
 
