@@ -56,10 +56,11 @@ class grand_livre(report_sxw.rml_parse):
 		newids = _rec_account_get(self.cr, self.uid, toprocess)
 		# filter out accounts which have no transaction in them
 		self.cr.execute(
-			'SELECT DISTINCT account_id ' \
-			'FROM account_move_line ' \
-			'WHERE date>=%s AND date<=%s ' \
-			'AND account_id IN (' + ','.join(map(str,newids)) + ')',
+			"SELECT DISTINCT account_id " \
+			"FROM account_move_line l " \
+			"WHERE date>=%s AND date<=%s " \
+			"AND l.state<>'draft' " \
+			"AND account_id IN (" + ','.join(map(str,newids)) + ")",
 			(data['form']['date1'], data['form']['date2']))
 		newids = [id for (id,) in self.cr.fetchall()]
 		objects = self.pool.get('account.account').browse(self.cr, self.uid, newids)
@@ -67,10 +68,10 @@ class grand_livre(report_sxw.rml_parse):
 
 	def lines(self, account):
 		self.cr.execute(
-			'select l.date, j.code, l.ref, l.name, l.debit, l.credit ' \
-			'from account_move_line l left join account_journal j on (l.journal_id=j.id) ' \
-			'where account_id=%d and date>=%s and date<=%s ' \
-			'order by l.id',
+			"select l.date, j.code, l.ref, l.name, l.debit, l.credit " \
+			"from account_move_line l left join account_journal j on (l.journal_id=j.id) " \
+			"where account_id=%d and date>=%s and date<=%s and (l.state<>'draft') " \
+			"order by l.id",
 			(account.id, self.datas['form']['date1'], self.datas['form']['date2']))
 		res = self.cr.dictfetchall()
 		sum = 0.0
@@ -80,23 +81,23 @@ class grand_livre(report_sxw.rml_parse):
 		return res
 		
 	def _sum_debit_account(self, account):
-		self.cr.execute('select sum(debit) from account_move_line where account_id=%d and date>=%s and date<=%s', (account.id, self.datas['form']['date1'], self.datas['form']['date2']))
+		self.cr.execute("select sum(debit) from account_move_line where account_id=%d and date>=%s and date<=%s and (state<>'draft')", (account.id, self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_credit_account(self, account):
-		self.cr.execute('select sum(credit) from account_move_line where account_id=%d and date>=%s and date<=%s', (account.id, self.datas['form']['date1'], self.datas['form']['date2']))
+		self.cr.execute("select sum(credit) from account_move_line where account_id=%d and date>=%s and date<=%s and (state<>'draft')", (account.id, self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_debit(self):
 		if not self.ids:
 			return 0.0
-		self.cr.execute('select sum(debit) from account_move_line where account_id in (' + ','.join(map(str, self.ids)) + ') and date>=%s and date<=%s', (self.datas['form']['date1'], self.datas['form']['date2']))
+		self.cr.execute("select sum(debit) from account_move_line where account_id in (" + ','.join(map(str, self.ids)) + ") and date>=%s and date<=%s and (state<>'draft')", (self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 		
 	def _sum_credit(self):
 		if not self.ids:
 			return 0.0
-		self.cr.execute('select sum(credit) from account_move_line where account_id in (' +','.join(map(str, self.ids)) + ') and date>=%s and date<=%s', (self.datas['form']['date1'], self.datas['form']['date2']))
+		self.cr.execute("select sum(credit) from account_move_line where account_id in (" +','.join(map(str, self.ids)) + ") and date>=%s and date<=%s and (state<>'draft')", (self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 report_sxw.report_sxw('report.account.grand.livre', 'account.account', 'addons/account/report/grand_livre.rml',parser=grand_livre)
 
