@@ -45,7 +45,7 @@ class grand_livre_tiers(report_sxw.rml_parse):
 		self.cr.execute(
 			"SELECT DISTINCT partner_id " \
 			"FROM account_move_line " \
-			"WHERE partner_id IS NOT NULL AND date>=%s AND date<=%s", 
+			"WHERE partner_id IS NOT NULL AND date>=%s AND date<=%s AND state<>'draft'", 
 			(data['form']['date1'], data['form']['date2']))
 		new_ids = [id for (id,) in self.cr.fetchall()]
 		self.cr.execute(
@@ -63,7 +63,7 @@ class grand_livre_tiers(report_sxw.rml_parse):
 			"FROM account_move_line l LEFT JOIN account_journal j ON (l.journal_id=j.id) " \
 			"WHERE l.partner_id=%d " \
 			"AND l.account_id IN (" + self.account_ids + ") " \
-			"AND l.date>=%s AND l.date<=%s " \
+			"AND l.date>=%s AND l.date<=%s  AND state<>'draft'" \
 			"ORDER BY l.id", 
 			(partner.id, self.datas['form']['date1'], self.datas['form']['date2']))
 		res = self.cr.dictfetchall()
@@ -79,7 +79,7 @@ class grand_livre_tiers(report_sxw.rml_parse):
 			"FROM account_move_line " \
 			"WHERE partner_id=%d " \
 			"AND account_id IN (" + self.account_ids + ") " \
-			"AND date>=%s AND date<=%s",
+			"AND date>=%s AND date<=%s AND state<>'draft'",
 			(partner.id, self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 		
@@ -89,7 +89,7 @@ class grand_livre_tiers(report_sxw.rml_parse):
 			"FROM account_move_line " \
 			"WHERE partner_id=%d " \
 			"AND account_id IN (" + self.account_ids + ") " \
-			"AND date>=%s AND date<=%s",
+			"AND date>=%s AND date<=%s AND state<>'draft'",
 			(partner.id, self.datas["form"]["date1"], self.datas["form"]["date2"]))
 		return self.cr.fetchone()[0] or 0.0
 		
@@ -102,7 +102,7 @@ class grand_livre_tiers(report_sxw.rml_parse):
 			"FROM account_move_line " \
 			"WHERE partner_id IN (" + self.partner_ids + ") " \
 			"AND account_id IN (" + self.account_ids + ") " \
-			"AND date>=%s AND date<=%s", 
+			"AND date>=%s AND date<=%s AND state<>'draft'", 
 			(self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 		
@@ -115,7 +115,7 @@ class grand_livre_tiers(report_sxw.rml_parse):
 			"FROM account_move_line " \
 			"WHERE partner_id IN (" + self.partner_ids + ") " \
 			"AND account_id IN (" + self.account_ids + ") " \
-			"AND date>=%s AND date<=%s", 
+			"AND date>=%s AND date<=%s AND state<>'draft'", 
 			(self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
 report_sxw.report_sxw('report.account.grand.livre.tiers', 'res.partner', 'addons/account/report/grand_livre_tiers.rml',parser=grand_livre_tiers)
