@@ -58,6 +58,7 @@ class partner_balance(report_sxw.rml_parse):
 			return []
 
 #TODO: use an SQL CASE to compute sdebit and scredit
+#CHECKME: est-ce que les "enlitige", il fautdrait pas tester l etat de la ligne aussi?
 		self.cr.execute(
 			"SELECT p.ref, p.name, sum(debit) as debit, sum(credit) as credit, " \
 			"(SELECT sum(debit-credit) FROM account_move_line WHERE partner_id=p.id AND date>=%s AND date<=%s AND blocked=TRUE) as enlitige " \
@@ -65,6 +66,7 @@ class partner_balance(report_sxw.rml_parse):
 			"WHERE partner_id IN (" + self.partner_ids + ") " \
 			"AND account_id IN (" + self.account_ids + ") " \
 			"AND l.date>=%s AND l.date<=%s " \
+			"AND l.state<>'draft' " \
 			"GROUP BY p.id, p.ref, p.name " \
 			"ORDER BY p.ref, p.name",
 			(self.datas['form']['date1'], self.datas['form']['date2'], self.datas['form']['date1'], self.datas['form']['date2']))
@@ -83,6 +85,7 @@ class partner_balance(report_sxw.rml_parse):
 			'FROM account_move_line ' \
 			'WHERE partner_id IN (' + self.partner_ids + ') ' \
 			'AND account_id IN (' + self.account_ids + ') ' \
+			"AND state<>'draft' " \
 			'AND date>=%s AND date<=%s', 
 			(self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
@@ -96,6 +99,7 @@ class partner_balance(report_sxw.rml_parse):
 			'FROM account_move_line ' \
 			'WHERE partner_id IN (' + self.partner_ids + ') ' \
 			'AND account_id IN (' + self.account_ids + ') ' \
+			"AND state<>'draft' " \
 			'AND date>=%s AND date<=%s', 
 			(self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
@@ -109,6 +113,7 @@ class partner_balance(report_sxw.rml_parse):
 			'FROM account_move_line ' \
 			'WHERE partner_id IN (' + self.partner_ids + ') ' \
 			'AND account_id IN (' + self.account_ids + ') ' \
+			"AND state<>'draft' " \
 			'AND date>=%s AND date<=%s AND blocked=TRUE', 
 			(self.datas['form']['date1'], self.datas['form']['date2']))
 		return self.cr.fetchone()[0] or 0.0
@@ -125,6 +130,7 @@ class partner_balance(report_sxw.rml_parse):
 				'WHERE partner_id IN (' + self.partner_ids + ') ' \
 				'AND account_id IN (' + self.account_ids + ') ' \
 				'AND date>=%s AND date<=%s ' \
+				"AND state<>'draft' " \
 				'GROUP BY partner_id' \
 			') AS dc (debit,credit) ' \
 			'WHERE debit>credit', 
@@ -143,6 +149,7 @@ class partner_balance(report_sxw.rml_parse):
 				'WHERE partner_id IN (' + self.partner_ids + ') ' \
 				'AND account_id IN (' + self.account_ids + ') ' \
 				'AND date>=%s AND date<=%s ' \
+				"AND state<>'draft' " \
 				'GROUP BY partner_id' \
 			') AS dc (debit,credit) ' \
 			'WHERE credit>debit', 
