@@ -62,16 +62,16 @@ class account_transfer(osv.osv):
 			('refund','Customer Refund'),
 			('sale','Manual Sale'),
 #			('expense reimburse','Remboursement Depense')
-		], 'Transaction Type', required=True, states={'posted':[('readonly',True)]} ),
+		], 'Transfer Type', required=True, states={'posted':[('readonly',True)]} ),
 		'reference': fields.char('Reference',size=64),
 		'account_src_id': fields.many2one('account.account', 'Source Account', required=True, states={'posted':[('readonly',True)]}),
 		'account_dest_id': fields.many2one('account.account', 'Destination Account', required=True, states={'posted':[('readonly',True)]}),
 		'amount': fields.float('Amount', digits=(16,2), required=True, states={'posted':[('readonly',True)]}),
 		'change': fields.float('Amount Changed', digits=(16,2), states={'posted':[('readonly',True)]}, readonly=True),
-		'move_id': fields.many2one('account.move', 'Transactions', readonly=True),
+		'move_id': fields.many2one('account.move', 'Entry', readonly=True),
 		'adjust_amount': fields.float('Adjustement amount', states={'posted':[('readonly',True)]}),
 		'adjust_account_id': fields.many2one('account.account', 'Adjustement Account', states={'posted':[('readonly',True)]}),
-		'invoice_id': fields.many2many('account.invoice','account_transfer_invoice','transfer_id','invoice_id','Invoices', states={'posted':[('readonly',True)]}, relate=True),
+		'invoice_id': fields.many2many('account.invoice','account_transfer_invoice','transfer_id','invoice_id','Invoices', states={'posted':[('readonly',True)]}, relate=True, help="You can select customer or supplier invoice that are related to this payment. This is optionnal but if you specify the invoices, Tiny ERP will automatically reconcile the entries and mark invoices as paid."),
 	}
 	_defaults = {
 		'date': lambda *a: time.strftime('%Y-%m-%d'),
@@ -133,24 +133,24 @@ class account_transfer(osv.osv):
 			value={}
 			
 			if type=='in_payment':
-				a = ir.ir_get(cr,uid,'meta','account.receivable', [('res.partner',partner_id)])[0][2]
+				a = self.pool.get('res.partner').browse(cr, uid, partner_id).property_account_receivable[0]
 				value['account_src_id'] = a
 				value['account_dest_id'] = False
 				
 				# compute the amount this partner owe us (the sum of all move lines which have not been matched)
-				cr.execute("SELECT COALESCE(SUM(debit-credit),0) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null", (a, partner_id))
+				cr.execute("SELECT COALESCE(SUM(debit-credit),0) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null and state<>'draft'", (a, partner_id))
 				value['amount'] = cr.fetchone()[0]
 				
 				d = self._onchange_account_domain(cr,uid,ids,type, value['account_src_id'], value['account_dest_id'])
 				return {'domain': d, 'value': value}
 				
 			elif type=='out_payment':
-				a = ir.ir_get(cr,uid,'meta','account.payable', [('res.partner',partner_id)])[0][2]
+				a = self.pool.get('res.partner').browse(cr, uid, partner_id).property_account_payable[0]
 				value['account_src_id'] = False
 				value['account_dest_id'] = a
 				
 				# compute the amount we owe this partner (the sum of all move lines which have not been matched)
-				cr.execute("SELECT COALESCE(SUM(debit-credit),0) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null", (a,partner_id))
+				cr.execute("SELECT COALESCE(SUM(debit-credit),0) from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null and state<>'draft'", (a,partner_id))
 				value['amount'] = -cr.fetchone()[0]
 
 				# get the new domain
@@ -175,6 +175,7 @@ class account_transfer(osv.osv):
 				'journal_id': pay['journal_id'][0],
 				'period_id': pay['period_id'][0],
 			}
+#CHECKME: why don't these two lines have period_id and journal_id defined?
 			l2 = {
 				'name':name,
 				'credit':pay['amount']<0 and -pay['amount'],
@@ -189,23 +190,23 @@ class account_transfer(osv.osv):
 				'account_id': pay['account_src_id'][0],
 				'partner_id': partner_id
 			}
-			line_id = [(0,0,l1), (0,0,l2)]
+			line_id = [l1, l2]
 
 			# possibly create two more lines if there is an adjustment
-			if pay['adjust_amount']:
+			if False and pay['adjust_amount']:
 				if pay['adjust_account_id']:
 					la = l.copy()
 					la.update({'name': name+' adjustment', 'credit':pay['adjust_amount']<0 and -pay['adjust_amount'], 'debit':pay['adjust_amount']>0 and pay['adjust_amount'], 'account_id': pay['adjust_account_id'][0],})
-					line_id.append((0,0,la))
+					line_id.append(la)
 					la = l.copy()
 					la.update({'name': name+' adjustment', 'debit':pay['adjust_amount']<0 and -pay['adjust_amount'], 'credit':pay['adjust_amount']>0 and pay['adjust_amount'], 'account_id': pay['account_src_id'][0],})
-					line_id.append((0,0,la))
+					line_id.append(la)
 				else:
 					raise Exception('No Adjust Account !', 'missing adjust account')
 					
 			# create the new move and its 2 (or 4) lines
 			move = l.copy()
-			move['line_id'] = line_id
+			move['line_id'] = [(0, 0, l) for l in line_id]
 			move_id = self.pool.get('account.move').create(cr, uid, move)
 			
 			# get account_id depending on the type of transfer
@@ -237,9 +238,9 @@ class account_transfer(osv.osv):
 					# get the id of the move_line for the current transfer
 					cr.execute("SELECT id FROM account_move_line WHERE account_id=%d AND move_id=%d", (account_id,move_id))
 					l_ids.append(cr.fetchall()[0][0])
-					self.pool.get('account.move.line').reconcile(cr, uid, l_ids)
+					self.pool.get('account.move.line').reconcile(cr, uid, l_ids, writeoff_period_id=pay['period_id'][0], writeoff_journal_id=pay['journal_id'][0], writeoff_acc_id=pay['adjust_account_id'] and pay['adjust_account_id'][0])
 				else:
-					raise osv.except_osv('Warning !', 'Could not confirm payment because its amount (%.2f) is different from the selected invoice(s) amount (%.2f) !' % (pay['amount'],s))
+					raise osv.except_osv('Warning !', 'Could not confirm payment because its amount (%.2f) is different from the selected invoice(s) amount (%.2f) !' % (pay['amount']+pay['adjust_amount'],s))
 					
 			elif account_id and partner_id:
 				# compute the sum of all move lines for this account and partner which were not matched
@@ -251,7 +252,7 @@ class account_transfer(osv.osv):
 					cr.execute("select id from account_move_line where account_id=%d and partner_id=%d and reconcile_id is null", (account_id, partner_id))
 					ids2 = [id for (id,) in cr.fetchall()]
 					if len(ids2):
-						self.pool.get('account.move.line').reconcile(cr, uid, ids2)
+						self.pool.get('account.move.line').reconcile(cr, uid, ids2, writeoff_period_id=pay['period_id'][0], writeoff_journal_id=pay['journal_id'][0], writeoff_acc_id=pay['adjust_account_id'] and pay['adjust_account_id'][0])
 
 			# change transfer state and assign it its move
 			self.write(cr, uid, [pay['id']], {'state':'posted', 'move_id': move_id})
