@@ -32,6 +32,7 @@ import wizard_unreconcile_select
 import wizard_reconcile
 import wizard_unreconcile
 import wizard_refund
+import wizard_pay_invoice
 
 import wizard_journal
 import wizard_journal_select
@@ -39,6 +40,7 @@ import wizard_bank_reconcile
 import wizard_budget_spread
 import wizard_subscription_generate
 
+import wizard_aged_trial_balance
 import wizard_budget_report
 import wizard_grand_livre_report
 import wizard_grand_livre_tiers_report
