@@ -44,7 +44,7 @@ def _action_open_window(self, cr, uid, data, context):
 	if not account_id:
 		raise 'You have to define the bank account\nin the journal definition for reconciliation.'
 	return {
-		'domain': "[('journal_id','=',%d), ('account_id','=',%d)]" % (form['journal_id'],account_id),
+		'domain': "[('journal_id','=',%d), ('account_id','=',%d), ('state','<>','draft')]" % (form['journal_id'],account_id),
 		'name': 'Saisie Standard',
 		'view_type': 'form',
 		'view_mode': 'tree,form',
