@@ -28,6 +28,14 @@
 import wizard
 import netsvc
 
+sur_form = '''<?xml version="1.0"?>
+<form string="Refund Invoice">
+	<label string="Are you sure you want to refund this invoice ?"/>
+</form>'''
+
+sur_fields = {
+}
+
 class wiz_refund(wizard.interface):
 	def _invoice_refund(self, cr, uid, data, context):
 		service = netsvc.LocalService("object_proxy")
@@ -36,6 +44,10 @@ class wiz_refund(wizard.interface):
 		
 	states = {
 		'init': {
+			'actions': [],
+			'result': {'type':'form', 'arch':sur_form, 'fields':sur_fields, 'state':[('end','Cancel'),('refund','Refund Invoice')]}
+		},
+		'refund': {
 			'actions': [_invoice_refund],
 			'result': {'type':'state', 'state':'end'}
 		}
