@@ -36,7 +36,7 @@ from osv.osv import osv_pools
 
 _subscription_form = '''<?xml version="1.0"?>
 <form string="%s">
-	<seperator string="Generate moves before:" colspan="4"/>
+	<seperator string="Generate entries before:" colspan="4"/>
 	<field name="date"/>
 </form>''' % ('Subscription Compute',)
 
@@ -54,7 +54,7 @@ class wiz_subscription(wizard.interface):
 	states = {
 		'init': {
 			'actions': [],
-			'result': {'type': 'form', 'arch':_subscription_form, 'fields':_subscription_fields, 'state':[('end','Cancel'),('generate','Compute Move Dates')]}
+			'result': {'type': 'form', 'arch':_subscription_form, 'fields':_subscription_fields, 'state':[('end','Cancel'),('generate','Compute Entry Dates')]}
 		},
 		'generate': {
 			'actions': [_action_generate],
