@@ -1,7 +1,7 @@
 {
 	"name" : "Accounting follow-ups management",
 	"version" : "1.0",
-	"depends" : ["base", "account"],
+	"depends" : ["account"],
 	"author" : "Tiny",
 	"website" : "http://tinyerp.com/module_account.html",
 	"category" : "Generic Modules/Accounting",
