@@ -21,6 +21,7 @@
 		"module/module_data.xml",
 		"module/module_wizard.xml",
 		"module/module_view.xml",
+		"module/module_report.xml",
 		"res/res_request_view.xml",
 		"res/partner/partner_report.xml",
 		"res/partner/partner_view.xml",
@@ -28,5 +29,6 @@
 		"res/res_currency_view.xml",
 		"res/partner/crm_view.xml",
 		"res/partner/partner_data.xml",
+		"res/ir_property_view.xml",
 	],
 }
