@@ -44,6 +44,7 @@ CREATE TABLE ir_model (
   id serial,
   model varchar(64) DEFAULT ''::varchar NOT NULL,
   name varchar(64),
+  info text,
   primary key(id)
 );
 
@@ -55,6 +56,8 @@ CREATE TABLE ir_model_fields (
   relation varchar(64),
   field_description varchar(256),
   ttype varchar(64),
+  group_name varchar(64),
+  view_load boolean,
   relate boolean default False,
   primary key(id)
 );
@@ -161,10 +164,11 @@ CREATE TABLE res_users (
     action_id int,
     primary key(id)
 );
+alter table res_users add constraint res_users_login_uniq unique (login);
+
 insert into res_users (id,login,password,name,action_id,perm_id) values (1,'root',NULL,'Administrator',NULL,1);
 select setval('res_users_id_seq', 2);
 
-
 CREATE TABLE res_groups (
     id serial NOT NULL,
     perm_id int references perm on delete set null,
