@@ -30,6 +30,7 @@ import ir
 import netsvc
 
 import time
+import tools
 import sql_db
 
 class ir_model(osv.osv):
@@ -38,6 +39,7 @@ class ir_model(osv.osv):
 	_columns = {
 		'name': fields.char('Name', size=64, translate=True),
 		'model': fields.char('Model', size=64, required=True),
+		'info': fields.text('Information'),
 		'field_id': fields.one2many('ir.model.fields', 'model_id', 'Fields', required=True),
 	}
 	_defaults = {
@@ -61,12 +63,18 @@ class ir_model_fields(osv.osv):
 		'field_description': fields.char('Field Description', size=256),
 		'ttype': fields.char('Field Type', size=64),
 		'relate': fields.boolean('Click and Relate'),
+
+		'groups': fields.many2many('res.groups', 'ir_model_fields_group_rel', 'field_id', 'group_id', 'Groups'),
+		'group_name': fields.char('Group Name', size=128),
+		'view_load': fields.boolean('View Auto-Load'),
 	}
 	_defaults = {
 		'relate': lambda *a: 1,
+		'view_load': lambda *a: 0,
 		'name': lambda *a: 'No Name',
 		'field_description': lambda *a: 'No description available',
 	}
+	_order = "id"
 ir_model_fields()
 
 class ir_model_access(osv.osv):
@@ -92,14 +100,31 @@ class ir_model_access(osv.osv):
 		# compute the list of groups which have the right we are looking for
 		ids = map(lambda x: str(x[0]), filter(lambda x: x[1], cr.fetchall()))
 		if not ids:
-			raise osv.except_osv('Access denied !', 'You can not %s this ressource !' % mode)
+			raise osv.except_osv('Access denied !', 'You can not %s this resource !' % mode)
 		ids_str = ','.join(ids)
 		
 		# check if the user is part of one of those groups
 		cr.execute('select count(*) from res_groups_users_rel where uid=%d and gid in ('+ids_str+')', (uid,))
 		if cr.fetchone()[0]:
 			return True
-		raise osv.except_osv('Access denied !', 'You can not %s this ressource !' % mode)
+		raise osv.except_osv('Access denied !', 'You can not %s this resource !' % mode)
+	check = tools.cache()(check)
+
+	#
+	# Methods to clean the cache on the Check Method.
+	#
+	def write(self, cr, uid, *args, **argv):
+		res = super(ir_model_access, self).write(cr, uid, *args, **argv)
+		self.check()
+		return res
+	def create(self, cr, uid, *args, **argv):
+		res = super(ir_model_access, self).create(cr, uid, *args, **argv)
+		self.check()
+		return res
+	def unlink(self, cr, uid, *args, **argv):
+		res = super(ir_model_access, self).unlink(cr, uid, *args, **argv)
+		self.check()
+		return res
 ir_model_access()
 
 class ir_model_data(osv.osv):
@@ -108,7 +133,7 @@ class ir_model_data(osv.osv):
 		'name': fields.char('XML Identifier', required=True, size=64),
 		'model': fields.char('Model', required=True, size=64),
 		'module': fields.char('Module', required=True, size=64),
-		'res_id': fields.integer('Ressource ID'),
+		'res_id': fields.integer('Resource ID'),
 		'noupdate': fields.boolean('Non Updatable'),
 		'date_update': fields.datetime('Update Date'),
 		'date_init': fields.datetime('Init Date')
@@ -190,7 +215,6 @@ class ir_model_data(osv.osv):
 		else:
 			res_id=None
 			model = models[0]
-
 		if res_id:
 			where = ' and res_id=%d' % (res_id,)
 		else:
@@ -241,7 +265,7 @@ class ir_model_data(osv.osv):
 					print '... ko'
 					print '-'*60
 					print 'WARNING: Could not delete id', id, 'of model', model
-					print '\tThere should be some relation that points to this ressource'
+					print '\tThere should be some relation that points to this resource'
 					print '\tYou should manually fix this and restart --update=module'
 				cr2.close()
 		return True
