@@ -26,8 +26,9 @@
 #
 ##############################################################################
 
-from osv import fields,osv
+import threading
 import time
+from osv import fields,osv
 
 class ir_sequence_type(osv.osv):
 	_name = 'ir.sequence.type'
@@ -59,29 +60,34 @@ class ir_sequence(osv.osv):
 		'number_next': lambda *a: 1,
 		'padding' : lambda *a : 0,
 	}
+
+	sequence_semaphore = threading.Semaphore()
+
 	def _process(self, s):
 		return (s or '') % {'year':time.strftime('%Y'), 'month': time.strftime('%m'), 'day':time.strftime('%d')}
-	#
-	# TODO: should be thread safe !
-	#
-	def get_id(self, cr, uid, sequence_id):
-		cr.execute('select id,number_next,number_increment,prefix,suffix from ir_sequence where id=%d and active=True', (sequence_id,))
+
+	def get_id(self, cr, uid, sequence_id, test='id=%d'):
+		self.sequence_semaphore.acquire()
+		cr.execute('select id,number_next,number_increment,prefix,suffix,padding from ir_sequence where '+test+' and active=True', (sequence_id,))
 		res = cr.dictfetchone()
 		if res:
-			cr.execute('update ir_sequence set number_next=number_next+number_increment where id=%d and active=True', (sequence_id,))
-			return self._process(res['prefix']) + str(res['number_next']) + self._process(res['suffix'])
+			cr.execute('update ir_sequence set number_next=number_next+number_increment where id=%d and active=True', (res['id'],))
+			self.sequence_semaphore.release()
+			if res['number_next']:
+				return self._process(res['prefix']) + '%%0%sd' % res['padding'] % res['number_next'] + self._process(res['suffix'])
+			else:
+				return self._process(res['prefix']) + self._process(res['suffix'])
+		else:
+			self.sequence_semaphore.release()
 		return False
 
 	def get(self, cr, uid, code):
-		cr.execute('select id,number_next,number_increment,prefix,suffix,padding from ir_sequence where code=%s and active=True', (code,))
-		res = cr.dictfetchone()
-		if res:
-			cr.execute('update ir_sequence set number_next=number_next+number_increment where code=%s and active=True', (code,))
-			return self._process(res['prefix']) + '%%0%sd' % res['padding'] % res['number_next'] + self._process(res['suffix'])
-		return False
+		return self.get_id(cr, uid, code, test='code=%s')
 
 	def set(self, cr, uid, code, next_number):
+		self.sequence_semaphore.acquire()
 		cr.execute('update ir_sequence set number_next=%d where code=%s and active=True', (next_number, code,))
+		self.sequence_semaphore.release()
 		return True
 ir_sequence()
 
