@@ -85,18 +85,20 @@ class ir_ui_menu(osv.osv):
 			res[m.id] = self._get_one_full_name(m)
 		return res
 
-	def _get_one_full_name(self, menu):
+	def _get_one_full_name(self, menu, level=6):
+		if level<=0:
+			return '...'
 		if menu.parent_id:
-			parent_path = self._get_one_full_name(menu.parent_id) + "/"
+			parent_path = self._get_one_full_name(menu.parent_id, level-1) + "/"
 		else:
 			parent_path = ''
 		return parent_path + menu.name
 
-	def copy(self, cr, uid, id, confirm=True):
-		res = super(ir_ui_menu, self).copy(cr, uid, id)
+	def copy(self, cr, uid, id, default=None, context={}):
+		res = super(ir_ui_menu, self).copy(cr, uid, id, context=context)
 		ids = self.pool.get('ir.values').search(cr, uid, [('model','=','ir.ui.menu'),('res_id','=',id)])
 		for iv in self.pool.get('ir.values').browse(cr, uid, ids):
-			new_id = self.pool.get('ir.values').copy(cr, uid, iv.id, default={'res_id':res})
+			new_id = self.pool.get('ir.values').copy(cr, uid, iv.id, default={'res_id':res}, context=context)
 		return res
 
 	_columns = {
@@ -105,7 +107,7 @@ class ir_ui_menu(osv.osv):
 		'child_id' : fields.one2many('ir.ui.menu', 'parent_id','Child ids'),
 		'parent_id': fields.many2one('ir.ui.menu', 'Parent Menu'),
 		'groups_id': fields.many2many('res.groups', 'ir_ui_menu_group_rel', 'menu_id', 'gid', 'Groups'),
-		'complete_name': fields.function(_get_full_name, method=True, string='Complete Name', type='char', size=64),
+		'complete_name': fields.function(_get_full_name, method=True, string='Complete Name', type='char', size=128),
 		'icon': fields.selection(lambda *a: icons, 'Icon', size=64)
 	}
 	_defaults = {
