@@ -39,9 +39,13 @@ class ir_values(osv.osv):
 		'key': fields.char('Name', size=128),
 		'key2': fields.char('Value', size=256),
 		'meta': fields.text('Meta Datas'),
-		'res_id': fields.integer('Ressource ID'),
+		'res_id': fields.integer('Resource ID'),
 		'user_id': fields.many2one('res.users', 'User', ondelete='cascade')
 	}
+	_defaults = {
+		'key': lambda *a: 'action',
+		'key2': lambda *a: 'tree_but_open',
+	}
 	def set(self, cr, uid, key, key2, name, models, value, replace=True, isobject=False, meta=False, preserve_user=False):
 		if not isobject:
 			value = pickle.dumps(value)
@@ -145,3 +149,5 @@ class ir_values(osv.osv):
 		res = filter(bool, map(lambda x: _result_get(x), list(result)))
 		return res
 ir_values()
+
+
