@@ -120,15 +120,25 @@ class report_graph_instance(object):
 			print 'Consider Installing PyDot or dependencies: http://dkbza.org/pydot.html'
 			raise e
 		self.done = False
-		cr = sql_db.db.cursor()
-		cr.execute('select * from wkf where osv=%s limit 1', (data['model'],))
-		wkfinfo = cr.dictfetchone()
-		cr.execute('select id from wkf_instance where res_id=%d and wkf_id=%d order by state limit 1', (data['id'],wkfinfo['id']))
-		inst_id = cr.fetchone()[0]
-		
-		graph = pydot.Dot(fontsize=16, label="\\n\\nWorkflow: %s\\n OSV: %s" % (wkfinfo['name'],wkfinfo['osv']))
-		graph_instance_get(cr, graph, inst_id, data.get('nested', False))
-		ps_string = graph.create_ps(prog='dot')
+
+		try:
+			cr = sql_db.db.cursor()
+			cr.execute('select * from wkf where osv=%s limit 1', (data['model'],))
+			wkfinfo = cr.dictfetchone()
+			cr.execute('select id from wkf_instance where res_id=%d and wkf_id=%d order by state limit 1', (data['id'],wkfinfo['id']))
+			inst_id = cr.fetchone()[0]
+
+			graph = pydot.Dot(fontsize=16, label="\\n\\nWorkflow: %s\\n OSV: %s" % (wkfinfo['name'],wkfinfo['osv']))
+			graph_instance_get(cr, graph, inst_id, data.get('nested', False))
+			ps_string = graph.create_ps(prog='dot')
+		except:
+			# string is in PS, like the success message would have been
+			ps_string = '''%PS-Adobe-3.0
+/inch {72 mul} def
+/Times-Roman findfont 50 scalefont setfont
+1.5 inch 15 inch moveto
+(No workflow available) show
+showpage'''
 		input, output = os.popen2('ps2pdf -sPAPERSIZE=a3 - -')
 		input.write(ps_string)
 		input.close()
