@@ -35,7 +35,7 @@ class workflow(osv.osv):
 	_log_access = False
 	_columns = {
 		'name': fields.char('Name', size=64, required=True),
-		'osv': fields.char('Ressource Model', size=64, required=True),
+		'osv': fields.char('Resource Model', size=64, required=True),
 		'on_create': fields.boolean('On Create'),
 		'activities': fields.one2many('workflow.activity', 'wkf_id', 'Activities'),
 	}
@@ -120,7 +120,7 @@ class wkf_triggers(osv.osv):
 	_name = "workflow.triggers"
 	_log_access = False
 	_columns = {
-		'res_id': fields.integer('Ressource ID', size=128),
+		'res_id': fields.integer('Resource ID', size=128),
 		'model': fields.char('Model', size=128),
 		'instance_id': fields.many2one('workflow.instance', 'Destination Instance', ondelete="cascade"),
 		'workitem_id': fields.many2one('workflow.workitem', 'Workitem', required=True, ondelete="cascade"),
