@@ -35,6 +35,7 @@ from osv import fields,osv
 #
 class res_partner_canal(osv.osv):
 	_name = "res.partner.canal"
+	_description = "Canals"
 	_columns = {
 		'name': fields.char('Canal Name',size=64, required=True),
 		'active': fields.boolean('Active'),
@@ -77,6 +78,7 @@ class res_partner_event(osv.osv):
 		'canal_id': fields.many2one('res.partner.canal', 'Canal'),
 		'partner_type': fields.selection([('customer','Customer'),('retailer','Retailer'),('prospect','Commercial Prospect')], 'Partner Relation'),
 		'type': fields.selection([('sale','Sale Oportunity'),('purchase','Purchase Offer'),('prospect','Prospect Contact')], 'Type of Event'),
+		'event_ical_id': fields.char('iCal id', size=64),
 	}
 	_order = 'date desc'
 	_defaults = {
@@ -87,6 +89,7 @@ res_partner_event()
 
 class res_partner_event_type(osv.osv):
 	_name = "res.partner.event.type"
+	_description = "Partner Events"
 	_columns = {
 		'name': fields.char('Event Type',size=64, required=True),
 		'key': fields.char('Key', size=64, required=True),
