@@ -35,6 +35,7 @@ import ir
 
 class res_partner_function(osv.osv):
 	_name = 'res.partner.function'
+	_description = 'Function of the contact'
 	_columns = {
 		'name': fields.char('Name', size=64, required=True),
 		'code': fields.char('Code', size=8),
@@ -44,9 +45,10 @@ res_partner_function()
 
 class res_country(osv.osv):
 	_name = 'res.country'
+	_description = 'Country'
 	_columns = {
-		'name': fields.char('Country Name', size=64),
-		'code': fields.char('Country Code', size=2),
+		'name': fields.char('Country Name', size=64, help='The full name of the country.'),
+		'code': fields.char('Country Code', size=2, help='The ISO country code in two chars. You can use this field for quick search.'),
 	}
 	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
 		ids = self.search(cr, user, [('code','=',name)]+ args)
@@ -133,9 +135,6 @@ class res_partner_title(osv.osv):
 	_order = 'name'
 res_partner_title()
 
-def is_pair(x):
-	return not x%2
-
 def _contact_title_get(self, cr, uid, context={}):
 	obj = self.pool.get('res.partner.title')
 	ids = obj.search(cr, uid, [('domain', '=', 'contact')])
@@ -157,24 +156,20 @@ def _lang_get(self, cr, uid, context={}):
 
 class res_partner(osv.osv):
 	_description='Partner'
-
 	_name = "res.partner"
 	_order = "name"
 	_columns = {
-		'name': fields.char('Name', size=64, required=True),
+		'name': fields.char('Name', size=128, required=True),
 		'date': fields.date('Date'),
 		'title': fields.selection(_partner_title_get, 'Title', size=32),
 		'parent_id': fields.many2one('res.partner','Main Company'),
-		'sale_order': fields.one2many('sale.order', 'partner_id', 'Sales Order'),
-		'commercial': fields.many2one('res.users','Commercial'),
 		'child_ids': fields.one2many('res.partner', 'parent_id', 'Partner Ref.'),
-		'ref': fields.char('Partner ID', size=64),
+		'ref': fields.char('Code', size=64),
 		'lang': fields.selection(_lang_get, 'Language', size=2),
-		'user_id': fields.many2one('res.users', 'Users'),
+		'user_id': fields.many2one('res.users', 'Salesman'),
 		'responsible': fields.many2one('res.users', 'Users'),
 		'vat': fields.char('VAT',size=32),
-		'bank': fields.char('Bank account',size=64),
-		'iban': fields.char('IBAN/BIC Number', size=34),
+		'bank_ids': fields.one2many('res.partner.bank', 'partner_id', 'Banks'),
 		'website': fields.char('Website',size=64),
 		'comment': fields.text('Notes'),
 		'client_lien': fields.many2one('res.partner', 'Liens'),
@@ -188,6 +183,9 @@ class res_partner(osv.osv):
 	_defaults = {
 		'active': lambda *a: 1,
 	}
+	_sql_constraints = [
+	#	('name_uniq', 'unique (name)', 'The name of the partner must be unique !')
+	]
 	
 	def _check_ean_key(self, cr, uid, ids):
 		for partner_o in osv.osv_pools.get('res.partner').read(cr, uid, ids, ['ean13',]):
@@ -197,7 +195,7 @@ class res_partner(osv.osv):
 					return False
 				sum=0
 				for i in range(12):
-					if is_pair(i):
+					if not (i % 2):
 						sum+=int(thisean[i])
 					else:
 						sum+=3*int(thisean[i])
@@ -207,7 +205,7 @@ class res_partner(osv.osv):
 
 #	_constraints = [(_check_ean_key, 'Error: Invalid ean code', ['ean13'])]
 
-	def name_get(self, cr, user, ids, context={}):
+	def name_get(self, cr, uid, ids, context={}):
 		if not len(ids):
 			return []
 		if context.get('show_ref', False):
@@ -215,21 +213,24 @@ class res_partner(osv.osv):
 		else:
 			rec_name = 'name'
 			
-		res = [(r['id'], r[rec_name]) for r in self.read(cr, user, ids, [rec_name], context)]
+		res = [(r['id'], r[rec_name]) for r in self.read(cr, uid, ids, [rec_name], context)]
 		return res
 		
-	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
-		ids = self.search(cr, user, [('ref','=',name)]+ args)
-		if not ids:
-			ids = self.search(cr, user, [('name',operator,name)]+ args)
-		return self.name_get(cr, user, ids, context)
+	def name_search(self, cr, uid, name, args=[], operator='ilike', context={}, limit=80):
+		if name:
+			ids = self.search(cr, uid, [('ref', '=', name)] + args, limit=limit)
+			if not ids:
+				ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit)
+		else:
+			ids = self.search(cr, uid, args, limit=limit)
+		return self.name_get(cr, uid, ids, context)
 
 	def _email_send(self, cr, uid, ids, email_from, subject, body, on_error=None):
 		partners = self.browse(cr, uid, ids)
 		for partner in partners:
 			if len(partner.address):
 				if partner.address[0].email:
-					tools.email_send(email_from, partner.address[0].email, subject, body, on_error)
+					tools.email_send(email_from, [partner.address[0].email], subject, body, on_error)
 		return True
 
 	def email_send(self, cr, uid, ids, email_from, subject, body, on_error=''):
@@ -276,22 +277,42 @@ class res_partner(osv.osv):
 		# update the current partner
 		cr.execute("update res_partner set ref=%d where id=%d", (nextref, ids[0]))
 		return True
-	
 res_partner()
 
+class res_partner_bank(osv.osv):
+	_description='Bank Details'
+	_name = "res.partner.bank"
+	_order = "sequence"
+	_columns = {
+		'name': fields.char('Account Name', size=64, required=True),
+		'sequence': fields.integer('Sequence'),
+		'iban': fields.char('IBAN', size=64),
+		'swift': fields.char('Swift', size=64),
+		'bank_name': fields.char('Bank Name', size=64),
+		'bank_guichet': fields.char('Branch', size=64),
+		'partner_id': fields.many2one('res.partner', 'Partner', required=True, ondelete='cascade'),
+		'active': fields.boolean('Active'),
+	}
+	_defaults = {
+		'active': lambda *a: 1,
+	}
+res_partner_bank()
+
+
 class res_partner_address(osv.osv):
 	_description ='Partner Contact'
 	_name = 'res.partner.address'
+	_order = 'id'
 	_columns = {
 		'partner_id': fields.many2one('res.partner', 'Partner', required=True, ondelete='cascade'),
 		'type': fields.selection( [ ('default','Default'),('invoice','Invoice'), ('delivery','Delivery'), ('contact','Contact'), ('other','Other') ],'Address Type'),
 		'function': fields.many2one('res.partner.function', 'Function', relate=True),
 		'title': fields.selection(_contact_title_get, 'Title', size=32),
 		'name': fields.char('Contact Name', size=64),
-		'street': fields.char('Street', size=64),
-		'street2': fields.char('Street2', size=64),
+		'street': fields.char('Street', size=128),
+		'street2': fields.char('Street2', size=128),
 		'zip': fields.char('Zip', change_default=True, size=24),
-		'city': fields.char('City', size=64),
+		'city': fields.char('City', size=128),
 		'state_id': fields.many2one("res.country.state", 'State', domain="[('country_id','=',country_id)]"),
 		'country_id': fields.many2one('res.country', 'Country'),
 		'email': fields.char('E-Mail', size=64),
@@ -299,6 +320,7 @@ class res_partner_address(osv.osv):
 		'fax': fields.char('Fax', size=64),
 		'mobile': fields.char('Mobile', size=64),
 		'birthdate': fields.char('Birthdate', size=64),
+		'active': fields.boolean('Active'),
 	}
 	_defaults = {
 		'active': lambda *a: 1,
@@ -317,12 +339,12 @@ class res_partner_address(osv.osv):
 			res.append((r['id'], addr))
 		return res
 
-	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
-		ids = self.search(cr, user, [('zip','=',name)] + args)
+	def name_search(self, cr, user, name, args=[], operator='ilike', context={}, limit=80):
+		ids = self.search(cr, user, [('zip','=',name)] + args, limit=limit)
 		if not ids: 
-			ids = self.search(cr, user, [('city',operator,name)] + args)
+			ids = self.search(cr, user, [('city',operator,name)] + args, limit=limit)
 		if not ids:
-			ids = self.search(cr, user, [('name',operator,name)] + args)
+			ids = self.search(cr, user, [('name',operator,name)] + args, limit=limit)
 		return self.name_get(cr, user, ids)
 res_partner_address()
 
