@@ -29,6 +29,7 @@
 
 import wizard
 from osv.osv import osv_pools
+import tools
 
 email_send_form = '''<?xml version="1.0"?>
 <form string="Mass Mailing">
@@ -45,20 +46,19 @@ email_send_fields = {
 	'text': {'string':'Message', 'type':'text', 'required':True}
 }
 
+# this sends an email to ALL the addresses of the selected partners.
 def _mass_mail_send(self, cr, uid, data, context):
 	nbr = 0
 	partners = osv_pools.get('res.partner').browse(cr, uid, data['ids'], context)
 	for partner in partners:
 		for adr in partner.address:
 			if adr.email:
-				try:
-					name = adr.name or partner.name
-					to = '%s <%s>' % (name, adr.email)
+				name = adr.name or partner.name
+				to = '%s <%s>' % (name, adr.email)
 #TODO: add some tests to check for invalid email addresses
-					tools.sendmail(data['form']['from'], to, data['form']['subject'], data['form']['text'])
-					nbr += 1
-				except:
-					pass
+#CHECKME: maybe we should use res.partner/email_send
+				tools.email_send(data['form']['from'], [to], data['form']['subject'], data['form']['text'])
+				nbr += 1
 #TODO: log number of message sent
 	return {'email_sent': nbr}
 
