@@ -42,7 +42,8 @@ class res_request(osv.osv):
 			cr.execute('update res_request set state=%s,date_sent=%s where id=%d', ('waiting', time.strftime('%Y-%m-%d %H:%M:%S'), id))
 			cr.execute('select act_from,act_to,body,date_sent from res_request where id=%d', (id,))
 			values = cr.dictfetchone()
-			values['name'] = (values['body'] or '/')[:40]
+			# this will be truncated automatically at creation
+			values['name'] = values['body'] or '/'
 			values['req_id'] = id
 			self.pool.get('res.request.history').create(cr, uid, values)
 		return True
@@ -106,7 +107,7 @@ class res_request_history(osv.osv):
 	_name = 'res.request.history'
 	_columns = {
 		'name': fields.char('Summary', size=128, states={'active':[('readonly',True)],'waiting':[('readonly',True)]}, required=True),
-		'req_id': fields.many2one('res.request', 'Request', required=True),
+		'req_id': fields.many2one('res.request', 'Request', required=True, ondelete='cascade'),
 		'act_from': fields.many2one('res.users', 'From', required=True, readonly=True),
 		'act_to': fields.many2one('res.users', 'To', required=True, states={'waiting':[('readonly',True)]}),
 		'body': fields.text('Body', states={'waiting':[('readonly',True)]}),
