@@ -27,6 +27,7 @@
 ##############################################################################
 
 from osv import fields,osv
+import tools
 
 class groups(osv.osv):
 	_name = "res.groups"
@@ -83,15 +84,26 @@ class users(osv.osv):
 		'roles_id': fields.many2many('res.roles', 'res_roles_users_rel', 'uid', 'rid', 'Roles'),
 		'company_id': fields.many2one('res.company', 'Company'),
 	}
-# this is useless since _sql clauses are only executed at the table creation and
-# user table is already created in base.sql
-#	_sql = 'ALTER TABLE res_users ADD CONSTRAINT res_users_login_uniq UNIQUE (login)'
+	_sql_constraints = [
+		('login_key', 'UNIQUE (login)', 'You can not have two users with the same login !')
+	]
 	_defaults = {
 		'password' : lambda obj,cr,uid,context={} : '',
 	}
+	def company_get(self, cr, uid, uid2):
+		company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id.id
+		return company_id
+	company_get = tools.cache()(company_get)
+
+	def write(self, cr, uid, *args, **argv):
+		res = super(users, self).write(cr, uid, *args, **argv)
+		# Restart the cache on the company_get method
+		self.company_get()
+		return res
+
 	def unlink(self, cr, uid, ids):
 		if 1 in ids:
-			raise osv.except_osv('Can not remove root user !', 'You can not remove the root user as it is used internally for ressources created by Tiny ERP (updates, module installation, ...)')
+			raise osv.except_osv('Can not remove root user !', 'You can not remove the root user as it is used internally for resources created by Tiny ERP (updates, module installation, ...)')
 		return super(users, self).unlink(cr, uid, ids)
 		
 	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
@@ -99,5 +111,10 @@ class users(osv.osv):
 		if not ids:
 			ids = self.search(cr, user, [('name',operator,name)]+ args)
 		return self.name_get(cr, user, ids)
+		
+	def copy(self, cr, uid, id, default=None, context={}):
+		login = self.read(cr, uid, [id], ['login'])[0]['login']
+		default.update({'login': login+' (copy)'})
+		return super(users, self).copy(cr, uid, id, default, context)
 users()
 
