@@ -51,8 +51,9 @@ class crm_case(osv.osv):
 	_name = "crm.case"
 	_description = "Case"
 	_columns = {
+		'id': fields.integer('Case ID', readonly=True),
 		'name': fields.char('Case Description',size=64, required=True),
-		'priority': fields.selection([('very low','Very Low'),('low','Low'),('medium','Medium'),('high','High'),('very high','Very High')], 'Priority', required=True),
+		'priority': fields.selection([('very low','Very Low'),('low','Low'),('medium','Medium'),('high','High'),('very high','Very High')], 'Priority'),
 		'active': fields.boolean('Active'),
 		'description': fields.text('Description'),
 		'type': fields.selection([('sale','Sale Case'),('support','Support Case (Helpdesk)'),('purchase','Purchase Case')], 'Case Type'),
@@ -62,6 +63,7 @@ class crm_case(osv.osv):
 		'probability': fields.float('Probability (0.50)'),
 		'document': fields.char('Document', size=64),
 		'partner_id': fields.many2one('res.partner', 'Partner', relate=True),
+		'partner_address_id': fields.many2one('res.partner.address', 'Contact', domain="[('partner_id','=',partner_id)]"),
 		'som': fields.many2one('res.partner.som', 'State of Mind'),
 		'date': fields.datetime('Date'),
 		'date_deadline': fields.date('Date Next Action'),
@@ -69,8 +71,9 @@ class crm_case(osv.osv):
 		'canal_id': fields.many2one('res.partner.canal', 'Channel', relate=True),
 		'user_id': fields.many2one('res.users', 'User Responsible', relate=True),
 		'history_line': fields.one2many('crm.case.history', 'case_id', 'Case History'),
-		'state': fields.selection([('draft','Draft'),('open','Open'),('success','Success'), ('close', 'Closed')], 'State', size=16, readonly=True),
-		'ref' : fields.reference('Reference', selection=_links_get, size=128)
+		'state': fields.selection([('draft','Draft'),('open','Open'),('cancel', 'Canceled'),('done', 'Closed')], 'State', size=16, readonly=True),
+		'ref' : fields.reference('Reference', selection=_links_get, size=128),
+		'ref2' : fields.reference('Reference 2', selection=_links_get, size=128)
 	}
 	_defaults = {
 		'active': lambda *a: 1,
@@ -78,28 +81,64 @@ class crm_case(osv.osv):
 		'priority': lambda *a: 'low',
 		'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
 	}
-	_order = 'date_deadline desc'
+	_order = 'date_deadline desc, id'
 	def case_log(self, cr, uid, ids, *args):
 		ptype = {'sale':'customer', 'support':'customer', 'vendor':'supplier'}
 		for case in self.read(cr, uid, ids):
-			self.pool.get('crm.case.history').create(cr, uid, {'name':'Contact Operation', 'description':case['description'], 'som':(case['som'] or False) and case['som'][0], 'canal_id':(case['canal_id'] or False) and case['canal_id'][0], 'user_id':(case['user_id'] or False) and case['user_id'][0], 'case_id':case['id']})
+			self.pool.get('crm.case.history').create(cr, uid,
+					{'name': 'Contact Operation',
+					 'description': case['description'],
+					 'som': (case['som'] or False) and case['som'][0],
+					 'canal_id': (case['canal_id'] or False) and case['canal_id'][0],
+					 'user_id': (case['user_id'] or False) and case['user_id'][0],
+					 'case_id': case['id']})
 		return self.write(cr, uid, ids, {'description':False, 'som':False, 'canal_id': False})
 
 	def case_close(self, cr, uid, ids, *args):
 		ptype = {'sale':'customer', 'support':'customer', 'vendor':'supplier'}
 		for case in self.read(cr, uid, ids):
-			if self.pool.get('res.partner.event.type').check(cr, uid, 'crm_case_close'):
-				self.pool.get('res.partner.event').create(cr, uid, {'name':'Close Case: '+case['name'], 'som':(case['som'] or False) and case['som'][0], 'description':case['description'], 'document':case['document'], 'partner_id':case['partner_id'][0], 'date':time.strftime('%Y-%m-%d %H:%M:%S'), 'canal_id':(case['canal_id'] or False) and case['canal_id'][0], 'user_id':(case['user_id'] or False) and case['user_id'][0], 'partner_type':ptype.get(case['type'], 'customer')})
-			self.pool.get('crm.case.history').create(cr, uid, {'name':'Close', 'description':case['description'], 'som':False, 'canal_id':False, 'user_id':uid, 'case_id':case['id']})
-		self.write(cr, uid, ids, {'state':'done', 'active':False, 'date_closed': time.strftime('%Y-%m-%d')})
+			if case['partner_id'] and self.pool.get('res.partner.event.type').check(cr, uid, 'crm_case_close'):
+				self.pool.get('res.partner.event').create(cr, uid,
+						{'name': 'Close Case: '+case['name'],
+						 'som': (case['som'] or False) and case['som'][0],
+						 'description': case['description'],
+						 'document': case['document'],
+						 'partner_id': case['partner_id'][0],
+						 'date': time.strftime('%Y-%m-%d %H:%M:%S'),
+						 'canal_id': (case['canal_id'] or False) and case['canal_id'][0],
+						 'user_id': (case['user_id'] or False) and case['user_id'][0],
+						 'partner_type': ptype.get(case['type'], 'customer')})
+			self.pool.get('crm.case.history').create(cr, uid,
+					{'name': 'Close',
+					 'description': case['description'],
+					 'som': False,
+					 'canal_id': False,
+					 'user_id': uid,
+					 'case_id': case['id']})
+		self.write(cr, uid, ids, {'state':'done', 'date_closed': time.strftime('%Y-%m-%d')})
 		return True
 
 	def case_open(self, cr, uid, ids, *args):
 		ptype = {'sale':'customer', 'support':'customer', 'vendor':'supplier'}
 		for case in self.read(cr, uid, ids):
-			if self.pool.get('res.partner.event.type').check(cr, uid, 'crm_case_open'):
-				self.pool.get('res.partner.event').create(cr, uid, {'name':'Open Case: '+case['name'], 'som':(case['som'] or False) and case['som'][0], 'description':case['description'], 'document':case['document'], 'partner_id':case['partner_id'][0], 'date':case['date'], 'canal_id':(case['canal_id'] or False) and case['canal_id'][0], 'user_id':uid, 'partner_type':ptype.get(case['type'], 'customer')})
-			self.pool.get('crm.case.history').create(cr, uid, {'name':'Open', 'description':case['description'], 'som':False, 'canal_id':False, 'user_id':uid, 'case_id':case['id']})
+			if case['partner_id'] and self.pool.get('res.partner.event.type').check(cr, uid, 'crm_case_open'):
+				self.pool.get('res.partner.event').create(cr, uid,
+						{'name':'Open Case: '+case['name'],
+						 'som':(case['som'] or False) and case['som'][0],
+						 'description':case['description'],
+						 'document':case['document'],
+						 'partner_id':case['partner_id'][0],
+						 'date':case['date'],
+						 'canal_id':(case['canal_id'] or False) and case['canal_id'][0],
+						 'user_id':uid,
+						 'partner_type':ptype.get(case['type'], 'customer')})
+			self.pool.get('crm.case.history').create(cr, uid,
+					{'name': 'Open',
+					 'description': case['description'],
+					 'som': False,
+					 'canal_id': False,
+					 'user_id': uid,
+					 'case_id': case['id']})
 		self.write(cr, uid, [case['id']], {'state':'open', 'active':True, 'user_id':uid})
 		return True
 
@@ -132,20 +171,24 @@ class crm_case_history(osv.osv):
 crm_case_history()
 
 class crm_segmentation(osv.osv):
+	'''
+		A segmentation is a tool to automatically assign categories on partners.
+		These assignations are based on criterions.
+	'''
 	_name = "crm.segmentation"
 	_description = "Partner Segmentation"
 	_columns = {
-		'name': fields.char('Name', size=64, required=True),
+		'name': fields.char('Name', size=64, required=True, help='The name of the segmentation.'),
 		'description': fields.text('Description'),
-		'categ_id': fields.many2one('res.partner.category', 'Partner Category', required=True),
-		'exclusif': fields.boolean('Exclusive'),
+		'categ_id': fields.many2one('res.partner.category', 'Partner Category', required=True, help='The partner category that will be added to partners that match the segmentation criterions after computation.'),
+		'exclusif': fields.boolean('Exclusive', help='Check if the category is limited to partners that match the segmentation criterions. If checked, remove the category from partners that doesn\'t match segmentation criterions'),
 		'state': fields.selection([('not running','Not Running'),('running','Running')], 'Execution State', readonly=True),
 		'partner_id': fields.integer('Max Partner ID processed'),
 		'segmentation_line': fields.one2many('crm.segmentation.line', 'segmentation_id', 'Criteria', required=True),
-		'som_interval': fields.integer('Days per Periode'),
-		'som_interval_max': fields.integer('Max Interval'),
-		'som_interval_decrease': fields.float('Decrease (0>1)'),
-		'som_interval_default': fields.float('Default (0=None)'),
+		'som_interval': fields.integer('Days per Periode', help="A period is the average number of days between two cycle of sale or purchase for this segmentation. It's mainly used to detect if a partner has not purchased or buy for a too long time, so we suppose that his state of mind has decreased because he probably bought goods to another supplier. Use this functionnality for recurring businesses."),
+		'som_interval_max': fields.integer('Max Interval', help="The computation is made on all events that occured during this interval, the past X periods."),
+		'som_interval_decrease': fields.float('Decrease (0>1)', help="If the partner has not purchased (or buied) during a period, decrease the state of mind by this factor. It\'s a multiplication"),
+		'som_interval_default': fields.float('Default (0=None)', help="Default state of mind for period preceeding the 'Max Interval' computation. This is the starting state of mind by default if the partner has no event."),
 	}
 	_defaults = {
 		'partner_id': lambda *a: 0,
