@@ -46,7 +46,7 @@ def _delivery_default(self, cr, uid, data, context):
 	order = order_obj.browse(cr, uid, data['ids'])[0]
 	if not order.state in ('draft'):
 		raise except_osv('Order not in draft state !', 'The order state have to be draft to add delivery lines.')
-	carrier_id = ir.ir_get(cr, uid, 'meta', 'delivery.carrier',[('res.partner',order.partner_id.id)])[0][2]
+	carrier_id = order.partner_id.property_delivery_carrier
 	return {'carrier_id': carrier_id}
 
 def _delivery_set(self, cr, uid, data, context):
@@ -92,7 +92,7 @@ def _delivery_set(self, cr, uid, data, context):
 			'product_id': grid.carrier_id.product_id.id,
 			'price_unit': price,
 			'date_planned': dt,
-			'tax_id': [lambda x: x.id for x in grid.carrier_id.product_id.taxes_id ],
+			'tax_id': [(6,0,[x.id for x in grid.carrier_id.product_id.taxes_id])],
 			'type': 'make_to_stock'
 		})
 	return {}
