@@ -1,10 +1,10 @@
 {
-	"name" : "Human Ressources",
+	"name" : "Human Resources",
 	"version" : "0.1",
 	"author" : "Tiny",
-	"category" : "Generic Modules/Human Ressources",
+	"category" : "Generic Modules/Human Resources",
 	"website" : "http://tinyerp.com/module_hr.html",
-	"depends" : ["base"],
+	"depends" : ["base","account"],
 	"init_xml" : [],
 	"demo_xml" : ["hr_demo.xml", "hr_bel_holidays_2005.xml"],
 	"update_xml" : ["hr_workflow.xml", "hr_view.xml", "hr_report.xml", "hr_wizard.xml"],
