@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2005-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: hr.py 2997 2006-05-02 08:33:47Z pinky $
+# $Id: hr.py 3751 2006-08-09 13:15:36Z mvd $
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -80,6 +80,17 @@ class hr_timesheet_group(osv.osv):
 		return result
 hr_timesheet_group()
 
+
+class hr_employee_category(osv.osv):
+	_name = "hr.employee.category"
+	_description = "Employee Category"
+	_columns = {
+		'name' : fields.char("Category", size=64, required=True),
+		'parent_id': fields.many2one('hr.employee.category', 'Parent category'),
+		'child_ids': fields.one2many('hr.employee.category', 'parent_id', 'Childs Categories')
+	}
+hr_employee_category()
+
 class hr_employee(osv.osv):
 	_name = "hr.employee"
 	_description = "Employee"
@@ -96,55 +107,52 @@ class hr_employee(osv.osv):
 		'expenses' : fields.one2many('hr.expense', 'employee_id', "Employee's expenses"),
 		'workgroups' : fields.many2many('hr.timesheet.group', 'hr_timesheet_employee_rel', 'emp_id', 'tgroup_id', "Employee's work team"),
 		'user_id' : fields.many2one('res.users', 'Tiny ERP User'),
+		'category_id' : fields.many2one('hr.employee.category', 'Category'),
 		'regime' : fields.float('Workhours by week'),
 		'holiday_max' : fields.integer("Number of holidays"),
+		'parent_id': fields.many2one('hr.employee', 'Boss'),
+		'child_ids': fields.one2many('hr.employee', 'parent_id','Subordinates'),
+		'bank_account': fields.char("Bank Account", size=16),
+		'monthly_max_expenses': fields.float("Max expenses per month"),
 	}
 	_defaults = {
 		'active' : lambda *a: True,
 		'state' : lambda *a: 'absent',
 	}
-	
-	def __checkSISOTime(self, cr, emp_id):
-		sql = '''
-		select name
-		from hr_attendance as att
-		where employee_id = %d
-		order by name desc
-		limit 2
-		''' % emp_id
-		cr.execute(sql)
-		atts = cr.fetchall()
-		return DateTime.strptime(atts[0][0], '%Y-%m-%d %H:%M:%S') - DateTime.strptime(atts[1][0], '%Y-%m-%d %H:%M:%S')
-
-	def sign_out(self, cr, uid, ids,*args):
+	def sign_change(self, cr, uid, ids, context={}, dt=False):
 		for emp in self.browse(cr, uid, ids):
-			att_id = self.pool.get('hr.attendance').create(cr, uid, {'action':'sign_out', 'employee_id':emp.id})
-			self.write(cr, uid, [emp.id], {'state':'absent'})
-			elapsed_time = self.__checkSISOTime(cr, emp.id)
-			if elapsed_time.hours >= 16:
-				yesterday = (now().day_of_week + 6) % 7
-				cr.execute("select hour_to from hr_timesheet as ts, hr_timesheet_group as tsg, hr_employee as emp, hr_timesheet_employee_rel as rel where ts.dayofweek<=%d and ts.tgroup_id= tsg.id and emp.id=%d and tsg.id=rel.tgroup_id and emp.id=rel.emp_id order by ts.dayofweek desc, ts.date_from desc,ts.hour_to desc limit 1", (yesterday, emp.id))
-				last = cr.dictfetchall()
-				if last:
-					self.pool.get('hr.attendance').write(cr, uid, [att_id], {'name' : last[0]['hour_to']})
-				if emp.workgroups and emp.workgroups[0].manager:
-					request = self.pool.get('res.request')
-					request.create(cr, uid, 
-						  {'name' : "%s's sign out time" % emp.name,
-						   'req_ref' : 'hr:employee:%d' % emp.id,
-						   'req_type' : 'info',
-						   'req_priority' : str(0),
-						   'state' : 'wait',
-						   'req_summary' : "Too long sign out delay (%f hours) for %s" % (elapsed_time.hours, emp.name),
-						   'act_from' : emp.id,
-						   'act_to' : emp.workgroups[0].manager.id,
-						   'act_title' : "Too long sign out delay",
-						   })
+			if not self._action_check(cr, uid, emp.id, dt, context):
+				raise osv.except_osv('Warning', 'You tried to sign with a date anterior to another event !\nTry to contact the administrator to correct attendances.')
+			res = {'action':'action', 'employee_id':emp.id}
+			if dt:
+				res['name'] = dt
+			att_id = self.pool.get('hr.attendance').create(cr, uid, res)
 		return True
-	
-	def sign_in(self, cr, uid, ids, *args):
+
+	def sign_out(self, cr, uid, ids, context={}, dt=False, *args):
 		for emp in self.browse(cr, uid, ids):
-			self.pool.get('hr.attendance').create(cr, uid, {'action':'sign_in', 'employee_id':emp.id})
+			if not self._action_check(cr, uid, emp.id, dt, context):
+				raise osv.except_osv('Warning', 'You tried to sign out with a date anterior to another event !\nTry to contact the administrator to correct attendances.')
+			res = {'action':'sign_out', 'employee_id':emp.id}
+			if dt:
+				res['name'] = dt
+			att_id = self.pool.get('hr.attendance').create(cr, uid, res)
+			self.write(cr, uid, [emp.id], {'state':'absent'})
+		return True
+
+	def _action_check(self, cr, uid, emp_id, dt=False,context={}):
+		cr.execute('select max(name) from hr_attendance where employee_id=%d', (emp_id,))
+		res = cr.fetchone()
+		return not (res and (res[0]>=(dt or time.strftime('%Y-%m-%d %H:%M:%S'))))
+
+	def sign_in(self, cr, uid, ids, context={}, dt=False, *args):
+		for emp in self.browse(cr, uid, ids):
+			if not self._action_check(cr, uid, emp.id, dt, context):
+				raise osv.except_osv('Warning', 'You tried to sign in with a date anterior to another event !\nTry to contact the administrator to correct attendances.')
+			res = {'action':'sign_in', 'employee_id':emp.id}
+			if dt:
+				res['name'] = dt
+			self.pool.get('hr.attendance').create(cr, uid, res)
 			self.write(cr, uid, [emp.id], {'state':'present'})
 		return True
 
@@ -186,8 +194,8 @@ class hr_attendance(osv.osv):
 	_name = "hr.attendance"
 	_description = "Attendance"
 	_columns = {
-		'name' : fields.datetime('Date', size=16),
-		'action' : fields.selection([('sign_in', 'Sign In'), ('sign_out', 'Sign Out')], 'Action'),
+		'name' : fields.datetime('Date'),
+		'action' : fields.selection([('sign_in', 'Sign In'), ('sign_out', 'Sign Out'),('action','Action')], 'Action'),
 		'action_desc' : fields.many2one("hr.action.reason", "Action reason", domain="[('action_type', '=', action)]"),
 		'employee_id' : fields.many2one('hr.employee', 'Employee',required=True),
 	}
@@ -201,7 +209,8 @@ class hr_attendance(osv.osv):
 		select action, name
 		from hr_attendance as att
 		where employee_id = (select employee_id from hr_attendance where id=%s)
-		order by id desc
+		and action in ('sign_in','sign_out')
+		order by name desc
 		limit 2
 		''' % ids[0]
 		cr.execute(sql)
@@ -210,7 +219,6 @@ class hr_attendance(osv.osv):
 	
 	_constraints = [(_altern_si_so, 'Error: Sign in (resp. Sign out) must follow Sign out (resp. Sign in)', ['action'])]
 	_order = 'name desc'
-
 hr_attendance()
 
 class hr_holidays_status(osv.osv):
@@ -255,12 +263,15 @@ class hr_expense(osv.osv):
 		'txt': fields.text('Long Description'),
 		'type': fields.many2one('hr.expense.type', 'Expense type'),
 		'employee_id': fields.many2one('hr.employee', 'Employee', required=True),
+		'analytic_account': fields.many2one('account.analytic.account','Analytic account'),
+		'piece': fields.many2one('account.move.line','Reference'),
 		'state': fields.selection([
 			('draft', 'Draft'),
 			('confirm', 'Waiting confirmation'),
 			('accepted', 'Accepted'),
 			('refused', 'Refused')],
 			'State', readonly=True),
+
 	}
 	_defaults = {
 		'date' : lambda *a: time.strftime('%Y-%m-%d'),
