@@ -39,23 +39,17 @@ import report.render
 import report.common
 
 from report.interface import report_rml
+from report.interface import toxml
 
 one_day = DateTime.RelativeDateTime(days=1)
 month2name = [0,'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']
 
-def toxml(val):
-	return val.replace('&', '&amp;').replace('<','&lt;').replace('>','&gt;').decode('utf-8').encode('utf-8')
-
 def hour2str(h):
 	hours = int(h)
 	minutes = int(round((h - hours) * 60, 0))
 	return '%02dh%02d' % (hours, minutes)
 
 class report_custom(report_rml):
-
-	def __init__(self, *args):
-		super(report_custom, self).__init__(*args)
-	
 	def create_xml(self, uid, ids, datas, context):
 		service = netsvc.LocalService('object_proxy')
 
