@@ -38,20 +38,15 @@ import report.render
 import report.common
 
 from report.interface import report_rml
+from report.interface import toxml
 
 one_week = DateTime.RelativeDateTime(days=7)
 num2day = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
 
-def toxml(val):
-	return val.replace('&', '&amp;').replace('<','&lt;').replace('>','&gt;').decode('utf-8').encode('utf-8')
-
 def to_hour(h):
 	return int(h), int(round((h - int(h)) * 60, 0))
 
 class report_custom(report_rml):
-	def __init__(self, *args):
-		super(report_custom, self).__init__(*args)
-	
 	def create_xml(self, uid, ids, datas, context):
 		service = netsvc.LocalService('object_proxy')
 
