@@ -1,8 +1,8 @@
 {
-	"name" : "Human Ressources (Timesheet encoding from project tasks)",
+	"name" : "Human Resources (Timesheet encoding from project tasks)",
 	"version" : "1.0",
 	"author" : "Tiny",
-	"category" : "Generic Modules/Human Ressources",
+	"category" : "Generic Modules/Human Resources",
 	"website" : "http://tinyerp.com/module_hr.html",
 	"depends" : ["project", "hr_timesheet"],
 	"update_xml" : ["hr_timesheet_project_view.xml"],
