@@ -1,5 +1,5 @@
 {
-	"name" : "Localisation for Switzerland", # based on belgium files
+	"name" : "Localisation for Switzerland",
 	"version" : "1.0",
 	"author" : "Tiny",
 	"category" : "Localisation/Europe",
