@@ -3,7 +3,7 @@
 	"version" : "1.0",
 	"author" : "Tiny",
 	"category" : "Localisation/Europe",
-	"depends" : ["base", "account"],
+	"depends" : ["account"],
 	"init_xml" : [],
 	"demo_xml" : ["account_demo.xml"],
 	"update_xml" : ["account_pcmn_belgium.xml"],
