@@ -50,7 +50,7 @@ class mrp_workcenter(osv.osv):
 	_columns = {
 		'name': fields.char('Name', size=64, required=True),
 		'active': fields.boolean('Active'),
-		'type': fields.selection([('machine','Machine'),('hr','Human Ressource'),('tool','Tool')], 'Type', required=True),
+		'type': fields.selection([('machine','Machine'),('hr','Human Resource'),('tool','Tool')], 'Type', required=True),
 		'code': fields.char('Code', size=8),
 		'timesheet_id': fields.many2one('hr.timesheet.group', 'Timesheet'),
 		'note': fields.text('Description'),
@@ -144,7 +144,7 @@ class mrp_bom(osv.osv):
 		'name': fields.char('Name', size=64, required=True),
 		'code': fields.char('Code', size=16),
 		'active': fields.boolean('Active'),
-		'type': fields.selection([('normal','Normal BoM'),('phantom','Phantom')],'BoM Type',required=True),
+		'type': fields.selection([('normal','Normal BoM'),('phantom','Phantom')], 'BoM Type', required=True, help="Use a phantom bill of material in lines that have a sub-bom and that have to be automatically computed in one line, without habing two production orders."),
 
 		'date_start': fields.date('Valid from'),
 		'date_stop': fields.date('Valid until'),
@@ -161,9 +161,9 @@ class mrp_bom(osv.osv):
 		'product_efficiency': fields.float('Product Efficiency', required=True),
 
 		'bom_lines': fields.one2many('mrp.bom', 'bom_id', 'BoM Lines'),
-		'bom_id': fields.many2one('mrp.bom', 'Parent BoM'),
+		'bom_id': fields.many2one('mrp.bom', 'Parent BoM', ondelete='cascade'),
 
-		'routing_id': fields.many2one('mrp.routing', 'Routing'),
+		'routing_id': fields.many2one('mrp.routing', 'Routing', help="The list of operations (list of workcenters) to produce the finnished product. The routing is mainly used to compute workcenter costs during operations and to plan futur loads on workcenters based on production plannification."),
 
 		'property_ids': fields.many2many('mrp.property', 'mrp_bom_property_rel', 'bom_id','property_id', 'Properties'),
 
@@ -178,6 +178,9 @@ class mrp_bom(osv.osv):
 		'type': lambda *a: 'normal',
 	}
 	_order = "sequence"
+	_sql_constraints = [
+		('bom_qty_zero', 'CHECK (product_qty>0)',  'All product quantities must be greater than 0 !'),
+	]
 	
 	def onchange_product_id(self, cr, uid, ids, product_id, name, context={}):
 		if product_id:
@@ -190,6 +193,7 @@ class mrp_bom(osv.osv):
 
 	def _bom_find(self, cr, uid, product_id, product_uom, properties = []):
 		bom_result = False
+		# Why searching on BoM without parent ?
 		cr.execute('select id from mrp_bom where product_id=%d and bom_id is null order by sequence', (product_id,))
 		ids = map(lambda x: x[0], cr.fetchall())
 		max_prop = 0
@@ -305,7 +309,7 @@ class mrp_production(osv.osv):
 		'date_start': fields.datetime('Start Date'),
 		'date_finnished': fields.datetime('End Date'),
 
-		'bom_id': fields.many2one('mrp.bom', 'Bill of Material', domain=[('bom_id','=','False')]),
+		'bom_id': fields.many2one('mrp.bom', 'Bill of Material', domain=[('bom_id','=',False)]),
 
 		'picking_id': fields.many2one('stock.picking', 'Picking list'),
 		'move_prod_id': fields.many2one('stock.move', 'Move Product'),
@@ -318,9 +322,7 @@ class mrp_production(osv.osv):
 		'state': fields.selection([('draft','Draft'),('picking_except', 'Picking Exception'),('confirmed','Waiting Goods'),('ready','Ready to Produce'),('in_production','In Production'),('cancel','Canceled'),('done','Done')],'State', readonly=True)
 	}
 	_defaults = {
-		'active': lambda *a: 1,
 		'priority': lambda *a: '1',
-		'type': lambda *a: 'make_to_stock',
 		'state': lambda *a: 'draft',
 		'product_qty':  lambda *a: 1.0,
 	}
@@ -393,7 +395,7 @@ class mrp_production(osv.osv):
 				vals= {'state':'confirmed', 'product_uom': production.product_uom.id, 'product_id': production.product_id.id, 'product_qty':production.product_qty}
 				self.pool.get('stock.move').write(cr, uid, [production.move_created_id.id], vals)
 			else:
-				source = ir.ir_get(cr,uid,'meta','stock.lot.production', [('product.product', production.product_id.id)])[0][2][0]
+				source = production.product_id.product_tmpl_id.property_stock_production[0]
 				vals = {
 					'name':'PROD:'+production.name,
 					'date_planned': production.date_planned,
@@ -470,22 +472,19 @@ class mrp_production(osv.osv):
 				self.action_compute(cr, uid, [production.id])
 				production = self.browse(cr, uid, [production.id])[0]
 			picking_id = self.pool.get('stock.picking').create(cr, uid, {
-				'name':'PICK:'+production.name,
-				'origin':'PROD:'+str(production.id)+':'+production.name,
-				'type':'internal',
+				'origin': 'PROD:%d:%s' % (production.id, production.name),
+				'type': 'internal',
 				'move_type': 'one',
 				'state': 'auto',
 				'auto_picking': True,
 			})
 			toconfirm = True
 			
-			delay = {}
-			for line in production.workcenter_lines:
-				delay[line.sequence] = max(line.hour, delay.get(line.sequence,0))
-			totaldelay = reduce(lambda x,y: x+y, delay.values(), 0)
+			if production.move_prod_id:
+				self.pool.get('stock.move').write(cr, uid, [production.move_prod_id.id], {'location_id':production.location_dest_id.id})
 
-			source = ir.ir_get(cr,uid,'meta','stock.lot.production', [('product.product', production.product_id.id)])[0][2]
-			res_final_id = self.pool.get('stock.move').create(cr, uid, {
+			source = production.product_id.product_tmpl_id.property_stock_production[0]
+			data = {
 				'name':'PROD:'+production.name,
 				'date_planned': production.date_planned,
 				'product_id': production.product_id.id,
@@ -494,12 +493,15 @@ class mrp_production(osv.osv):
 				'location_id': source,
 				'location_dest_id': production.location_dest_id.id,
 				'move_dest_id': production.move_prod_id.id,
-				'state': 'waiting',
-			})
+				'state': 'waiting'
+			}
+			res_final_id = self.pool.get('stock.move').create(cr, uid, data)
+			
 			self.write(cr, uid, [production.id], {'move_created_id':res_final_id})
 			moves = []
 			for line in production.product_lines:
 				move_id=False
+				newdate = production.date_planned
 				if line.product_id.type=='product':
 					res_dest_id = self.pool.get('stock.move').create(cr, uid, {
 						'name':'PROD:'+production.name,
@@ -519,18 +521,16 @@ class mrp_production(osv.osv):
 						'product_id': line.product_id.id,
 						'product_qty': line.product_qty,
 						'product_uom': line.product_uom.id,
-						'date_planned': production.date_planned, # TODO: - production delay
+						'date_planned': newdate,
 						'move_dest_id': res_dest_id,
 						'location_id': production.location_src_id.id,
 						'location_dest_id': production.location_src_id.id,
 						'state': 'waiting',
 					})
-				newdate = DateTime.strptime(production.date_planned, '%Y-%m-%d') - DateTime.RelativeDateTime(days=totaldelay/3)
 				proc_id = self.pool.get('mrp.procurement').create(cr, uid, {
 					'name': 'PROC:'+production.name,
 					'origin': 'PROD:'+str(production.id)+':'+production.name,
-					#TODO: A deduire: le delai de production
-					'date_planned': newdate.strftime('%Y-%m-%d'),
+					'date_planned': newdate,
 					'product_id': line.product_id.id,
 					'product_qty': line.product_qty,
 					'product_uom': line.product_uom.id,
@@ -545,6 +545,12 @@ class mrp_production(osv.osv):
 				wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
 			self.write(cr, uid, [production.id], {'picking_id':picking_id, 'move_lines': [(6,0,moves)], 'state':'confirmed'})
 		return picking_id
+
+	def force_production(self, cr, uid, ids, *args):
+		pick_obj = self.pool.get('stock.picking')
+		pick_obj.force_assign(cr, uid, [prod.picking_id.id for prod in self.browse(cr, uid, ids)])
+		return True
+
 mrp_production()
 
 class mrp_production_workcenter_line(osv.osv):
@@ -694,7 +700,12 @@ class mrp_procurement(osv.osv):
 			if procurement.product_id.product_tmpl_id.supply_method=='produce':
 				return False
 			if not procurement.product_id.seller_ids:
-				cr.execute('update mrp_procurement set message=%s where id=%d', ('No default seller defined for this product !', procurement.id))
+				cr.execute('update mrp_procurement set message=%s where id=%d', ('No supplier defined for this product !', procurement.id))
+				return False
+			partner = procurement.product_id.seller_ids[0].name
+			address_id = self.pool.get('res.partner').address_get(cr, uid, [partner.id], ['delivery'])['delivery']
+			if not address_id:
+				cr.execute('update mrp_procurement set message=%s where id=%d', ('No address defined for the supplier', procurement.id))
 				return False
 		return True
 
@@ -710,7 +721,7 @@ class mrp_procurement(osv.osv):
 				if not procurement.move_id:
 					source = procurement.location_id.id
 					if procurement.procure_method=='make_to_order':
-						source = ir.ir_get(cr,uid,'meta','stock.lot.procurement', [('product.product', procurement.product_id.id)])[0][2]
+						source = procurement.product_id.product_tmpl_id.property_stock_procurement[0]
 					id = self.pool.get('stock.move').create(cr, uid, {
 						'name': 'PROC:'+procurement.name,
 						'location_id': source,
@@ -742,6 +753,9 @@ class mrp_procurement(osv.osv):
 			id = procurement.move_id.id
 			if not (procurement.move_id.state in ('done','assigned','cancel')):
 				ok = ok and self.pool.get('stock.move').action_assign(cr, uid, [id])
+				cr.execute('select count(id) from stock_warehouse_orderpoint where product_id=%d', (procurement.product_id.id,))
+				if not cr.fetchone()[0]:
+					cr.execute('update mrp_procurement set message=%s where id=%d', ('MTS and no orderpoint defined', procurement.id))
 		return ok
 
 	def action_produce_assign_service(self, cr, uid, ids, context={}):
@@ -754,6 +768,7 @@ class mrp_procurement(osv.osv):
 		for procurement in self.browse(cr, uid, ids):
 			res_id = procurement.move_id.id
 			loc_id = procurement.location_id.id
+			newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d') - DateTime.RelativeDateTime(days=procurement.product_id.product_tmpl_id.produce_delay or 0.0)
 			produce_id = self.pool.get('mrp.production').create(cr, uid, {
 				'name': procurement.name,
 				'origin': 'PROC:'+str(procurement.id)+':'+procurement.name,
@@ -762,7 +777,7 @@ class mrp_procurement(osv.osv):
 				'product_uom': procurement.product_uom.id,
 				'location_src_id': procurement.location_id.id,
 				'location_dest_id': procurement.location_id.id,
-				'date_planned': procurement.date_planned, 
+				'date_planned': newdate,
 				'move_prod_id': res_id,
 			})
 			self.write(cr, uid, [procurement.id], {'state':'running'})
@@ -778,29 +793,30 @@ class mrp_procurement(osv.osv):
 			partner = procurement.product_id.seller_ids[0].name
 			partner_id = partner.id
 			address_id = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])['delivery']
-			pricelist_id=ir.ir_get(cr,uid,'meta','product.pricelist.purchase', [('res.partner',partner_id)])[0][2]
-			price = self.pool.get('product.pricelist').price_get(cr,uid,[pricelist_id], procurement.product_id.id, procurement.product_qty, {'uom': procurement.product_uom.id})[pricelist_id]
+			pricelist_id = partner.property_product_pricelist_purchase[0]
+			price = self.pool.get('product.pricelist').price_get(cr, uid, [pricelist_id], procurement.product_id.id, procurement.product_qty, {'uom': procurement.product_uom.id})[pricelist_id]
+
+			newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d') - DateTime.RelativeDateTime(days=procurement.product_id.product_tmpl_id.seller_delay or 0.0)
 			line = {
-				'name': procurement.name,
+				'name': procurement.product_id.name,
 				'product_qty': procurement.product_qty,
 				'product_id': procurement.product_id.id,
 				'product_uom': procurement.product_uom.id,
 				'price_unit': price,
-				'date_planned': procurement.date_planned,
-				'taxes_id': [(6,0,[x.id for x in procurement.product_id.product_tmpl_id.taxes_id])],
+				'date_planned': newdate.strftime('%Y-%m-%d'),
+				'taxes_id': [(6, 0, [x.id for x in procurement.product_id.product_tmpl_id.taxes_id])],
 				'move_dest_id': res_id,
 			}
 
-			# TODO: may use a draft order instead
 			purchase_id = self.pool.get('purchase.order').create(cr, uid, {
 				'origin': 'PROC:'+str(procurement.id)+':'+str(procurement.name),
-				'partner_id':partner_id,
-				'partner_address_id':address_id,
+				'partner_id': partner_id,
+				'partner_address_id': address_id,
 				'location_id': procurement.location_id.id,
 				'pricelist_id': pricelist_id,
 				'order_line': [(0,0,line)]
 			})
-			self.write(cr, uid, [procurement.id], {'state':'running','purchase_id':purchase_id})
+			self.write(cr, uid, [procurement.id], {'state':'running', 'purchase_id':purchase_id})
 		return purchase_id
 
 	def action_cancel(self, cr, uid, ids):
@@ -811,6 +827,11 @@ class mrp_procurement(osv.osv):
 		if len(todo):
 			self.pool.get('stock.move').action_cancel(cr, uid, [proc.move_id.id])
 		self.write(cr, uid, ids, {'state':'cancel'})
+
+		wf_service = netsvc.LocalService("workflow")
+		for id in ids:
+			wf_service.trg_trigger(uid, 'mrp.procurement', id, cr)
+
 		return True
 
 	def action_check_finnished(self, cr, uid, ids):
@@ -829,9 +850,15 @@ class mrp_procurement(osv.osv):
 			if procurement.move_id:
 				if procurement.close_move and (procurement.move_id.state <> 'done'):
 					self.pool.get('stock.move').action_done(cr, uid, [procurement.move_id.id])
-		return self.write(cr, uid, ids, {'state':'done', 'date_close':time.strftime('%Y-%m-%d')})
+		res = self.write(cr, uid, ids, {'state':'done', 'date_close':time.strftime('%Y-%m-%d')})
+
+		wf_service = netsvc.LocalService("workflow")
+		for id in ids:
+			wf_service.trg_trigger(uid, 'mrp.procurement', id, cr)
+		return res
 mrp_procurement()
 
+#CHECKME: why the **** is this object defined here and not in stock???
 class stock_warehouse_orderpoint(osv.osv):
 	_name = "stock.warehouse.orderpoint"
 	_description = "Warehouse Orderpoint"
