@@ -31,16 +31,11 @@ import sql_db
 import ir
 from osv.osv import osv_pools
 from report.interface import report_rml
-from report.int_to_text import int_to_text
+from report.interface import toxml
 
 
-def toxml(val):
-	return val.replace('&', '&amp;').replace('<','&lt;').replace('>','&gt;').decode('utf-8').encode('latin1')
-
+#FIXME: we should use toxml
 class report_custom(report_rml):
-	def __init__(self, name, table, tmpl, xsl):
-		report_rml.__init__(self, name, table, tmpl, xsl)
-
 	def create_xml(self, uid, ids, datas, context={}):
 		cr = sql_db.db.cursor()
 		number = (datas.get('form', False) and datas['form']['number']) or 1
