@@ -102,15 +102,52 @@ product_ul()
 # Categories
 #----------------------------------------------------------
 class product_category(osv.osv):
+
+	def name_get(self, cr, uid, ids, context={}):
+		if not len(ids):
+			return []
+		reads = self.read(cr, uid, ids, ['name','parent_id'], context)
+		res = []
+		for record in reads:
+			name = record['name']
+			if record['parent_id']:
+				name = record['parent_id'][1]+' / '+name
+			res.append((record['id'], name))
+		return res
+
+	def _name_get_fnc(self, cr, uid, ids, prop, unknow_none, context):
+		res = self.name_get(cr, uid, ids)
+		return dict(res)
+
 	_name = "product.category"
 	_description = "Product Category"
 	_columns = {
 		'name': fields.char('Name', size=64, required=True),
+		'complete_name': fields.function(_name_get_fnc, method=True, type="char", string='Name'),
 		'parent_id': fields.many2one('product.category','Parent Category'),
-		'child_id': fields.one2many('product.category', 'parent_id', string='Childs Categories')
+		'child_id': fields.one2many('product.category', 'parent_id', string='Childs Categories'),
+		'property_account_income_categ': fields.property(
+		  'account.account',
+		  type='many2one', 
+		  relation='account.account', 
+		  string="Income Account", 
+		  method=True,
+		  view_load=True,
+		  group_name="Accounting Properties",
+		  help="This account will be used, instead of the default one, to value incoming stock for the current product category"),
+		'property_account_expense_categ': fields.property(
+		  'account.account',
+		  type='many2one', 
+		  relation='account.account', 
+		  string="Expense Account", 
+		  method=True,
+		  view_load=True,
+		  group_name="Accounting Properties",
+		  help="This account will be used, instead of the default one, to value outgoing stock for the current product category"),
 	}
 	def child_get(self, cr, uid, ids):
 		return [ids]
+
 product_category()
 
 
@@ -129,26 +166,47 @@ class product_template(osv.osv):
 		'name_ids': fields.one2many('product.template.name', 'template_id', 'Names for Partners'),
 		'type': fields.selection([('product','Stockable Product'),('service','Service')], 'Product Type', required=True),
 		'supply_method': fields.selection([('produce','Produce'),('buy','Buy')], 'Supply method', required=True),
-		'seller_delay': fields.float('Supplier lead time'),
-		'sale_delay': fields.float('Procurement lead time'),
+		'seller_delay': fields.float('Supplier lead time', help="This is the average delay in days between the purchase order confirmation and the reception of goods for this product and for the default supplier. It is used by the sceduler to order requests based on reordering delays."),
+		'sale_delay': fields.float('Procurement lead time', help="This is the average time between the confirmation of the customer order and the delivery of the finnished products. It's the time you promise to your customers."),
+		'produce_delay': fields.float('Production lead time', help="Average time to produce this product. This is only for the production order and, if it is a multi-level bill of material, it's only for the level of this product. Different delays will be summed for all levels and purchase orders."),
 		'procure_method': fields.selection([('make_to_stock','Make to Stock'),('make_to_order','Make to Order')], 'Procure Method', required=True),
 		'rental': fields.boolean('Rentable product'),
 		'categ_id': fields.many2one('product.category','Category', required=True, change_default=True),
 		'list_price': fields.float('List Price'),
-		'standard_price': fields.float('Standard Price', required=True),
+		'standard_price': fields.float('Standard Price', required=True, digit=(12,6)),
 		'volume': fields.float('Volume'),
 		'weight': fields.float('Weight'),
 		'cost_method': fields.selection([('standard','Standard Price'), ('pmp','PMP (Not implemented!)'), ('fifo','FIFO')], 'Costing Method', required=True),
 		'warranty': fields.float('Warranty (months)'),
-		'sale_ok': fields.boolean('Can be sold'),
-		'purchase_ok': fields.boolean('Can be Purchased'),
+		'sale_ok': fields.boolean('Can be sold', help="Determine if the product can be visible in the list of product within a selection from a sale order line."),
+		'purchase_ok': fields.boolean('Can be Purchased', help="Determine if the product is visible in the list of products within a selection from a purchase order line."),
 		'taxes_id': fields.many2many('account.tax', 'product_taxes_rel', 'prod_id', 'tax_id', 'Product Taxes', domain=[('parent_id','=',False)]),
 		'uom_id': fields.many2one('product.uom', 'Default UOM', required=True),
 		'uom_po_id': fields.many2one('product.uom', 'Purchase UOM', required=True),
 		'state': fields.selection([('draft', 'In Development'),('sellable','In production'),('end','End of lifecycle'),('obsolete','Obsolete')], 'State'),
 		'uos_id' : fields.many2one('product.uom', 'Unit of Sale'),
-		'uos_coeff' : fields.float('UOM -> UOS Coeff'),
-		'intrastat' : fields.char('Intrastat number', size=16),
+		'uos_coeff': fields.float('UOM -> UOS Coeff'),
+		'intrastat': fields.char('Intrastat number', size=16),
+		'mes_type': fields.selection((('fixed', 'Fixed'), ('variable', 'Variable')), 'Mesure type', required=True),
+		'tracking': fields.boolean('Track lots'),
+		'property_account_income': fields.property(
+		  'account.account',
+		  type='many2one', 
+		  relation='account.account', 
+		  string="Income Account", 
+		  method=True,
+		  view_load=True,
+		  group_name="Accounting Properties",
+		  help="This account will be used, instead of the default one, to value incoming stock for the current product"),
+		'property_account_expense': fields.property(
+		  'account.account',
+		  type='many2one', 
+		  relation='account.account', 
+		  string="Expense Account", 
+		  method=True,
+		  view_load=True,
+		  group_name="Accounting Properties",
+		  help="This account will be used, instead of the default one, to value outgoing stock for the current product"),
 	}
 	def _get_uom_id(self, cr, uid, *args):
 		cr.execute('select id from product_uom order by id limit 1')
@@ -163,16 +221,16 @@ class product_template(osv.osv):
 		'standard_price': lambda *a: 1,
 		'sale_ok': lambda *a: 1,
 		'sale_delay': lambda *a: 7,
+		'produce_delay': lambda *a: 1,
+		'seller_delay': lambda *a: 1,
 		'purchase_ok': lambda *a: 1,
 		'procure_method': lambda *a: 'make_to_stock',
 		'uom_id': _get_uom_id,
 		'uom_po_id': _get_uom_id,
-		'uom_price_id' : _get_uom_id,
+		#'uom_price_id' : _get_uom_id,
 		'uos_id' : _get_uom_id,
 		'uos_coeff' : lambda *a: 1.0,
-		#FIXME
-		'specific_bom': lambda *a: False,
-		'specific_routing': lambda *a: False,
+		'mes_type' : lambda *a: 'fixed',
 	}
 	# TODO: redefine name_get & name_search for product.template.name
 	def name_get(self, cr, user, ids, context={}):
@@ -209,49 +267,46 @@ class product_product(osv.osv):
 	#
 	# Utiliser browse pour limiter les queries !
 	#
-	def _product_virtual_available(self, cr, uid, ids, name, arg, context={}):
-		if context.get('shop', False):
-			cr.execute('select warehouse_id from sale_shop where id=%d', (int(context['shop']),))
-			res2 = cr.fetchone()
-			if res2:
-				context['warehouse'] = res2[0]
-		if context.get('warehouse', False):
-			cr.execute('select lot_stock_id from stock_warehouse where id=%d', (int(context['warehouse']),))
-			res2 = cr.fetchone()
-			if res2:
-				context['location'] = res2[0]
-		res = {}
-		if context.get('location', False):
-			res = self.pool.get('stock.location')._product_virtual_get(cr, uid, context['location'], ids, context)
-		for id in ids:
-			res.setdefault(id, 0)
-		return res
+	def _get_product_available_func(states, what):
+		def _product_available(self, cr, uid, ids, name, arg, context={}):
+			if context.get('shop', False):
+				cr.execute('select warehouse_id from sale_shop where id=%d', (int(context['shop']),))
+				res2 = cr.fetchone()
+				if res2:
+					context['warehouse'] = res2[0]
 
-	def _product_qty_available(self, cr, uid, ids, name, arg, context={}):
-		if context.get('shop', False):
-			cr.execute('select warehouse_id from sale_shop where id=%d', (int(context['shop']),))
-			res2 = cr.fetchone()
-			if res2:
-				context['warehouse'] = res2[0]
-		if context.get('warehouse', False):
-			cr.execute('select lot_stock_id from stock_warehouse where id=%d', (int(context['warehouse']),))
-			res2 = cr.fetchone()
-			if res2:
-				context['location'] = res2[0]
+			if context.get('warehouse', False):
+				cr.execute('select lot_stock_id from stock_warehouse where id=%d', (int(context['warehouse']),))
+				res2 = cr.fetchone()
+				if res2:
+					context['location'] = res2[0]
 
-		res = {}
-		if context.get('location', False):
-			res = self.pool.get('stock.location')._product_all_get(cr, uid, context['location'], ids, context) or {}
-		for id in ids:
-			res.setdefault(id, 0)
-		return res
+			if context.get('location', False):
+				location_ids = [context['location']]
+			else:
+				# get the list of ids of the stock location of all warehouses
+				cr.execute("select lot_stock_id from stock_warehouse")
+				location_ids = [id for (id,) in cr.fetchall()]
+				
+			# build the list of ids of children of the location given by id
+			location_ids = self.pool.get('stock.location').search(cr, uid, [('location_id', 'child_of', location_ids)])
+			res = self.pool.get('stock.location')._product_get_multi_location(cr, uid, location_ids, ids, context, states, what)
+			for id in ids:
+				res.setdefault(id, 0.0)
+			return res
+		return _product_available
 
+	_product_qty_available = _get_product_available_func(('done',), ('in', 'out'))
+	_product_virtual_available = _get_product_available_func(('confirmed','waiting','assigned','done'), ('in', 'out'))
+	_product_outgoing_qty = _get_product_available_func(('confirmed','waiting','assigned'), ('out',))
+	_product_incoming_qty = _get_product_available_func(('confirmed','waiting','assigned'), ('in',))
+	
 	def _product_lst_price(self, cr, uid, ids, name, arg, context={}):
 		res = {}
-		for p in self.browse(cr, uid, ids):
+		for p in self.browse(cr, uid, ids, context=context):
 			res[p.id] = p.list_price
 		for id in ids:
-			res.setdefault(id, 0)
+			res.setdefault(id, 0.0)
 		if 'uom' in context:
 			for id in ids:
 				res[id] = self.pool.get('product.uom')._compute_price(cr, uid, context['uom'], res[id])
@@ -278,7 +333,8 @@ class product_product(osv.osv):
 
 	_defaults = {
 		'active': lambda *a: 1,
-		'mes_type' : lambda *a: 'fixed',
+		'price_extra': lambda *a: 0.0,
+		'price_margin': lambda *a: 1.0,
 	}
 
 	_name = "product.product"
@@ -286,8 +342,11 @@ class product_product(osv.osv):
 	_table = "product_product"
 	_inherits = {'product.template': 'product_tmpl_id'}
 	_columns = {
-		'qty_available': fields.function(_product_qty_available, method=True, type='integer', string='Real Stock'),
-		'virtual_available': fields.function(_product_virtual_available, method=True, type='integer', string='Virtual Stock'),
+		'qty_available': fields.function(_product_qty_available, method=True, type='float', string='Real Stock'),
+		'virtual_available': fields.function(_product_virtual_available, method=True, type='float', string='Virtual Stock'),
+		'seller_ids': fields.one2many('product.supplierinfo', 'product_id', 'Suppliers'),
+		'incoming_qty': fields.function(_product_incoming_qty, method=True, type='float', string='Incoming'),
+		'outgoing_qty': fields.function(_product_outgoing_qty, method=True, type='float', string='Outgoing'),
 		'price': fields.function(_product_price, method=True, type='float', string='Customer Price'),
 		'lst_price' : fields.function(_product_lst_price, method=True, type='float', string='List price'),
 		'code': fields.function(_product_code, method=True, type='char', string='Code'),
@@ -297,10 +356,9 @@ class product_product(osv.osv):
 		'variants': fields.char('Variants', size=64),
 		'product_tmpl_id': fields.many2one('product.template', 'Product Template', required=True),
 		'ean13': fields.char('EAN13', size=13),
-		'packaging' : fields.one2many('product.packaging', 'product_id', 'Palettization'),
-		'mes_type' : fields.selection((('fixed', 'Fixed'), ('variable', 'Variable')), 'Mesure type', required=True),
-		'tracking' : fields.boolean('Track lots'),
-		'seller_ids': fields.one2many('product.supplierinfo', 'product_id', 'Suppliers'),
+		'packaging' : fields.one2many('product.packaging', 'product_id', 'Palettization', help="Gives the different ways to package the same product. This has no impact on the picking order and is mainly used if you use the EDI module."),
+		'price_extra': fields.float('Price Extra'),
+		'price_margin': fields.float('Price Margin')
 	}
 
 	def _check_ean_key(self, cr, uid, ids):
@@ -328,20 +386,24 @@ class product_product(osv.osv):
 		if not len(ids):
 			return []
 		def _name_get(d):
-			name = self._product_partner_ref(cr, user, [d['id']], '', '', context)[d['id']]
-			code = self._product_code(cr, user, [d['id']], '', '', context)[d['id']]
+			#name = self._product_partner_ref(cr, user, [d['id']], '', '', context)[d['id']]
+			#code = self._product_code(cr, user, [d['id']], '', '', context)[d['id']]
+			name = d.get('name','')
+			code = d.get('default_code',False)
 			if code:
 				name = '[%s] %s' % (code,name)
 			if d['variants']:
 				name = name + ' - %s' % (d['variants'],)
 			return (d['id'], name)
-		result = map(_name_get, self.read(cr, user, ids, ['variants'], context))
+		result = map(_name_get, self.read(cr, user, ids, ['variants','name','default_code'], context))
 		return result
 
-	def name_search(self, cr, user, name, args=[], operator='ilike', context={}):
-		ids = self.search(cr, user, [('default_code','=',name)]+ args)
+	def name_search(self, cr, user, name, args=[], operator='ilike', context={}, limit=80):
+		ids = self.search(cr, user, [('default_code','=',name)]+ args, limit=limit)
 		if not len(ids):
-			ids = self.search(cr, user, [('name',operator,name)]+ args)
+			ids = self.search(cr, user, [('ean13','=',name)]+ args, limit=limit)
+		if not len(ids):
+			ids = self.search(cr, user, [('name',operator,name)]+ args, limit=limit)
 		result = self.name_get(cr, user, ids, context)
 		return result
 
@@ -349,10 +411,12 @@ class product_product(osv.osv):
 	# Could be overrided for variants matrices prices
 	#
 	def price_get(self, cr, uid, ids, ptype='list_price', context={}):
-		result = self.read(cr, uid, ids)
+		result = self.read(cr, uid, ids, [ptype, 'price_extra','price_margin'])
 		result2 = {}
 		for res in result:
 			result2[res['id']] = res[ptype] or 0.0
+			if ptype=='list_price':
+				result2[res['id']] = result2[res['id']] * res['price_margin'] + res['price_extra']
 			if 'uom' in context:
 				result2[res['id']] = self.pool.get('product.uom')._compute_price(cr, uid, context['uom'], result2[res['id']])
 		return result2
@@ -363,17 +427,18 @@ class product_packaging(osv.osv):
 	_description = "Conditionnement"
 	_columns = {
 		'name' : fields.char('Description', size=64),
-		'qty' : fields.float('Quantity by UL'),
+		'qty' : fields.float('Quantity by UL', help="The total number of products you can put on one transport unit (box or palet)."),
 		'ul' : fields.many2one('product.ul', 'Type of UL', required=True),
 		'ul_qty' : fields.integer('UL by row'),
 		'rows' : fields.integer('# of rows', required=True),
 		'product_id' : fields.many2one('product.product', 'Product'),
-		'ean' : fields.char('EAN', size=14),
+		'ean' : fields.char('EAN', size=14, help="The EAN code of the transport unit."),
+		'code' : fields.char('Code', size=14, help="The code of the transport unit."),
 		'weight': fields.float('Weight Palette'),
 		'weight_ul': fields.float('Weight UL'),
 		'active' : fields.boolean('Active'),
-		}
-	
+	}
+
 	def _get_1st_ul(self, cr, uid, context={}):
 		cr.execute('select id from product_ul order by id asc limit 1')
 		res = cr.fetchone()
@@ -404,14 +469,13 @@ product_packaging()
 class product_supplierinfo(osv.osv):
 	_name = "product.supplierinfo"
 	_description = "Information about a product supplier"
-	
 	_columns = {
 		'name' : fields.many2one('res.partner', 'Supplier', required=True),
 		'sequence' : fields.integer('Sequence'),
 		'qty' : fields.integer('Minimal quantity', required=True),
 		'product_id' : fields.many2one('product.product', 'Product', required=True),
 		'delay' : fields.integer('Delivery delay', required=True),
-		}
+	}
 	_order = 'sequence'
 product_supplierinfo()
 
