@@ -7,6 +7,6 @@
 	"depends" : ["base", "sale", "account", "hr", "mrp"],
 	"init_xml" : [],
 	"demo_xml" : ["project_demo.xml"],
-	"update_xml": ["project_data.xml", "project_view.xml", "project_report.xml", "project_wizard.xml", "project_workflow.xml"],
+	"update_xml": ["project_data.xml", "project_wizard.xml", "project_view.xml", "project_report.xml", "project_workflow.xml"],
 	"active": True
 }
