@@ -32,7 +32,6 @@ from mx.DateTime import now
 import time
 
 import netsvc
-from tools import email_send
 from osv import fields, osv
 import ir
 
@@ -68,7 +67,7 @@ class project(osv.osv):
 		if not part:
 			return {'value':{'contact_id': False, 'pricelist_id': False}}
 		addr = self.pool.get('res.partner').address_get(cr, uid, [part], ['contact'])
-		pricelist=ir.ir_get(cr,uid,'meta','product.pricelist',[('res.partner',part)])[0][2]
+		pricelist = self.pool.get('res.partner').browse(cr, uid, part).property_product_pricelist[0]
 		return {'value':{'contact_id': addr['contact'], 'pricelist_id': pricelist}}
 
 	_columns = {
@@ -92,7 +91,7 @@ class project(osv.osv):
 		 'contact_id': fields.many2one('res.partner.address', 'Contact'),
 		 'shop_id': fields.many2one('sale.shop', 'Shop'),
 		 'pricelist_id': fields.many2one('product.pricelist', 'Pricelist'),
-		 'uom_id': fields.many2one('product.uom', 'Task UoM'),
+		 'uom_id': fields.many2one('product.uom', 'Tasks UoM'),
 		 'tax_ids': fields.many2many('account.tax', 'project_account_tax_rel', 'project_id','tax_id', 'Applicable taxes'),
 		 'warn_customer': fields.boolean('Warn customer'),
 		 'warn_header': fields.text('Mail header'),
@@ -156,7 +155,7 @@ class task(osv.osv):
 		'active': fields.boolean('Active'),
 		'description': fields.text('Description'),
 		'cust_desc': fields.text("Description for the customer"),
-		'priority': fields.selection([('0', 'Very Low'), ('1', 'Low'), ('2', 'Normal'), ('3', 'High'), ('4', 'Very High')], 'Priority'),
+		'priority' : fields.selection([('4','Very Low'), ('3','Low'), ('2','Medium'), ('1','Urgent'), ('0','Very urgent')], 'Priority'),
 		'sequence': fields.integer('Sequence'),
 		'type': fields.many2one('project.task.type', 'Type'),
 		'state': fields.selection([('open', 'Open'), ('progress', 'In progress'), ('cancelled', 'Cancelled'), ('done', 'Done')], 'State', readonly=True),
@@ -187,16 +186,15 @@ class task(osv.osv):
 		'sequence': lambda *a: 10,
 		'active': lambda *a: True,
 	}
-	_order = "state, sequence, priority desc, id"
+	_order = "state, sequence, priority, date_deadline, id"
 
 	def do_close(self, cr, uid, ids, *args):
-		import md5
 		request = self.pool.get('res.request')
 		tasks = self.browse(cr, uid, ids)
 		for task in tasks:
 			project = task.project_id
 			if project: 
-				if project.warn_manager:
+				if project.warn_manager and project.manager and (project.manager.id != uid):
 					request.create(cr, uid, {
 						'name': "Task '%s' closed" % task.name,
 						'state': 'waiting',
@@ -206,27 +204,7 @@ class task(osv.osv):
 						'ref_doc1': 'project.task,%d'% (task.id,),
 						'ref_doc2': 'project.project,%d'% (project.id,),
 					})
-				if project.warn_customer and task.cust_desc:
-					signature = '' 
-					if task.user_id.address_id and task.user_id.address_id.email:
-						From = task.user_id.address_id.email
-						signature = task.user_id.signature
-					else:
-						From = 'no-one@nowhere'
-					if project.contact_id and project.contact_id.email:
-						To = project.contact_id.email
-					else:
-						To = None
-						
-					if To:
-						subj = "Task '%s' closed" % task.name
-						header = project.warn_header % {'name': task.name, 'user_id': task.user_id.name, 'task_id': md5.new("%sTiny%s" % (task.id, project.id)).hexdigest()}
-						footer = project.warn_footer % {'name': task.name, 'user_id': task.user_id.name, 'task_id': md5.new("%sTiny%s" % (task.id, project.id)).hexdigest()}
-						body = '%s\n%s\n%s\n\n-- \n%s' % (header, task.cust_desc, footer, signature)
-						email_send(From, To, subj, body, None)
-
 			self.write(cr, uid, [task.id], {'state': 'done', 'date_close':time.strftime('%Y-%m-%d %H:%M:%S'), 'progress': 100})
-
 			if task.procurement_id:
 				wf_service = netsvc.LocalService("workflow")
 				wf_service.trg_validate(uid, 'mrp.procurement', task.procurement_id.id, 'subflow.done', cr)
@@ -236,15 +214,16 @@ class task(osv.osv):
 		request = self.pool.get('res.request')
 		tasks = self.browse(cr, uid, ids)
 		for task in tasks:
-			if task.project_id.warn_manager:
+			project = task.project_id
+			if project.warn_manager and project.manager.id and (project.manager.id != uid):
 				request.create(cr, uid, {
 					'name': "Task '%s' reopened" % task.name,
+					'state': 'waiting',
 					'act_from': uid,
+					'act_to': project.manager.id,
 					'ref_partner_id': task.partner_id.id,
-					'state':'waiting',
-					'act_to': task.project_id.manager.id,
-					'ref_doc1': 'project.task,%d'% (task.id,),
-					'ref_doc2': 'project.project,%d'% (task.project_id.id,),
+					'ref_doc1': 'project.task,%d' % task.id,
+					'ref_doc2': 'project.project,%d' % project.id,
 				})
 
 			self.write(cr, uid, [task.id], {'state': 'open'})
@@ -254,32 +233,17 @@ class task(osv.osv):
 		request = self.pool.get('res.request')
 		tasks = self.browse(cr, uid, ids)
 		for task in tasks:
-			if task.project_id.warn_manager:
+			project = task.project_id
+			if project.warn_manager and project.manager and (project.manager.id != uid):
 				request.create(cr, uid, {
 					'name': "Task '%s' cancelled" % task.name,
 					'state': 'waiting',
 					'act_from': uid,
-					'act_to': task.project_id.manager.id,
+					'act_to': project.manager.id,
 					'ref_partner_id': task.partner_id.id,
-					'ref_doc1': 'project.task,%d'% (task.id,),
-					'ref_doc2': 'project.project,%d'% (task.project_id.id,),
+					'ref_doc1': 'project.task,%d' % task.id,
+					'ref_doc2': 'project.project,%d' % project.id,
 				})
-			if task.project_id.warn_customer:
-				if task.user_id.address_id and task.user_id.address_id.email:
-					From = task.user_id.address_id.email
-				else:
-					From = 'no-one@nowhere'
-				if task.project_id.contact_id and task.project_id.contact_id.email:
-					To = task.project_id.contact_id.email
-				else:
-					To = None
-					
-				if To:
-					subj = "Task '%s' cancelled" % task.name
-					header = task.project_id.warn_header % {'name': task.name, 'user_id': task.user_id.name}
-					footer = task.project_id.warn_footer % {'name': task.name, 'user_id': task.user_id.name}
-					body = '%s\n%s\n%s\n\n-- \n%s' % (header, task.cust_desc, footer, signature)
-					email_send(From, To, subj, body, None)
 			self.write(cr, uid, [task.id], {'state': 'cancelled'})
 			if task.procurement_id:
 				wf_service = netsvc.LocalService("workflow")
