@@ -43,7 +43,7 @@ def _compute_tasks(cr, uid, task_list, date_begin):
 	last_date = date_begin
 	for task in task_list:
 		# TODO: reorder ! with dependencies
-		if task.state in ('open','progress'):
+		if task.state in ('open','progress') and task.user_id:
 
 			# Find the starting date of the task
 			if task.user_id.id in users:
@@ -65,7 +65,7 @@ def _compute_tasks(cr, uid, task_list, date_begin):
 			tasks[task.id] = []
 			res = osv_pools.get('hr.timesheet.group').interval_get(cr, uid, task.project_id.timesheet_id.id, date_start, task.planned_hours or 0)
 			for (d1,d2) in res:
-				tasks[task.id].append((d1,d2,task.name,task.user_id.login))
+				tasks[task.id].append((d1, d2, task.name, task.user_id.login))
 			date_close = tasks[task.id][-1][1]
 
 			# Store result
