@@ -1,8 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
-#
-# $Id: purchase.py 1005 2005-07-25 08:41:42Z nicoe $
+# Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -39,7 +37,6 @@ from mx import DateTime
 # Model definition
 #
 class purchase_order(osv.osv):
-
 	def _calc_amount(self, cr, uid, ids, prop, unknow_none, unknow_dict):
 		res = {}
 		for order in self.browse(cr, uid, ids):
@@ -48,29 +45,29 @@ class purchase_order(osv.osv):
 				res[order.id] += oline.price_unit * oline.product_qty
 		return res
 
-	def _amount_untaxed(self, cr, uid, ids, prop, unknow_none,unknow_dict):
-		id_set=",".join(map(str,ids))
+	def _amount_untaxed(self, cr, uid, ids, field_name, arg, context):
+		id_set = ",".join(map(str, ids))
 		cr.execute("SELECT s.id,COALESCE(SUM(l.price_unit*l.product_qty),0) AS amount FROM purchase_order s LEFT OUTER JOIN purchase_order_line l ON (s.id=l.order_id) WHERE s.id IN ("+id_set+") GROUP BY s.id ")
-		res=dict(cr.fetchall())
+		res = dict(cr.fetchall())
 		return res
 
-	def _amount_tax(self, cr, uid, ids, prop, unknow_none,unknow_dict):
+	def _amount_tax(self, cr, uid, ids, field_name, arg, context):
 		res = {}
 		for order in self.browse(cr, uid, ids):
 			val = 0.0
 			for line in order.order_line:
 				for tax in line.taxes_id:
-					for c in self.pool.get('account.tax').compute(cr, uid, [tax.id], line.price_unit, line.product_qty):
+					for c in self.pool.get('account.tax').compute(cr, uid, [tax.id], line.price_unit, line.product_qty, order.partner_address_id.id):
 						val+=c['amount']
 			res[order.id]=val
 		return res
 
-	def _amount_total(self, cr, uid, ids, prop, unknow_none,unknow_dict):
+	def _amount_total(self, cr, uid, ids, field_name, arg, context):
 		res = {}
-		untax = self._amount_untaxed(cr, uid, ids, prop, unknow_none,unknow_dict) 
-		tax = self._amount_tax(cr, uid, ids, prop, unknow_none,unknow_dict)
+		untax = self._amount_untaxed(cr, uid, ids, field_name, arg, context) 
+		tax = self._amount_tax(cr, uid, ids, field_name, arg, context)
 		for id in ids:
-			res[id] = untax.get(id,0) + tax.get(id,0)
+			res[id] = untax.get(id, 0.0) + tax.get(id, 0.0)
 		return res
 
 	_columns = {
@@ -90,7 +87,7 @@ class purchase_order(osv.osv):
 
 		'pricelist_id':fields.many2one('product.pricelist', 'Pricelist', required=True, states={'confirmed':[('readonly',True)], 'approved':[('readonly',True)]}),
 
-		'state': fields.selection([('draft','draft'), ('wait', 'Waiting'), ('confirmed','confirmed'),('approved','approved'),('except_ship','Shipping Exception'),('except_invoice','Invoice Exception'),('done','done'),('cancel','cancel')], 'Order State', readonly=True),
+		'state': fields.selection([('draft', 'Draft'), ('wait', 'Waiting'), ('confirmed', 'Confirmed'), ('approved', 'Approved'),('except_ship', 'Shipping Exception'), ('except_invoice', 'Invoice Exception'), ('done', 'Done'), ('cancel', 'Cancelled')], 'Order State', readonly=True, help="The state of the purchase order or the quotation request. A quotation is a purchase order in a 'Draft' state. Then the order has to be confirmed by the user, the state switch to 'Confirmed'. Then the supplier must confirm the order to change the state to 'Approved'. When the purchase order is paid and received, the state becomes 'Done'. If a cancel action occurs in the invoice or in the reception of goods, the state becomes in exception."),
 		'order_line': fields.one2many('purchase.order.line', 'order_id', 'Order State', states={'confirmed':[('readonly',True)], 'approved':[('readonly',True)]}),
 		'validator' : fields.many2one('res.users', 'Validated by', readonly=True),
 		'notes': fields.text('Notes'),
@@ -120,7 +117,7 @@ class purchase_order(osv.osv):
 		if not adr_id:
 			return {}
 		part_id = self.pool.get('res.partner.address').read(cr, uid, [adr_id], ['partner_id'])[0]['partner_id'][0]
-		loc_id = ir.ir_get(cr,uid,'meta','stock.lot.customer',[('res.partner',part_id)])[0][2]
+		loc_id = self.pool.get('res.partner').browse(cr, uid, part_id).property_stock_customer[0]
 		return {'value':{'location_id': loc_id, 'warehouse_id': False}}
 
 	def onchange_warehouse_id(self, cr, uid, ids, warehouse_id):
@@ -133,11 +130,11 @@ class purchase_order(osv.osv):
 		if not part:
 			return {'value':{'partner_address_id': False}}
 		addr = self.pool.get('res.partner').address_get(cr, uid, [part], ['contact'])
-		pricelist=ir.ir_get(cr,uid,'meta','product.pricelist.purchase',[('res.partner',part)])[0][2]
+		pricelist = self.pool.get('res.partner').browse(cr, uid, part).property_product_pricelist_purchase[0]
 		return {'value':{'partner_address_id': addr['contact'], 'pricelist_id': pricelist}}
 
 	def wkf_approve_order(self, cr, uid, ids):
-		self.write(cr, uid, ids, {'state':'approved', 'date_approve':time.strftime('%Y-%m-%d')})
+		self.write(cr, uid, ids, {'state': 'approved', 'date_approve': time.strftime('%Y-%m-%d')})
 		return True
 
 	def wkf_confirm_order(self, cr, uid, ids, context={}):
@@ -168,26 +165,32 @@ class purchase_order(osv.osv):
 					   'ref_doc1': 'purchase.order,%d' % (po.id,),
 					   })
 
-	def action_invoice_create(self,cr, uid, ids, *args):
+	def action_invoice_create(self, cr, uid, ids, *args):
 		res = False
-		for o in self.browse(cr,uid,ids):
-			il=[]
+		for o in self.browse(cr, uid, ids):
+			il = []
 			for ol in o.order_line:
-				opt=[('id',str(ol.product_id.id or -1)),('uid',str(uid))]
 
-				place = []
-				place.append( ('product.product',ol.product_id.id) )
-				if ol.product_id.categ_id.id:
-					place.append( ('product.category', ol.product_id.categ_id.id) )
+				if ol.product_id:
+					a = ol.product_id.product_tmpl_id.property_account_expense
+					if not a:
+						a = ol.product_id.categ_id.property_account_expense_categ[0]
+					else:
+						a = a[0]
+				else:
+					a = self.pool.get('ir.property').get(cr, uid, 'property_account_expense_categ', 'product.category')
+				il.append((0, False, {
+					'name': ol.name,
+					'account_id': a,
+					'price_unit': ol.price_unit or 0.0,
+					'quantity': ol.product_qty,
+					'invoice_line_tax_id': [(6, 0, [x.id for x in ol.taxes_id])]
+				}))
 
-				a=ir.ir_get(cr,uid,'meta','account.expense',place)[0][2]
-				il.append( (0,False,{'name':ol.name, 'account_id':a, 'price_unit':ol.price_unit or 0.0, 'quantity':ol.product_qty, 'invoice_line_tax_id':[(6,0,[x.id for x in ol.taxes_id])] }) )
-
-			opt=[('id',str(o.partner_id.id or -1)), ('uid',str(uid))]
-			a=ir.ir_get(cr,uid,'meta','account.payable',[('res.partner',o.partner_id.id or False)])[0][2]
-			inv={
+			a = o.partner_id.property_account_payable[0]
+			inv = {
 				'name': o.name,
-				'reference': "P%dPO%d"%(o.partner_id.id,o.id),
+				'reference': "P%dPO%d" % (o.partner_id.id, o.id),
 				'account_id': a,
 				'type': 'in_invoice',
 				'partner_id': o.partner_id.id,
@@ -199,7 +202,7 @@ class purchase_order(osv.osv):
 			}
 			inv_id = self.pool.get('account.invoice').create(cr, uid, inv)
 
-			self.write(cr,uid,[o.id],{'invoice_id':inv_id})
+			self.write(cr, uid, [o.id], {'invoice_id': inv_id})
 			res = inv_id
 		return res
 
@@ -212,16 +215,16 @@ class purchase_order(osv.osv):
 
 	def action_picking_create(self,cr, uid, ids, *args):
 		picking_id = False
-		orders = self.browse(cr, uid, ids)
-		for order in orders:
-			loc_id = ir.ir_get(cr,uid,'meta','stock.lot.supplier',[('res.partner',order.partner_id.id)])[0][2]
+		for order in self.browse(cr, uid, ids):
+			loc_id = order.partner_id.property_stock_supplier[0]
 			picking_id = self.pool.get('stock.picking').create(cr, uid, {
-				'name': 'PO:'+order.name,
-				'origin': 'PO:'+str(order.id),
+				'origin': 'PO:%d:%s' % (order.id, order.name),
 				'type': 'in',
 				'address_id': order.dest_address_id.id or order.partner_address_id.id,
 			})
 			for order_line in order.order_line:
+				if not order_line.product_id:
+					continue
 				if order_line.product_id.product_tmpl_id.type=='product':
 					dest = order.location_id.id
 					self.pool.get('stock.move').create(cr, uid, {
@@ -244,6 +247,19 @@ class purchase_order(osv.osv):
 			wf_service = netsvc.LocalService("workflow")
 			wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
 		return picking_id
+	def copy(self, cr, uid, id, default=None,context={}):
+		if not default:
+			default = {}
+		default.update({
+			'state':'draft',
+			'shipped':False,
+			'invoiced':False,
+			'invoice_id':False,
+			'picking_id':False,
+			'name': self.pool.get('ir.sequence').get(cr, uid, 'purchase.order'),
+		})
+		return super(purchase_order, self).copy(cr, uid, id, default, context)
+
 purchase_order()
 
 class purchase_order_line(osv.osv):
@@ -273,6 +289,12 @@ class purchase_order_line(osv.osv):
 	_table = 'purchase_order_line'
 	_name = 'purchase.order.line'
 	_description = 'Purchase Order line'
+	def copy(self, cr, uid, id, default=None,context={}):
+		if not default:
+			default = {}
+		default.update({'state':'draft', 'move_id':False})
+		return super(purchase_order_line, self).copy(cr, uid, id, default, context)
+
 	def product_id_change(self, cr, uid, ids, pricelist, product, qty, uom):
 		if not pricelist:
 			raise osv.except_osv('No Pricelist !', 'You have to select a pricelist in the sale form !\n Please set one before choosing a product.')
