@@ -29,7 +29,7 @@
 
 import time
 import netsvc
-from osv import fields,osv,orm
+from osv import fields, osv
 import ir
 from mx import DateTime
 
@@ -38,7 +38,7 @@ class sale_shop(osv.osv):
 	_description = "Sale Shop"
 	_columns = {
 		'name': fields.char('Shop name',size=64, required=True),
-		'payment_default_id': fields.many2one('account.account','Default Payment',required=True),
+		'payment_default_id': fields.many2one('account.payment.term','Default Payment Term',required=True),
 		'payment_account_id': fields.many2many('account.account','sale_shop_account','shop_id','account_id','Payment accounts'),
 		'warehouse_id': fields.many2one('stock.warehouse','Warehouse'),
 		'pricelist_id': fields.many2one('product.pricelist', 'Pricelist'),
@@ -56,36 +56,43 @@ class sale_order(osv.osv):
 	def copy(self, cr, uid, id, default=None,context={}):
 		if not default:
 			default = {}
-		default.update( {'state':'draft','shipped':False, 'invoiced':False, 'invoice_ids': [], 'picking_ids':[] })
+		default.update({
+			'state':'draft',
+			'shipped':False,
+			'invoiced':False,
+			'invoice_ids':[],
+			'picking_ids':[],
+			'name': self.pool.get('ir.sequence').get(cr, uid, 'sale.order'),
+		})
 		return super(sale_order, self).copy(cr, uid, id, default, context)
 
-	def _amount_untaxed(self, cr, uid, ids, prop, unknow_none,unknow_dict):
-		id_set=",".join(map(str,ids))
+	def _amount_untaxed(self, cr, uid, ids, field_name, arg, context):
+		id_set = ",".join(map(str, ids))
 		cr.execute("SELECT s.id,COALESCE(SUM(l.price_unit*l.product_uos_qty),0) AS amount FROM sale_order s LEFT OUTER JOIN sale_order_line l ON (s.id=l.order_id) WHERE s.id IN ("+id_set+") GROUP BY s.id ")
-		res=dict(cr.fetchall())
+		res = dict(cr.fetchall())
 		return res
 
-	def _amount_tax(self, cr, uid, ids, prop, unknow_none,unknow_dict):
+	def _amount_tax(self, cr, uid, ids, field_name, arg, context):
 		res = {}
 		for order in self.browse(cr, uid, ids):
 			val = 0.0
 			for line in order.order_line:
 				for tax in line.tax_id:
-					for c in self.pool.get('account.tax').compute(cr, uid, [tax.id], line.price_unit, line.product_uom_qty):
-						val+=c['amount']
+					for c in self.pool.get('account.tax').compute(cr, uid, [tax.id], line.price_unit, line.product_uom_qty, order.partner_invoice_id.id):
+						val += c['amount']
 			res[order.id]=val
 		return res
 
-	def _amount_total(self, cr, uid, ids, prop, unknow_none,unknow_dict):
+	def _amount_total(self, cr, uid, ids, field_name, arg, context):
 		res = {}
-		untax = self._amount_untaxed(cr, uid, ids, prop, unknow_none,unknow_dict) 
-		tax = self._amount_tax(cr, uid, ids, prop, unknow_none,unknow_dict)
+		untax = self._amount_untaxed(cr, uid, ids, field_name, arg, context) 
+		tax = self._amount_tax(cr, uid, ids, field_name, arg, context)
 		for id in ids:
-			res[id] = untax.get(id,0) + tax.get(id,0)
+			res[id] = untax.get(id, 0.0) + tax.get(id, 0.0)
 		return res
 
 	_columns = {
-		'name': fields.char('Order Description',size=64, required=True),
+		'name': fields.char('Order Description', size=64, required=True),
 		'shop_id':fields.many2one('sale.shop', 'Shop', required=True, readonly=True, states={'draft':[('readonly',False)]}),
 		'origin': fields.char('Origin', size=64),
 		'client_order_ref': fields.char('Order Reference',size=64),
@@ -99,32 +106,30 @@ class sale_order(osv.osv):
 			('invoice_except','Invoice Exception'),
 			('done','Done'),
 			('cancel','Cancel')
-		], 'Order State', readonly=True),
-
+		], 'Order State', readonly=True, help="Gives the state of the quotation or sale order. The exception state are automatically setted when a cancel operation occurs in the invoice validation (Invoice Exception) or in picking list process (Shipping Exception). The 'Waiting Schedule' state is set when the invoice is confirmed but waiting the 'Date Order' the schedule."),
 		'date_order':fields.date('Date Ordered', required=True, readonly=True, states={'draft':[('readonly',False)]}),
 
 		'user_id':fields.many2one('res.users', 'Salesman', states={'draft':[('readonly',False)]}, relate=True),
 		'partner_id':fields.many2one('res.partner', 'Partner', readonly=True, states={'draft':[('readonly',False)]}, change_default=True, relate=True),
 		'partner_invoice_id':fields.many2one('res.partner.address', 'Invoice Address', readonly=True, required=True, states={'draft':[('readonly',False)]}),
-		'partner_order_id':fields.many2one('res.partner.address', 'Ordering Address', readonly=True, required=True, states={'draft':[('readonly',False)]}),
+		'partner_order_id':fields.many2one('res.partner.address', 'Ordering Contact', readonly=True, required=True, states={'draft':[('readonly',False)]}, help="The name and address of the contact that requested the order or quotation."),
 		'partner_shipping_id':fields.many2one('res.partner.address', 'Shipping Address', readonly=True, required=True, states={'draft':[('readonly',False)]}),
 
 		'incoterm': fields.selection(_incoterm_get, 'Incoterm',size=3),
 		'picking_policy': fields.selection([('direct','Direct Delivery'),('one','All at once')], 'Picking Policy', required=True ),
-		'payment_term': fields.selection((('no','No'), ('month','2% 10 Net 30')), 'Payment Term'),
 		'order_policy': fields.selection([
 			('prepaid','Pay before delivery'),
 			('manual','Shipping & Manual Invoice'),
 			('postpaid','Invoice after delivery'),
-		], 'Shipping Policy', required=True, readonly=True, states={'draft':[('readonly',False)]}),
+		], 'Shipping Policy', required=True, readonly=True, states={'draft':[('readonly',False)]}, help="The Shipping Policy is used to synchronise invoive and delivery operations. The 'Pay before delivery' choice will first generate the invoice and then generate the picking order after the payment of this invoice. The 'Shipping & Manual Invoice' will create the picking order directly and wait the user to manually click on the 'Invoice Button' to generate the draft invoice. The 'Invoice after delivery' choice will generate the draft invoice after the picking list have been finnished"),
 		'pricelist_id':fields.many2one('product.pricelist', 'Pricelist', required=True, readonly=True, states={'draft':[('readonly',False)]}),
-		'project_id':fields.many2one('account.analytic.account', 'Profit/Cost Center', readonly=True, states={'draft':[('readonly',False)]}),
+		'project_id':fields.many2one('account.analytic.account', 'Profit/Cost Center', readonly=False, states={'draft':[('done', True)]}),
 
 		'order_line': fields.one2many('sale.order.line', 'order_id', 'Order Lines', readonly=True, states={'draft':[('readonly',False)]}),
 		'payment_line': fields.one2many('sale.order.payment', 'order_id', 'Order Payments', readonly=True, states={'draft':[('readonly',False)]}),
 
-		'invoice_ids': fields.many2many('account.invoice', 'sale_order_invoice_rel', 'order_id', 'invoice_id', 'Invoice', readonly=True),
-		'picking_ids': fields.many2many('stock.picking', 'sale_order_picking_rel', 'order_id','picking_id', 'Picking List', readonly=True),
+		'invoice_ids': fields.many2many('account.invoice', 'sale_order_invoice_rel', 'order_id', 'invoice_id', 'Invoice', readonly=True, help="This is the list of invoices that have been generated for this sale order. The same sale order may have been invoiced in several times (by line for example)."),
+		'picking_ids': fields.many2many('stock.picking', 'sale_order_picking_rel', 'order_id','picking_id', 'Picking List', readonly=True, help="This is the list of picking list that have been generated for this invoice"),
 
 		'shipped':fields.boolean('Picked', readonly=True),
 		'invoiced':fields.boolean('Paid', readonly=True),
@@ -134,7 +139,7 @@ class sale_order(osv.osv):
 		'amount_untaxed': fields.function(_amount_untaxed, method=True, string='Untaxed Amount'),
 		'amount_tax': fields.function(_amount_tax, method=True, string='Taxes'),
 		'amount_total': fields.function(_amount_total, method=True, string='Total'),
-		'invoice_quantity': fields.selection([('order','Ordered Quantities'),('procurement','Shipped Quantities')], 'Invoice on'),
+		'invoice_quantity': fields.selection([('order','Ordered Quantities'),('procurement','Shipped Quantities')], 'Invoice on', help="The sale order will automatically create the invoice proposition (draft invoice). Ordered and delivered quantities may not be the same. You have to choose if you invoice based on ordered or shipped quantities. If the product is a service, shipped quantities means hours spent on the associated tasks"),
 	}
 	_defaults = {
 		'picking_policy': lambda *a: 'direct',
@@ -155,11 +160,16 @@ class sale_order(osv.osv):
 			# Que faire si le client a une pricelist a lui ?
 			if shop.pricelist_id.id:
 				v['pricelist_id']=shop.pricelist_id.id
-			v['payment_default_id']=shop.payment_default_id.id
+			#v['payment_default_id']=shop.payment_default_id.id
 		return {'value':v}
 
 	def action_cancel_draft(self, cr, uid, ids, *args):
-		self.write(cr, uid, ids, {'state':'draft'})
+		if not len(ids):
+			return False
+		cr.execute('select id from sale_order_line where order_id in ('+','.join(map(str, ids))+')', ('draft',))
+		line_ids = map(lambda x: x[0], cr.fetchall())
+		self.write(cr, uid, ids, {'state':'draft', 'invoice_ids':[(6,0,[])], 'shipped':0, 'invoiced':0, 'picking_ids':[(6,0,[])]})
+		self.pool.get('sale.order.line').write(cr, uid, line_ids, {'invoiced':False, 'state':'draft', 'invoice_lines':[(6,0,[])]})
 		wf_service = netsvc.LocalService("workflow")
 		for inv_id in ids:
 			wf_service.trg_create(uid, 'sale.order', inv_id, cr)
@@ -169,18 +179,23 @@ class sale_order(osv.osv):
 		if not part:
 			return {'value':{'partner_invoice_id': False, 'partner_shipping_id':False, 'partner_order_id':False}}
 		addr = self.pool.get('res.partner').address_get(cr, uid, [part], ['delivery','invoice','contact'])
-		pricelist=ir.ir_get(cr,uid, 'meta','product.pricelist',[('res.partner',part)])[0][2]
+		pricelist = self.pool.get('res.partner').browse(cr, uid, part).property_product_pricelist[0]
 		return {'value':{'partner_invoice_id': addr['invoice'], 'partner_order_id':addr['contact'], 'partner_shipping_id':addr['delivery'], 'pricelist_id': pricelist}}
 
 	def button_dummy(self, cr, uid, ids, context={}):
 		return True
 
+#FIXME: the method should return the list of invoices created (invoice_ids) 
+# and not the id of the last invoice created (res). The problem is that we 
+# cannot change it directly since the method is called by the sale order
+# workflow and I suppose it expects a single id...
 	def action_invoice_create(self, cr, uid, ids, grouped=False, states=['confirmed','done']):
 		res = False
-		factures = {}
+		invoices = {}
+		invoice_ids = []
 
 		def make_invoice(order, lines):
-			a = ir.ir_get(cr,uid,'meta','account.receivable',[('res.partner',o.partner_id.id)])[0][2]
+			a = order.partner_id.property_account_receivable[0]
 			inv = {
 				'name': order.name,
 				'origin': 'SO:'+str(order.id)+':'+order.name,
@@ -206,9 +221,9 @@ class sale_order(osv.osv):
 					lines.append(line.id)
 			created_lines = self.pool.get('sale.order.line').invoice_line_create(cr, uid, lines)
 			if created_lines:
-				factures.setdefault(o.partner_id.id, []).append((o, created_lines))
+				invoices.setdefault(o.partner_id.id, []).append((o, created_lines))
 
-		for val in factures.values():
+		for val in invoices.values():
 			if grouped:
 				res = make_invoice(val[0][0], reduce(lambda x,y: x + y, [l for o,l in val], []))
 				for o,l in val:
@@ -217,6 +232,7 @@ class sale_order(osv.osv):
 			else:
 				for order, il in val:
 					res = make_invoice(order, il)
+					invoice_ids.append(res)
 					self.write(cr, uid, [order.id], {'state' : 'progress'})
 					cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%d,%d)', (o.id, res))
 		return res
@@ -274,7 +290,7 @@ class sale_order(osv.osv):
 			self.pool.get('sale.order.line').write(cr, uid, write_done_ids, {'state': 'done'})
 		if write_cancel_ids:
 			self.pool.get('sale.order.line').write(cr, uid, write_cancel_ids, {'state': 'cancel'})
-			
+
 		if mode=='finished':
 			return finished
 		elif mode=='canceled':
@@ -292,11 +308,10 @@ class sale_order(osv.osv):
 				if line.product_id and line.product_id.product_tmpl_id.type=='product':
 					location_id = order.shop_id.warehouse_id.lot_stock_id.id
 					if not picking_id:
-						loc_dest_id = ir.ir_get(cr,uid,'meta','stock.lot.customer',[('res.partner',order.partner_id.id)])[0][2]
+						loc_dest_id = order.partner_id.property_stock_customer[0]
 						picking_id = self.pool.get('stock.picking').create(cr, uid, {
-							'name':'SO:'+order.name,
-							'origin':'SO:'+str(order.id)+':'+order.name,
-							'type':'out',
+							'origin': 'SO:%d:%s' % (order.id, order.name),
+							'type': 'out',
 							'state': 'auto',
 							'move_type': order.picking_policy,
 							'loc_move_id': loc_dest_id,
@@ -426,8 +441,8 @@ class sale_order_line(osv.osv):
 	_name = 'sale.order.line'
 	_description = 'Sale Order line'
 	_columns = {
-		'order_id': fields.many2one('sale.order', 'Order Ref', required=True),
-		'name': fields.char('Description', size=64, required=True),
+		'order_id': fields.many2one('sale.order', 'Order Ref', required=True, ondelete='cascade'),
+		'name': fields.char('Description', size=256, required=True),
 		'sequence': fields.integer('Sequence'),
 		'date_planned': fields.date('Date Promised', required=True),
 		'product_id': fields.many2one('product.product', 'Product', domain=[('sale_ok','=',True)], change_default=True),
@@ -442,9 +457,9 @@ class sale_order_line(osv.osv):
 		'address_allotment_id' : fields.many2one('res.partner.address', 'Allotment Partner'),
 		'product_uom_qty': fields.float('Quantity (UOM)', digits=(16,2), required=True),
 		'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
-		'product_uos_qty' : fields.float('Quantity (UOS)'),
-		'product_uos' : fields.many2one('product.uom', 'Product UOS'),
-		'product_packaging' : fields.many2one('product.packaging', 'Packaging used'),
+		'product_uos_qty': fields.float('Quantity (UOS)'),
+		'product_uos': fields.many2one('product.uom', 'Product UOS'),
+		'product_packaging': fields.many2one('product.packaging', 'Packaging used'),
 		'move_ids': fields.one2many('stock.move', 'sale_line_id', 'Inventory Moves', readonly=True),
 		'property_ids': fields.many2many('mrp.property', 'sale_order_line_property_rel', 'order_id', 'property_id', 'Properties'),
 		'number_packages': fields.function(_number_packages, method=True, type='integer', string='Number packages'),
@@ -473,10 +488,15 @@ class sale_order_line(osv.osv):
 		create_ids = []
 		for line in self.browse(cr, uid, ids, context):
 			if not line.invoiced:
-				place = [ ('product.product',line.product_id.id) ]
-				if line.product_id and line.product_id.categ_id:
-					place.append( ('product.category', line.product_id.categ_id.id) )
-				a = ir.ir_get(cr,uid,'meta','account.income', place)[0][2]
+				if line.product_id:
+					print line.product_id.product_tmpl_id.id
+					a =  line.product_id.product_tmpl_id.property_account_income
+					if not a:
+						a = line.product_id.categ_id.property_account_income_categ[0]
+					else:
+						a = a[0]
+				else:
+					a = self.pool.get('ir.property').get(cr, uid, 'property_account_income_categ', 'product.category', context=context)
 				uosqty = _get_line_qty(line)
 				uos_id = line.product_uos and line.product_uos.accounting_uos_id
 				uos_id = (uos_id and uos_id.id) or False
@@ -499,16 +519,9 @@ class sale_order_line(osv.osv):
 	def button_done(self, cr, uid, ids, context={}):
 		return self.write(cr, uid, ids, {'state':'done'})
 
-	def uom_change(self, cr, uid, ids, product_uom, product_uom_qty=0, product_id=None):
-		if not product_id:
-			return {'value': {'product_uos': False, 'product_uos_qty': 0, 'weight' : 0}, 'domain':{}}
-		res = self.pool.get('product.product').read(cr, uid, [product_id], ['uom_id', 'uos_id', 'uos_coeff', 'weight'])[0]
-		q = self.pool.get('product.uom')._compute_qty(cr, uid, product_uom, product_uom_qty, res['uom_id'][0])
-		return {'value' : {'product_uos' : res['uos_id'], 'product_uos_qty' : q * res['uos_coeff'], 'weight' : q * res['weight']}}
-
 	def uos_change(self, cr, uid, ids, product_uos, product_uos_qty=0, product_id=None):
 		if not product_id:
-			return {'value': {'product_uom': False, 'product_uom_qty': 0}, 'domain':{}}
+			return {'value': {'product_uom': product_uos, 'product_uom_qty': product_uos_qty}, 'domain':{}}
 		res = self.pool.get('product.product').read(cr, uid, [product_id], ['uom_id', 'uos_id', 'uos_coeff', 'weight'])[0]
 		value = {
 			'product_uom' : res['uom_id'], 
@@ -521,20 +534,24 @@ class sale_order_line(osv.osv):
 		except ZeroDivisionError:
 			pass
 		return {'value' : value}
-	
+
+	def copy(self, cr, uid, id, default=None,context={}):
+		if not default:
+			default = {}
+		default.update({'state':'draft', 'move_ids':[], 'invoiced':False, 'invoice_lines':[]})
+		return super(sale_order_line, self).copy(cr, uid, id, default, context)
+
 	def product_id_change(self, cr, uid, ids, pricelist, product, qty=0, uom=False, qty_uos=0, uos=False, name=''):
 		if not product:
 			return {'value': {'price_unit': 0.0, 'notes':'', 'weight' : 0}, 'domain':{'product_uom':[]}}
 		if not pricelist:
 			raise osv.except_osv('No Pricelist !', 'You have to select a pricelist in the sale form !\nPlease set one before choosing a product.')
 		price = self.pool.get('product.pricelist').price_get(cr,uid,[pricelist], product, qty or 1.0)[pricelist]
-		if not price:
-			raise osv.except_osv('Product not sellable !', 'You can not sell this product !\nYou have to change either the product, the quantity or the pricelist.')
 		res = self.pool.get('product.product').read(cr, uid, [product])[0]
 		dt = (DateTime.now() + DateTime.RelativeDateTime(days=res['sale_delay'] or 0.0)).strftime('%Y-%m-%d')
-		result = {'price_unit': price, 'name':res['name'], 'tax_id':res['taxes_id'], 'type':res['procure_method'], 'date_planned':dt, 'notes':res['description_sale']}
-		if res['code'] and not name:
-			result['name'] = res['code']
+		result = {'price_unit': price, 'tax_id':res['taxes_id'], 'type':res['procure_method'], 'date_planned':dt, 'notes':res['description_sale']}
+		if not name:
+			result['name'] = res['name']
 		domain = {}
 		if not uom and not uos:
 			result['product_uom'] = res['uom_id'] and res['uom_id'][0]
@@ -545,13 +562,13 @@ class sale_order_line(osv.osv):
 				res2 = self.pool.get('product.uom').read(cr, uid, [result['product_uom']], ['category_id'])
 				if res2 and res2[0]['category_id']:
 					domain = {'product_uom':[('category_id','=',res2[0]['category_id'][0])]}
-		elif uom:
+		elif uom: # whether uos is set or not
 			default_uom = res['uom_id'] and res['uom_id'][0]
 			q = self.pool.get('product.uom')._compute_qty(cr, uid, uom, qty, default_uom)
 			result['product_uos'] = res['uos_id']
 			result['product_uos_qty'] = q * res['uos_coeff']
 			result['weight'] = q * res['weight']
-		elif uos:
+		elif uos: # only happens if uom is False
 			result['product_uom'] = res['uom_id'] and res['uom_id'][0]
 			result['product_uom_qty'] = qty_uos / res['uos_coeff']
 			result['weight'] = result['product_uom_qty'] * res['weight']
