@@ -36,7 +36,9 @@ from osv.osv import osv_pools
 
 sale_form = """<?xml version="1.0"?>
 <form string="Make an order">
+	<separator string="Create a sale order from this opportunity"/>
 	<field name="name" required="True" />
+	<label string="(empty for default value)" colspan="2"/>
 	<field name="shop_id" required="True" />
 	<field name="partner_id" required="True" />
 	<field name="state" required="True" />
@@ -71,8 +73,7 @@ class make_sale(wizard.interface):
 		pricelist = ir.ir_get(cr, uid,
 			[('meta','res.partner'), ('name','product.pricelist')],
 			[('id',str(data['form']['partner_id'])), ('uid',str(uid))])[0][2]
-		nid = sale.create(cr, uid, {
-			'name': data['form']['name'],
+		ddata = {
 			'origin': 'BO: %s' % str(data['ids'][0]),
 			'picking_policy': data['form']['picking_policy'],
 			'shop_id': data['form']['shop_id'],
@@ -83,7 +84,10 @@ class make_sale(wizard.interface):
 			'partner_shipping_id': partner_addr['delivery'],
 			'order_policy': 'manual',
 			'date_order': now(),
-		})
+		}
+		if data['form']['name']:
+			ddata['name'] = data['form']['name']
+		nid = sale.create(cr, uid, ddata)
 		case.write(cr, uid, data['ids'], {'ref': 'sale.order,%s' % nid})
 		return {}
 
