@@ -7,6 +7,6 @@
 	"category" : "Generic Modules/Inventory Control",
 	"init_xml" : [],
 	"demo_xml" : ["stock_demo.xml"],
-	"update_xml" : ["stock_workflow.xml", "stock_data.xml", "stock_incoterms.xml","stock_wizard.xml", "stock_view.xml", "stock_report.xml"],
+	"update_xml" : ["stock_workflow.xml", "stock_data.xml", "stock_incoterms.xml","stock_wizard.xml", "stock_view.xml", "stock_report.xml", "stock_sequence.xml"],
 	"active": True
 }
