@@ -27,17 +27,12 @@
 
 from osv.osv import osv_pools
 from report.interface import report_rml
-from report.render import render
+from report.interface import toxml
 
 import sql_db
-
-def toxml(val):
-	return val.replace('&', '&amp;').replace('<','&lt;').replace('>','&gt;').decode('utf-8').encode('latin1')
+#FIXME: we should use toxml
 
 class report_custom(report_rml):
-	def __init__(self, name, table, tmpl, xsl):
-		report_rml.__init__(self, name, table, tmpl, xsl)
-
 	def create_xml(self, uid, ids, datas, context={}):
 		cr = sql_db.db.cursor()
 		products = osv_pools.get('product.product').read(cr, uid, ids, ['name','uom_id'])
