@@ -28,7 +28,6 @@
 
 import os
 from mx import DateTime
-from mx.DateTime import now
 
 import tools, osv
 import time
@@ -78,13 +77,15 @@ class report_stock(report_int):
 		cr.execute("select sum(r.product_qty * u.factor), r.date_planned, r.product_id from stock_move r left join product_uom u on (r.product_uom=u.id) where state in ('confirmed','assigned','waiting') and location_id in ("+loc_ids_s+") and product_id in ("+prod_ids_s+") group by date_planned,product_id")
 
 		for (qty, dt, prod_id) in cr.fetchall():
-			if dt<dt_from:
-				dt=dt_from
+			if dt<=dt_from:
+				dt= (DateTime.now() + DateTime.RelativeDateTime(days=1)).strftime('%Y-%m-%d')
 			products.setdefault(prod_id, [])
 			products[prod_id].append((dt,-qty))
 
-		cr.execute("select sum(r.product_qty * u.factor), r.date_planned, r.product_id from stock_move r left join product_uom u on (r.product_uom=u.id) where date_planned>=%s and state in ('confirmed','assigned','waiting') and location_dest_id in ("+loc_ids_s+") and product_id in ("+prod_ids_s+") group by date_planned,product_id", (dt_from, ))
+		cr.execute("select sum(r.product_qty * u.factor), r.date_planned, r.product_id from stock_move r left join product_uom u on (r.product_uom=u.id) where state in ('confirmed','assigned','waiting') and location_dest_id in ("+loc_ids_s+") and product_id in ("+prod_ids_s+") group by date_planned,product_id")
 		for (qty, dt, prod_id) in cr.fetchall():
+			if dt<=dt_from:
+				dt= (DateTime.now() + DateTime.RelativeDateTime(days=1)).strftime('%Y-%m-%d')
 			products.setdefault(prod_id, [])
 			products[prod_id].append((dt,qty))
 
