@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
+# Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
 # $Id: stock.py 1005 2005-07-25 08:41:42Z nicoe $
 #
@@ -78,7 +78,7 @@ class stock_location(osv.osv):
 		'allocation_method': fields.selection([('fifo','FIFO'),('lifo','LIFO'),('nearest','Nearest')], 'Allocation Method', required=True),
 
 		'account_id': fields.many2one('account.account', string='Inventory Account', domain=[('type','=','stock_inventory')]),
-		'location_id' : fields.many2one('stock.location', 'Parent Location'),
+		'location_id': fields.many2one('stock.location', 'Parent Location'),
 		'child_ids': fields.one2many('stock.location', 'location_id', 'Contains'),
 
 		'comment': fields.text('Additional Information'),
@@ -119,46 +119,62 @@ class stock_location(osv.osv):
 					})
 		return result
 
-	def _product_get(self, cr, uid, id, product_ids=False, context={}, states=['done']):
-		states_str = ','.join(map(lambda x:"'"+x+"'", states))
+	def _product_get_multi_location(self, cr, uid, ids, product_ids=False, context={}, states=['done'], what=('in', 'out')):
+		states_str = ','.join(map(lambda s: "'%s'" % s, states))
 		if not product_ids:
 			product_ids = self.pool.get('product.product').search(cr, uid, [])
-		prod_ids_str = ','.join(map(str,product_ids))
-		cr.execute('select sum(product_qty),product_id,product_uom from stock_move where location_dest_id=%d and product_id in ('+prod_ids_str+') and state in ('+states_str+') group by product_id,product_uom', (id,))
-		results = cr.fetchall()
-		cr.execute('select -sum(product_qty),product_id,product_uom from stock_move where location_id=%d and product_id in ('+prod_ids_str+') and state in ('+states_str+') group by product_id,product_uom', (id,))
-		results += cr.fetchall()
 		res = {}
 		for id in product_ids:
 			res[id] = 0.0
-		for (amount, prod_id, prod_uom) in results:
-			amount = self.pool.get('product.uom')._compute_qty(cr, uid, prod_uom, amount, context.get('uom',False))
-			res[prod_id]+=amount
+		if not ids:
+			return res
+			
+		prod_ids_str = ','.join(map(str, product_ids))
+		location_ids_str = ','.join(map(str, ids))
+		results = []
+		if 'in' in what:
+			# all moves from a location out of the set to a location in the set
+			cr.execute(
+				'select sum(product_qty), product_id, product_uom '\
+				'from stock_move '\
+				'where location_id not in ('+location_ids_str+') '\
+				'and location_dest_id in ('+location_ids_str+') '\
+				'and product_id in ('+prod_ids_str+') '\
+				'and state in ('+states_str+') '\
+				'group by product_id,product_uom'
+			)
+			results += cr.fetchall()
+		if 'out' in what:
+			# all moves from a location in the set to a location out of the set
+			cr.execute(
+				'select -sum(product_qty), product_id, product_uom '\
+				'from stock_move '\
+				'where location_id in ('+location_ids_str+') '\
+				'and location_dest_id not in ('+location_ids_str+') '\
+				'and product_id in ('+prod_ids_str+') '\
+				'and state in ('+states_str+') '\
+				'group by product_id,product_uom'
+			)
+			results += cr.fetchall()
+		uom_obj = self.pool.get('product.uom')
+		for amount, prod_id, prod_uom in results:
+			amount = uom_obj._compute_qty(cr, uid, prod_uom, amount, context.get('uom', False))
+			res[prod_id] += amount
 		return res
+		
+	def _product_get(self, cr, uid, id, product_ids=False, context={}, states=['done']):
+		ids = id and [id] or []
+		return self._product_get_multi_location(cr, uid, ids, product_ids, context, states)
 
 	def _product_all_get(self, cr, uid, id, product_ids=False, context={}, states=['done']):
-		result = self._product_get(cr, uid, id, product_ids, context, states)
-		childs = self.read(cr, uid, [id], ['child_ids'])[0]['child_ids']
-		for child in childs:
-			result2 = self._product_all_get(cr, uid, child, product_ids, context, states)
-			for (key,val) in result2.items():
-				result.setdefault(key, 0.0)
-				result[key]+=val
-		return result
+		# build the list of ids of children of the location given by id
+		ids = id and [id] or []
+		location_ids = self.search(cr, uid, [('location_id', 'child_of', ids)])
+		return self._product_get_multi_location(cr, uid, location_ids, product_ids, context, states)
 
 	def _product_virtual_get(self, cr, uid, id, product_ids=False, context={}, states=['done']):
 		return self._product_all_get(cr, uid, id, product_ids, context, ['confirmed','waiting','assigned','done'])
 
-	#
-	# TODO: Different method for location search !
-	#
-	def _location_order_get(self, cr, uid, ids, context):
-		ids2 = ids[:]
-		datas = self.read(cr, uid, ids, ['child_ids'])
-		for data in datas:
-			ids2 += self._location_order_get(cr, uid, data['child_ids'], context)
-		return ids2
-
 	#
 	# TODO:
 	#	 Improve this function
@@ -169,7 +185,7 @@ class stock_location(osv.osv):
 	def _product_reserve(self, cr, uid, ids, product_id, product_qty, context={}):
 		result = []
 		amount = 0.0
-		for id in self._location_order_get(cr, uid, ids, context):
+		for id in self.search(cr, uid, [('location_id', 'child_of', ids)]):
 			cr.execute("select product_uom,sum(product_qty) as product_qty from stock_move where location_dest_id=%d and product_id=%d and state='done' group by product_uom", (id,product_id))
 			results = cr.dictfetchall()
 			cr.execute("select product_uom,-sum(product_qty) as product_qty from stock_move where location_id=%d and product_id=%d and state in ('done', 'assigned') group by product_uom", (id,product_id))
@@ -222,20 +238,23 @@ class stock_move_lot(osv.osv):
 	_defaults = {
 		'active': lambda *a: 1,
 		'state': lambda *a: 'draft',
-		'date_planned': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
+		'date_planned': lambda *a: time.strftime('%Y-%m-%d'),
 	}
 	#
 	# TODO: test if valid
+	# ERROR: does this function should call action_done instead of doing him self on
+	# stock.move
 	#
 	def action_move(self, cr, uid, ids, context={}):
 		for move in self.browse(cr, uid, ids, context):
 			lot_remove = []
 			for m in move.lot_id.move_ids:
-				new_id = self.pool.get('stock.move').copy(cr, uid, m.id, {'location_id': m.location_dest_id.id, 'location_dest_id': move.loc_dest_id.id, 'date_moved': time.strftime('%Y-%m-%d'), 'state':'done', 'picking_id': False})
+				new_id = self.pool.get('stock.move').copy(cr, uid, m.id, {'location_id': m.location_dest_id.id, 'location_dest_id': move.loc_dest_id.id, 'date_moved': time.strftime('%Y-%m-%d'), 'picking_id': False, 'state':'draft','prodlot_id':False, 'tracking_id':False, 'lot_id': False, 'move_history_ids':[], 'move_history_ids2':[]})
+				self.pool.get('stock.move').action_done(cr, uid, [new_id], context)
 				cr.execute('insert into stock_move_history_ids (parent_id,child_id) values (%d,%d)', (m.id, new_id))
 				lot_remove.append(m.id)
 			self.pool.get('stock.move').write(cr, uid, lot_remove, {'lot_id':False})
-		self.write(cr,uid, ids, {'state':'done'})
+		self.write(cr,uid, ids, {'state':'done','date_moved':time.strftime('%Y-%m-%d')})
 		return True
 stock_move_lot()
 
@@ -261,7 +280,6 @@ class stock_tracking(osv.osv):
 		'serial': fields.char('Reference', size=64),
 		'move_ids' : fields.one2many('stock.move', 'tracking_id', 'Moves tracked'),
 		'date': fields.datetime('Date create', required=True),
-		'revisions': fields.one2many('stock.tracking.revision','tracking_id','Revisions'),
 	}
 	_defaults = {
 		'active': lambda *a: 1,
@@ -285,24 +303,6 @@ class stock_tracking(osv.osv):
 		raise 'You can not remove a lot line !'
 stock_tracking()
 
-class stock_tracking_revision(osv.osv):
-	_name = 'stock.tracking.revision'
-	_description = 'Stock Tracking Revisions'
-	_columns = {
-		'name' : fields.char('Revision name', size=64, required=True),
-		'description': fields.text('Description'),
-		'date': fields.date('Revision Date'),
-		'indice' : fields.char('Revision', size=16),
-		'author_id' : fields.many2one('res.users', 'Author'),
-		'tracking_id' : fields.many2one('stock.tracking', 'Tracking'),
-	}
-
-	_defaults = {
-		'author_id': lambda x,y,z,c: z,
-		'date': lambda *a: time.strftime('%Y-%m-%d'),
-	}
-stock_tracking_revision()
-
 #----------------------------------------------------------
 # Stock Picking
 #----------------------------------------------------------
@@ -340,6 +340,7 @@ class stock_picking(osv.osv):
 		'move_lot_id': fields.many2one('stock.move.lot', 'Moves Created')
 	}
 	_defaults = {
+		'name': lambda *a: 'Unconfirmed picking',
 		'work': lambda *a: 0,
 		'active': lambda *a: 1,
 		'state': lambda *a: 'draft',
@@ -348,10 +349,12 @@ class stock_picking(osv.osv):
 		'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
 	}
 	def action_confirm(self, cr, uid, ids, *args):
-		self.write(cr,uid,ids, {'state':'confirmed'})
+		self.write(cr, uid, ids, {'state': 'confirmed'})
 		todo = []
-		for line in self.browse(cr,uid, ids):
-			for r in line.move_lines:
+		for picking in self.browse(cr, uid, ids):
+			number = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.%s' % picking.type)
+			self.write(cr, uid, [picking.id], {'name': number})
+			for r in picking.move_lines:
 				if r.state=='draft':
 					todo.append(r.id)
 		if len(todo):
@@ -401,10 +404,10 @@ class stock_picking(osv.osv):
 				ok = ok and (move.state in ('cancel','done','assigned'))
 		return ok
 
-	def action_cancel(self, cr, uid, ids):
+	def action_cancel(self, cr, uid, ids, context={}):
 		for pick in self.browse(cr, uid, ids):
 			ids2 = [move.id for move in pick.move_lines]
-			self.pool.get('stock.move').action_cancel(cr, uid, ids2)
+			self.pool.get('stock.move').action_cancel(cr, uid, ids2, context)
 		self.write(cr,uid, ids, {'state':'cancel'})
 		return True
 
@@ -494,7 +497,7 @@ stock_production_lot_revision()
 #
 class stock_move(osv.osv):
 	def _getSSCC(self, cr, uid, context={}):
-		cr.execute('select id from stock_tracking  where create_uid=%d order by id desc limit 1', (uid,))
+		cr.execute('select id from stock_tracking where create_uid=%d order by id desc limit 1', (uid,))
 		res = cr.fetchone()
 		return (res and res[0]) or False
 	_name = "stock.move"
@@ -598,7 +601,7 @@ class stock_move(osv.osv):
 	#
 	# Cancel move => cancel others move and pickings
 	#
-	def action_cancel(self, cr, uid, ids):
+	def action_cancel(self, cr, uid, ids, context={}):
 		if not len(ids):
 			return True
 		pickings = {}
@@ -615,7 +618,11 @@ class stock_move(osv.osv):
 		for res in self.read(cr, uid, ids, ['move_dest_id']):
 			if res['move_dest_id']:
 				ids2.append(res['move_dest_id'][0])
-		self.action_cancel(cr,uid, ids2)
+
+		wf_service = netsvc.LocalService("workflow")
+		for id in ids:
+			wf_service.trg_trigger(uid, 'stock.move', id, cr)
+		self.action_cancel(cr,uid, ids2, context)
 		return True
 
 	def action_done(self, cr, uid, ids, *args):
@@ -647,9 +654,9 @@ class stock_move(osv.osv):
 #				if move.product_id.categ_id:
 #					test.append( ('product.category', move.product_id.categ_id.id) )
 #				if not acc_src:
-#					acc_src = ir.ir_get(cr,uid,'meta','account.expense',test)[0][2]
+#					acc_src = move.product_id.product_tmpl_id.property_account_expense[0]
 #				if not acc_dest:
-#					acc_dest = ir.ir_get(cr,uid,'meta','account.expense',test)[0][2]
+#					acc_dest = move.product_id.product_tmpl_id.property_account_income[0]
 #				if acc_src != acc_dest:
 #					amount = move.product_qty * move.product_id.standard_price
 #					lines = [
@@ -658,6 +665,11 @@ class stock_move(osv.osv):
 #					]
 #					self.pool.get('account.move').create(cr, uid, {'name': 'Stock', 'type':'undefined', 'line_id': lines})
 		self.write(cr, uid, ids, {'state':'done'})
+
+		wf_service = netsvc.LocalService("workflow")
+		for id in ids:
+			wf_service.trg_trigger(uid, 'stock.move', id, cr)
+
 		return True
 stock_move()
 
@@ -689,7 +701,7 @@ class stock_inventory(osv.osv):
 				amount=self.pool.get('stock.location')._product_get(cr, uid, line.location_id.id, [pid], {'uom': line.product_uom.id})[pid]
 				change=line.product_qty-amount
 				if change:
-					location_id = ir.ir_get(cr,uid,'meta','stock.lot.inventory',[('product.product', pid)])[0][2]
+					location_id = line.product_id.product_tmpl_id.property_stock_inventory[0]
 					value = {
 						'name': 'INV:'+str(line.inventory_id.id)+':'+line.inventory_id.name,
 						'product_id': line.product_id.id,
@@ -718,7 +730,7 @@ class stock_inventory(osv.osv):
 
 	def action_cancel(self, cr, uid, ids, context={}):
 		for inv in self.browse(cr,uid,ids):
-			self.pool.get('stock.move').action_cancel(cr, uid, [x.id for x in inv.move_ids])
+			self.pool.get('stock.move').action_cancel(cr, uid, [x.id for x in inv.move_ids], context)
 			self.write(cr, uid, [inv.id], {'state':'draft'})
 		return True
 stock_inventory()
