@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2005-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: wizard_partial_picking.py 2871 2006-04-25 14:08:22Z ged $
+# $Id: wizard_partial_picking.py 4058 2006-09-06 13:50:25Z pinky $
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -34,10 +34,12 @@ from tools.misc import UpdateableStr
 import wizard
 from osv import osv
 
-#FIXME: this is not concurrency-safe !!!!!!!!!!
 _moves_arch = UpdateableStr()
 _moves_fields = {}
 
+_moves_arch_end = '''<?xml version="1.0"?><form string="Picking Result"><label string="The picking has been successfully made !" colspan="4" /></form>'''
+_moves_fields_end = {}
+
 def make_default(val):
 	def fct(uid, data, state):
 		return val
@@ -54,7 +56,7 @@ def _get_moves(self, cr, uid, data, context):
 		if m.state<>'assigned':
 			quantity = 0
 		_moves_arch_lst.append('<field name="move%s" />\n<newline />' % (m.id,))
-		_moves_fields['move%s' % m.id] = {'string' : m.product_id.name, 'type' : 'float', 'required' : True, 'default' : make_default(quantity)}
+		_moves_fields['move%s' % m.id] = {'string' : '%s - %s' % (m.product_id.code, m.product_id.name), 'type' : 'float', 'required' : True, 'default' : make_default(quantity)}
 		res.setdefault('moves', []).append(m.id)
 	_moves_arch_lst.append('</form>')
 	_moves_arch.string = '\n'.join(_moves_arch_lst)
@@ -66,23 +68,47 @@ def _do_split(self, cr, uid, data, context):
 	pick = pick_obj.browse(cr, uid, [data['id']])[0]
 	new_picking = None
 	new_moves = []
-	# Iterate on the moves, copying them if necessary and setting them to the
-	# right quantity
-	for move in move_obj.browse(cr, uid, data['form']['moves']):
-		# Check the number of product received/sent
-		if move.product_qty != data['form']['move%s' % move.id]:
-			if not new_picking:
-				new_picking = pick_obj.copy(cr, uid, pick.id, {'name' : '%s (splitted)' % pick.name, 'move_lines' : [], 'state':'draft'})
-			new_obj = move_obj.copy(cr, uid, move.id, {'product_qty' : move.product_qty - data['form']['move%s' % move.id], 'picking_id' : new_picking, 'state': move.state})
-			new_moves.append(new_obj)
-			move_obj.write(cr, uid, [move.id], {'product_qty' : data['form']['move%s' % move.id]})
-	# We process the picking list
-	pick_obj.action_move(cr, uid, [pick.id])
-	wf_service = netsvc.LocalService("workflow")
-	wf_service.trg_validate(uid, 'stock.picking', pick.id, 'button_done', cr)
+
+#	# Iterate on the moves, copying them if necessary and setting them to the
+#	# right quantity
+#	for move in move_obj.browse(cr, uid, data['form'].get('moves',[])):
+#		# Check the number of product received/sent
+#		if move.product_qty != data['form']['move%s' % move.id]:
+#			if not new_picking:
+#				new_picking = pick_obj.copy(cr, uid, pick.id, {'name' : '%s (splitted)' % pick.name, 'move_lines' : [], 'state':'draft'})
+#			new_obj = move_obj.copy(cr, uid, move.id, {'product_qty' : data['form']['move%s' % move.id], 'picking_id' : new_picking, 'state': move.state, 'move_dest_id': False})
+#			new_moves.append(new_obj)
+#			move_obj.write(cr, uid, [move.id], {'product_qty' : move.product_qty - data['form']['move%s' % move.id]})
+	
+
+	complete, too_many, too_few = [], [], []
+	for move in move_obj.browse(cr, uid, data['form'].get('moves',[])):
+		if move.product_qty == data['form']['move%s' % move.id]:
+			complete.append(move)
+		elif move.product_qty > data['form']['move%s' % move.id]:
+			too_few.append(move)
+		else:
+			too_many.append(move)
+
+	for move in too_few:
+		if not new_picking:
+			new_picking = pick_obj.copy(cr, uid, pick.id, {'name' : '%s (splitted)' % pick.name, 'move_lines' : [], 'state':'draft'})
+		new_obj = move_obj.copy(cr, uid, move.id, {'product_qty' : data['form']['move%s' % move.id], 'picking_id' : new_picking, 'state': move.state, 'move_dest_id': False})
+		move_obj.write(cr, uid, [move.id], {'product_qty' : move.product_qty - data['form']['move%s' % move.id]})
 
+	if new_picking:
+		move_obj.write(cr, uid, [c.id for c in complete], {'picking_id': new_picking})
+		for move in too_many:
+			move_obj.write(cr, uid, [move.id], {'product_qty' : data['form']['move%s' % move.id], 'picking_id': new_picking})
+	
+	# At first we confirm the new picking (if necessary)
+	wf_service = netsvc.LocalService("workflow")
 	if new_picking:
 		wf_service.trg_validate(uid, 'stock.picking', new_picking, 'button_confirm', cr)
+	# Then we finish the good picking
+	picking_toclose = new_picking or pick.id
+	pick_obj.action_move(cr, uid, [picking_toclose])
+	wf_service.trg_validate(uid, 'stock.picking', picking_toclose, 'button_done', cr)
 	return {}
 
 class partial_picking(wizard.interface):
@@ -96,6 +122,10 @@ class partial_picking(wizard.interface):
 			'actions' : [ _do_split ],
 			'result' : { 'type' : 'state', 'state' : 'end' },
 		},
+		'end2' : {
+			'actions' : [ ],
+			'result' : { 'type' : 'form', 'arch' : _moves_arch_end, 'fields' : _moves_fields_end, 'state' : (('end', 'Close'),) },
+		},
 	}
 
 partial_picking('stock.partial_picking')
