@@ -103,7 +103,7 @@ class subscription_subscription(osv.osv):
 			self.write(cr, uid, [row['id']], {'cron_id':id, 'state':'running'})
 		return True
 
-	def model_copy(self, cr, uid, ids):
+	def model_copy(self, cr, uid, ids, context={}):
 		for row in self.read(cr, uid, ids):
 			cron_ids = [row['cron_id'][0]]
 			remaining = self.pool.get('ir.cron').read(cr, uid, cron_ids, ['numbercall'])[0]['numbercall']
@@ -114,23 +114,6 @@ class subscription_subscription(osv.osv):
 			except:
 				raise osv.except_osv('Wrong Source Document !', 'Please provide another source document.\nThis one does not exist !')
 
-			def model_copy(model_name, id, default={}):
-				model = self.pool.get(model_name)
-				datas = model.read(cr, uid, [id])[0]
-				fields = model.fields_get(cr, uid)
-				for f in fields:
-					if f in default:
-						datas[f]=default[f]
-					elif fields[f]['type']=='function':
-						del datas[f]
-					elif fields[f]['type']=='many2one':
-						datas[f]=datas[f] and datas[f][0]
-					elif fields[f]['type']=='one2many':
-						datas[f] = map(lambda id: (4,model_copy(fields[f]['relation'], id)), datas[f])
-				del datas['id']
-				id = self.pool.get(model_name).create(cr, uid, datas)
-				return id
-
 			default = {'state':'draft'}
 			doc_obj = self.pool.get('subscription.document')
 			document_ids = doc_obj.search(cr, uid, [('model','=',model_name)])
@@ -148,7 +131,7 @@ class subscription_subscription(osv.osv):
 			# the subscription is over and we mark it as being done
 			if remaining == 1:
 				state = 'done'
-			id = model_copy(model_name, id, default)
+			id = self.pool.get(model_name).copy(cr, uid, id, default, context)
 			self.pool.get('subscription.subscription.history').create(cr, uid, {'subscription_id': row['id'], 'name':row['name'], 'date':time.strftime('%Y-%m-%d %H:%M:%S'), 'document_id': model_name+','+str(id)})
 			self.write(cr, uid, [row['id']], {'state':state})
 		return True
