@@ -8,6 +8,12 @@ import SocketServer
 import socket
 import logging
 
+try:
+	from ssl import *
+	HAS_SSL = True
+except ImportError:
+	HAS_SSL = False
+
 _service={}
 _group={}
 _res_id=1
@@ -171,7 +177,7 @@ class xmlrpc(object):
 		def __init__(self, name):
 			self.name=name
 
-class XMLRPCRequestHandler(SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
+class GenericXMLRPCRequestHandler:
 	def _dispatch(self, method, params):
 		import traceback
 		try:
@@ -200,16 +206,36 @@ class XMLRPCRequestHandler(SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
 				pdb.post_mortem(tb)
 			raise xmlrpclib.Fault(1,s)
 
+class SimpleXMLRPCRequestHandler(GenericXMLRPCRequestHandler, SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
+	pass
+
+if HAS_SSL:
+	class SecureXMLRPCRequestHandler(GenericXMLRPCRequestHandler, SecureXMLRPCServer.SecureXMLRPCRequestHandler):
+		pass
+else:
+	pass
+
 class SimpleThreadedXMLRPCServer(SocketServer.ThreadingMixIn, SimpleXMLRPCServer.SimpleXMLRPCServer):
 	def server_bind(self):
 		self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 		SimpleXMLRPCServer.SimpleXMLRPCServer.server_bind(self)
 
+if HAS_SSL:
+	class SecureThreadedXMLRPCServer(SocketServer.ThreadingMixIn, SecureXMLRPCServer.SecureXMLRPCServer):
+		def server_bind(self):
+			self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+			SecureXMLRPCServer.SecureXMLRPCServer.server_bind(self)
+else:
+	pass
+
 class HttpDaemon(object):
-	def __init__(self, interface,port):
+	def __init__(self, interface,port, secure=False):
 		self.__port=port
 		self.__interface=interface
-		self.server = SimpleThreadedXMLRPCServer((interface, port), XMLRPCRequestHandler,0)
+		if secure and HAS_SSL:
+			self.server = SecureThreadedXMLRPCServer((interface, port), SecureXMLRPCRequestHandler,0)
+		else:
+			self.server = SimpleThreadedXMLRPCServer((interface, port), SimpleXMLRPCRequestHandler,0)
 
 	def attach(self,path,gw):
 		pass
@@ -232,6 +258,17 @@ class HttpDaemon(object):
 		signal.signal(signal.SIGINT, self.handler)
 		signal.signal(signal.SIGTERM, self.handler)
 		self.server.register_introspection_functions()
+
 		self.server.serve_forever()
 
+		# If the server need to be run recursively
+		#
+		#signal.signal(signal.SIGALRM, self.my_handler)
+		#signal.alarm(6)
+		#while True:
+		#	self.server.handle_request()
+		#signal.alarm(0)          # Disable the alarm
+
 # vim:noexpandtab:
+
+
