@@ -69,6 +69,7 @@ class _column(object):
 		self.readonly = readonly
 		self.required = required
 		self.size = size
+		self.help = args.get('help', '')
 		self.priority = priority
 		self.change_default = change_default
 		self.ondelete = ondelete
@@ -76,6 +77,8 @@ class _column(object):
 		self._domain = domain
 		self.relate =False
 		self._context = context
+		self.group_name = False
+		self.view_load = 0
 		for a in args:
 			if args[a]:
 				setattr(self, a, args[a])
@@ -83,10 +86,10 @@ class _column(object):
 	def set(self, cr, obj, id, name, value, user=None, context={}):
 		cr.execute('update '+obj._table+' set '+name+'='+self._symbol_set[0]+' where id=%d', (self._symbol_set[1](value),id) )
 
-	def get(self, cr, obj, ids, name, context={}):
+	def get(self, cr, obj, ids, name, context={}, values={}):
 		raise 'undefined get method !'
 
-	def search(self, cr, obj, args, name, value, offset=0, limit=2000, uid=None):
+	def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None):
 		ids = obj.search(cr, uid, args+self._domain+[(name,'ilike',value)], offset, limit)
 		res = obj.read(cr, uid, ids, [name])
 		return [x[name] for x in res]
@@ -130,7 +133,7 @@ class char(_column):
 		# we need to convert the string to a unicode object to be able 
 		# to evaluate its length (and possibly truncate it) reliably
 		if isinstance(symb, str):
-			u_symb = unicode(symb, 'utf8')	
+			u_symb = unicode(symb, 'utf8')
 		elif isinstance(symb, unicode):
 			u_symb = symb
 		else:
@@ -244,7 +247,7 @@ class one2one(_column):
 			id =cr.fetchone()[0]
 			obj.write(cr, user, [id] , act[1], context=context)
 
-	def search(self, cr, obj, args, name, value, offset=0, limit=2000, uid=None):
+	def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None):
 		return obj.pool.get(self._obj).search(cr, uid, args+self._domain+[('name','like',value)], offset, limit)
 
 
@@ -261,11 +264,12 @@ class many2one(_column):
 	# TODO: speed improvement
 	#
 	# name is the name of the relation field
-	def get(self, cr, obj, ids, name, user=None, context={}):
-		cr.execute('select id,' + name + ' from '+obj._table+' where id in (' + ','.join(map(str,ids)) + ')')
-		# build a dictionary of the form {'id_of_local_resource': value_of_relation_field, ...}
-		res = dict(cr.fetchall())
-		# distant obj
+	def get(self, cr, obj, ids, name, user=None, context={}, values={}):
+		res = {}
+		for r in values:
+			res[r['id']] = r[name]
+		for id in ids:
+			res.setdefault(id, '')
 		obj = obj.pool.get(self._obj)
 		# build a dictionary of the form {'id_of_distant_resource': name_of_distant_resource}
 		names = dict(obj.name_get(cr, user, filter(None, res.values()), context))
@@ -296,7 +300,7 @@ class many2one(_column):
 			else:
 				cr.execute('update '+obj_src._table+' set '+field+'=null where id=%d', (id,))
 
-	def search(self, cr, obj, args, name, value, offset=0, limit=2000, uid=None):
+	def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None):
 		return obj.pool.get(self._obj).search(cr, uid, args+self._domain+[('name','like',value)], offset, limit)
 
 class one2many(_column):
@@ -312,16 +316,13 @@ class one2many(_column):
 		#one2many can't be used as condition for defaults
 		assert(self.change_default != True)
 	
-	#
-	# TODO: speed improvement
-	#
-	def get(self, cr, obj, ids, name, user=None, offset=0, context={}):
+	def get(self, cr, obj, ids, name, user=None, offset=0, context={}, values={}):
 		res = {}
 		for id in ids:
 			res[id] = []
 		ids2 = obj.pool.get(self._obj).search(cr, user, [(self._fields_id,'in',ids)], limit=self._limit)
-		for r in obj.pool.get(self._obj).read(cr, user, ids2, [self._fields_id]):
-			res[r[self._fields_id][0]].append( r['id'] )
+		for r in obj.pool.get(self._obj)._read_flat(cr, user, ids2, [self._fields_id], context=context, load='_classic_write'):
+			res[r[self._fields_id]].append( r['id'] )
 		return res
 
 	def set(self, cr, obj, id, field, values, user=None, context={}):
@@ -342,12 +343,9 @@ class one2many(_column):
 			elif act[0]==5:
 				cr.execute('update '+_table+' set '+self._fields_id+'=null where '+self._fields_id+'=%d', (id,))
 			elif act[0]==6:
-				#
-				# TODO: change this in one line add remove
-				#
-				for act_nbr in act[2]:
-					cr.execute('update '+_table+' set '+self._fields_id+'=%d where id=%d', (id,act_nbr))
-	def search(self, cr, obj, args, name, value, offset=0, limit=2000, uid=None, operator='like'):
+				if len(act[2]):
+					cr.execute('update '+_table+' set '+self._fields_id+'=%d where id in ('+','.join(map(str, act[2]))+')', (id,))
+	def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None, operator='like'):
 		return obj.pool.get(self._obj).name_search(cr, uid, value, self._domain, offset, limit)
 
 #
@@ -372,7 +370,7 @@ class many2many(_column):
 		self._id2 = id2
 		self._limit = limit
 
-	def get(self, cr, obj, ids, name, user=None, offset=0, context={}):
+	def get(self, cr, obj, ids, name, user=None, offset=0, context={}, values={}):
 		res = {}
 		if not ids:
 			return res
@@ -398,7 +396,7 @@ class many2many(_column):
 			elif act[0]==3:
 				cr.execute('update '+self._rel+' set '+self._id2+'=null where ' +  self._id1 + '=%d', (act[1],))
 			elif act[0]==4:
-				cr.execute('update '+self._rel+' set '+self._id2+'=%d where ' + self._id1 + '=%d', (id,act[1]))
+				cr.execute('update '+self._rel+' set '+self._id2+'=%d where ' + self._id1 + '=%d and ' + self._id2 + '=%d', (act[1], id, act[1]))
 			elif act[0]==5:
 				cr.execute('update '+self._rel+' set '+self._id2+'=null where '+self._id2+'=%d', (id,))
 			elif act[0]==6:
@@ -409,7 +407,7 @@ class many2many(_column):
 	#
 	# TODO: use a name_search
 	#
-	def search(self, cr, obj, args, name, value, offset=0, limit=2000, uid=None, operator='like'):
+	def search(self, cr, obj, args, name, value, offset=0, limit=None, uid=None, operator='like'):
 		return obj.pool.get(self._obj).search(cr, uid, args+self._domain+[('name',operator,value)], offset, limit)
 
 # ---------------------------------------------------------
@@ -429,6 +427,8 @@ class function(_column):
 		self._fnct = fnct
 		self._fnct_inv = fnct_inv
 		self._arg = arg
+		if 'relation' in args:
+			self._obj = args['relation']
 		self._fnct_inv_arg = fnct_inv_arg
 		if not fnct_inv:
 			self.readonly = 1
@@ -440,7 +440,7 @@ class function(_column):
 			return []
 		return self._fnct_search(obj, cr, uid, obj, name, args)
 
-	def get(self, cr, obj, ids, name, user=None, context={}):
+	def get(self, cr, obj, ids, name, user=None, context={}, values={}):
 		res = {}
 		table = obj._table
 		if self._method:
@@ -451,9 +451,8 @@ class function(_column):
 
 	def set(self, cr, obj, id, name, value, user=None, context={}):
 		if self._fnct_inv:
-			table = obj._table
-			self._fnct_inv(cr, table, id, name, value, self._fnct_inv_arg)
-			
+			self._fnct_inv(obj, cr, user, id, name, value, self._fnct_inv_arg, context)
+
 # ---------------------------------------------------------
 # Serialized fields
 # ---------------------------------------------------------
@@ -466,3 +465,71 @@ class serialized(_column):
 		self._symbol_get = self._deserialize_func
 		super(serialized, self).__init__(string=string, **args)
 
+class property(function):
+	def _fnct_write(self2, self, cr, uid, id, prop, id_val, val, context={}):
+		(obj_dest,) = val
+		definition_id = self2._field_get(self, cr, uid, prop)
+
+		property = self.pool.get('ir.property')
+		nid = property.search(cr, uid, [('fields_id','=',definition_id),('res_id','=',self._name+','+str(id))])
+		while len(nid):
+			cr.execute('delete from ir_property where id=%d', (nid.pop(),))
+
+		nid = property.search(cr, uid, [('fields_id','=',definition_id),('res_id','=',False)])
+		default_val = False
+		if nid:
+			default_val = property.browse(cr, uid, nid[0], context).value
+
+		company_id = self.pool.get('res.users').company_get(cr, uid, uid)
+		res = False
+		newval = (id_val and obj_dest+','+str(id_val)) or False
+		if (newval != default_val) and newval:
+			propdef = self.pool.get('ir.model.fields').browse(cr, uid, definition_id, context=context)
+			res = property.create(cr, uid, {
+				'name': propdef.name,
+				'value': newval,
+				'res_id': self._name+','+str(id),
+				'company_id': company_id,
+				'fields_id': definition_id
+			}, context=context)
+		return res
+
+	def _fnct_read(self2, self, cr, uid, ids, prop, val, context={}):
+		property = self.pool.get('ir.property')
+		definition_id = self2._field_get(self, cr, uid, prop)
+
+		nid = property.search(cr, uid, [('fields_id','=',definition_id),('res_id','=',False)])
+		default_val = False
+		if nid:
+			d = property.browse(cr, uid, nid[0], context).value
+			default_val = (d and int(d.split(',')[1])) or False
+
+		vids = map(lambda id: self._name+','+str(id), ids)
+		nids = property.search(cr, uid, [('fields_id','=',definition_id),('res_id','in', vids)])
+
+		res = {}
+		for id in ids:
+			res[id]= default_val
+		for prop in property.browse(cr, uid, nids):
+			res[int(prop.res_id.split(',')[1])] = (prop.value and int(prop.value.split(',')[1])) or False
+
+		obj = self.pool.get(self2._obj)
+		names = dict(obj.name_get(cr, uid, filter(None, res.values()), context))
+		for r in res.keys():
+			if res[r]:
+				res[r] = (res[r], names.get(res[r], False))
+		return res
+
+	def _field_get(self, self2, cr, uid, prop):
+		if not self.field_id:
+			cr.execute('select id from ir_model_fields where name=%s and model=%s', (prop, self2._name))
+			res = cr.fetchone()
+			self.field_id = res and res[0]
+		return self.field_id
+
+	def __init__(self, obj_prop, **args):
+		self.field_id = False
+		function.__init__(self,
+		  self._fnct_read, False,
+		  self._fnct_write, (obj_prop, ),
+		  **args)
