@@ -136,7 +136,7 @@ class browse_record(object):
 		self._cache = cache
 
 	def __getitem__(self, name):
-		if name=='id':
+		if name == 'id':
 			return self._id
 		if not self._data[self._id].has_key(name):
 			# build the list of fields we will fetch
@@ -193,7 +193,21 @@ class browse_record(object):
 		return self._id
 
 	def __str__(self):
-		return "browse_record(" + self._table_name + ", " + str(self._id) + ")"
+		return "browse_record(%s, %d)" % (self._table_name, self._id)
+
+	def __eq__(self, other):
+		return (self._table_name, self._id) == (other._table_name, other._id)
+
+	def __ne__(self, other):
+		return (self._table_name, self._id) != (other._table_name, other._id)
+
+	# we need to define __unicode__ even though we've already defined __str__ 
+	# because we have overridden __getattr__
+	def __unicode__(self):
+		return unicode(str(self))
+
+	def __hash__(self):
+		return hash((self._table_name, self._id))
 
 	__repr__ = __str__
 
@@ -223,11 +237,6 @@ def get_pg_type(f):
 			f_type = ('int4', 'INTEGER')
 		else:
 			f_type = ('varchar', 'VARCHAR(%d)' % f_size)
-	elif isinstance(f, fields.function):
-		print '-'*60
-		print "WARNING: column", f.string, "is no more used, converted to a function !"
-		print "\tYou should remove this column from your database."
-		f_type = None
 	else:
 		print "WARNING: type not supported!"
 		f_type = None
@@ -246,18 +255,17 @@ class orm(object):
 	_inherits = {}
 	_sequence = None
 	_description = None
-	_protected = ['read','write','create','default_get','perm_read','perm_write','unlink','fields_get','fields_view_get','search','name_get','distinct_field_get','name_search','copy']
-
+	_protected = ['read','write','create','default_get','perm_read','perm_write','unlink','fields_get','fields_view_get','search','name_get','distinct_field_get','name_search','copy','import_data']
 	def _auto_init(self, cr):
 		create = False
 		if not hasattr(self, "_auto") or self._auto:
 			cr.execute("SELECT id FROM ir_model WHERE model='%s'" % self._name)
 			if not cr.rowcount:
 				# reference model in order to have a description of its fonctionnality in custom_report
-				cr.execute("INSERT INTO ir_model (model, name) VALUES (%s, %s)", (self._name, self._description)) 
+				cr.execute("INSERT INTO ir_model (model, name, info) VALUES (%s, %s, %s)", (self._name, self._description, self.__doc__)) 
 			cr.commit()
 
-			cr.execute("SELECT relname FROM pg_class WHERE relkind='r' AND relname='%s'" % self._table)
+			cr.execute("SELECT relname FROM pg_class WHERE relkind in ('r','v') AND relname='%s'" % self._table)
 			if not cr.rowcount:
 				cr.execute("CREATE TABLE %s(id SERIAL NOT NULL, perm_id INTEGER, PRIMARY KEY(id))" % self._table)
 				create = True
@@ -280,28 +288,40 @@ class orm(object):
 						cr.execute("ALTER TABLE %s ADD COLUMN %s %s" % 
 							(self._table, k, logs[k]))
 						cr.commit()
-#TODO: we should also iterate on the existing columns on the database (for example to drop the NOT NULL constraint)
-#DROP NULL constraint
-#ALTER TABLE products ALTER COLUMN product_no DROP NOT NULL;
+
+			# iterate on the database columns to drop the NOT NULL constraints
+			# of fields which were required but have been removed
+			cr.execute(
+				"SELECT a.attname, a.attnotnull "\
+				"FROM pg_class c, pg_attribute a "\
+				"WHERE c.oid=a.attrelid AND c.relname='%s'" % self._table)
+			db_columns = cr.dictfetchall()
+			for column in db_columns:
+				if column['attname'] not in ('id', 'oid', 'tableoid', 'ctid', 'xmin', 'xmax', 'cmin', 'cmax'):
+					if column['attnotnull'] and column['attname'] not in self._columns:
+						cr.execute("ALTER TABLE %s ALTER COLUMN %s DROP NOT NULL" % (self._table, column['attname']))
+
+			# iterate on the "object columns"
 			for k in self._columns:
 				if k in ('id','perm_id','write_uid','write_date','create_uid','create_date'):
-					raise 'Can not define a column %s. Reserved keyword !' % (k,)
+					continue
+					#raise 'Can not define a column %s. Reserved keyword !' % (k,)
 				f = self._columns[k]
 
 				cr.execute("select id from ir_model_fields where model=%s and name=%s", (self._name,k))
 				if not cr.rowcount:
 					cr.execute("select id from ir_model where model='%s'" % self._name)
 					model_id = cr.fetchone()[0]
-					cr.execute("INSERT INTO ir_model_fields (model_id, model, name, field_description, ttype, relate,relation) VALUES (%d,%s,%s,%s,%s,%s,%s)", (model_id, self._name, k, f.string.replace("'", " "), f._type, (f.relate and 'True') or 'False', f._obj or 'NULL'))
+					cr.execute("INSERT INTO ir_model_fields (model_id, model, name, field_description, ttype, relate,relation,group_name,view_load) VALUES (%d,%s,%s,%s,%s,%s,%s,%s,%s)", (model_id, self._name, k, f.string.replace("'", " "), f._type, (f.relate and 'True') or 'False', f._obj or 'NULL', f.group_name or '', (f.view_load and 'True') or 'False'))
 				
 				if isinstance(f, fields.one2many):
 					cr.execute("SELECT relname FROM pg_class WHERE relkind='r' AND relname='%s'"%f._obj)
 					if cr.fetchone():
-						q=""" SELECT count(*) as c FROM pg_class c,pg_attribute a WHERE c.relname='%s' AND a.attname='%s' AND c.oid=a.attrelid"""%(f._obj,f._fields_id)
+						q = """SELECT count(*) as c FROM pg_class c,pg_attribute a WHERE c.relname='%s' AND a.attname='%s' AND c.oid=a.attrelid"""%(f._obj,f._fields_id)
 						cr.execute(q)
 						res = cr.fetchone()[0]
 						if not res:
-							cr.execute(("ALTER TABLE %s ADD FOREIGN KEY (%s) REFERENCES %s ON DELETE SET NULL") % (self._obj, f._fields_id, f._table))
+							cr.execute("ALTER TABLE %s ADD FOREIGN KEY (%s) REFERENCES %s ON DELETE SET NULL" % (self._obj, f._fields_id, f._table))
 				elif isinstance(f, fields.many2many):
 					cr.execute("SELECT relname FROM pg_class WHERE relkind='r' AND relname='%s'"%f._rel)
 					if not cr.dictfetchall():
@@ -325,8 +345,7 @@ class orm(object):
 							cr.execute("ALTER TABLE %s ADD COLUMN %s %s" % (self._table, k, get_pg_type(f)[1]))
 							
 							# initialize it
-							# maybe we should skip this step if it is the first run of the server and the database is initialized
-							if not create and self._defaults.has_key(k):
+							if not create and k in self._defaults:
 								default = self._defaults[k](self, cr, 1, {})
 								if not default:
 									cr.execute("UPDATE %s SET %s=NULL" % (self._table, k))
@@ -334,7 +353,7 @@ class orm(object):
 									cr.execute("UPDATE %s SET %s='%s'" % (self._table, k, default))
 
 							# and add constraints if needed
-							if isinstance(f,fields.many2one):
+							if isinstance(f, fields.many2one):
 								#FIXME: Remove this try/except
 								try:
 									ref = self.pool.get(f._obj)._table
@@ -354,13 +373,17 @@ class orm(object):
 									print "\tALTER TABLE %s ALTER COLUMN %s SET NOT NULL" % (self._table, k)
 							cr.commit()
 					elif len(res)==1:
-						type_dict = {'boolean':'bool', 'integer':'int4', 'char':'varchar', 'text':'text', 'date':'date', 'time':'time', 'datetime':'timestamp', 'binary':'bytea', 'many2one':'int4'}
-
 						f_pg_def = res[0]
 						f_pg_type = f_pg_def['typname']
 						f_pg_size = f_pg_def['size']
 						f_pg_notnull = f_pg_def['attnotnull']
-						f_obj_type = get_pg_type(f) and get_pg_type(f)[0]
+						if isinstance(f, fields.function):
+							print '-'*60
+							print "WARNING: column %s (%s) in table %s was converted to a function !" % (k, f.string, self._table)
+							print "\tYou should remove this column from your database."
+							f_obj_type = None
+						else:
+							f_obj_type = get_pg_type(f) and get_pg_type(f)[0]
 						
 						if f_obj_type:
 							if f_pg_type != f_obj_type:
@@ -400,6 +423,10 @@ class orm(object):
 								cr.commit()
 					else:
 						print "ERROR"
+		else:
+			cr.execute("SELECT relname FROM pg_class WHERE relkind in ('r','v') AND relname='%s'" % self._table)
+			create = not bool(cr.fetchone())
+
 		if create:
 			for (key,con,_) in self._sql_constraints:
 				cr.execute('alter table %s add constraint %s_%s %s' % (self._table,self._table,key, con,))
@@ -412,6 +439,7 @@ class orm(object):
 						cr.execute(line2)
 						cr.commit()
 
+			
 
 	def __init__(self):
 		if not self._table:
@@ -424,19 +452,30 @@ class orm(object):
 		module = module[:len(module)-1]
 		module = module.split('.')[0][2:]
 
-		if hasattr(self,'init'):
-			cr=sql_db.db.cursor()
-			self.init(cr)
-			cr.commit()
-			cr.close()
-			del cr
 		if hasattr(self,'_auto_init') and ((module in tools.config['update']) or (module in tools.config['init'])):
 			cr=sql_db.db.cursor()
+			if hasattr(self,'init'):
+				self.init(cr)
 			self._auto_init(cr)
 			cr.commit()
 			cr.close()
 			del cr
 
+		self._inherits_reload()
+		if not self._sequence:
+			self._sequence = self._table+'_id_seq'
+		for k in self._defaults:
+			assert (k in self._columns) or (k in self._inherit_fields), 'Default function defined but field %s does not exist !' % (k,)
+
+	#
+	# Update objects that uses this one to update their _inherits fields
+	#
+	def _inherits_reload_src(self):
+		for obj in self.pool.obj_pool.values():
+			if self._name in obj._inherits:
+				obj._inherits_reload()
+
+	def _inherits_reload(self):
 		res = {}
 		for table in self._inherits:
 			res.update(self.pool.get(table)._inherit_fields)
@@ -445,24 +484,171 @@ class orm(object):
 			for col in self.pool.get(table)._inherit_fields.keys():
 				res[col]=(table, self._inherits[table], self.pool.get(table)._inherit_fields[col][2])
 		self._inherit_fields=res
-		if not self._sequence:
-			self._sequence = self._table+'_id_seq'
+		self._inherits_reload_src()
 
 	def browse(self, cr, uid, select, context={}, list_class=None):
 		self._list_class = list_class or browse_record_list
 		cache = {}
-		if isinstance(select,int):
+		# need to accepts ints and longs because ids coming from a method 
+		# launched by button in the interface have a type long...
+		if isinstance(select, (int, long)):
 			return browse_record(cr,uid,select,self,cache, context=context, list_class=self._list_class)
 		elif isinstance(select,list):
 			return self._list_class([browse_record(cr,uid,id,self,cache, context=context, list_class=self._list_class) for id in select], context)
 		else:
 			return []
 
+	# TODO: implement this
+	def __export_row(self, cr, uid, row, fields, prefix, context={}):
+		lines = []
+		data = map(lambda x: '', range(len(fields)))
+		done = []
+		for fpos in range(len(fields)):
+			f = fields[fpos]
+			if f and f[:len(prefix)] == prefix:
+				r = row
+				i = 0
+				while i<len(f):
+					r = r[f[i]]
+					if not r:
+						break
+					if isinstance(r, (browse_record_list, list)):
+						first = True
+						fields2 = map(lambda x: (x[:i+1]==f[:i+1] and x[i+1:]) or [], fields)
+						if fields2 in done:
+							break
+						done.append(fields2)
+						for row2 in r:
+							lines2 = self.__export_row(cr, uid, row2, fields2, f[i+2:], context)
+							if first:
+								for fpos2 in range(len(fields)):
+									if lines2 and lines2[0][fpos2]:
+										data[fpos2] = lines2[0][fpos2]
+								lines+= lines2[1:]
+								first = False
+							else:
+								lines+= lines2
+						break
+					i+=1
+				if i==len(f):
+					data[fpos] = str(r or '')
+		return [data] + lines
+
+	def export_data(self, cr, uid, ids, fields, context={}):
+		fields = map(lambda x: x.split('/'), fields)
+		datas = []
+		for row in self.browse(cr, uid, ids, context):
+			datas += self.__export_row(cr, uid, row, fields, [], context)
+		return datas
+
+	def import_data(self, cr, uid, fields, datas, context={}):
+		fields = map(lambda x: x.split('/'), fields)
+
+		def process_liness(self, datas, prefix, fields_def, position=0):
+			line = datas[position]
+			row = {}
+			todo = []
+			warning = ''
+			#
+			# Import normal fields
+			#
+			for i in range(len(fields)):
+				if i>=len(line):
+					raise 'Please check that all your lines have %d cols.' % (len(fields),)
+				field = fields[i]
+				if (len(field)==len(prefix)+1) and (prefix==field[0:len(prefix)]):
+					if fields_def[field[len(prefix)]]['type']=='integer':
+						res =line[i] and int(line[i])
+					elif fields_def[field[len(prefix)]]['type']=='float':
+						res =line[i] and float(line[i])
+					elif fields_def[field[len(prefix)]]['type']=='selection':
+						res = False
+						for key,val in fields_def[field[len(prefix)]]['selection']:
+							if val==line[i]:
+								res = key
+					elif fields_def[field[len(prefix)]]['type']=='many2one':
+						res = False
+						if line[i]:
+							relation = fields_def[field[len(prefix)]]['relation']
+							res2 = self.pool.get(relation).name_search(cr, uid, line[i], [])
+							res = (res2 and res2[0][0]) or False
+							if not res and line[i]:
+								warning += ('Relation not found: '+line[i]+' on '+relation + ' !\n')
+							if not res:
+								print 'Relation not found: '+line[i]+' on '+relation + ' !\n'
+					elif fields_def[field[len(prefix)]]['type']=='many2many':
+						res = []
+						if line[i]:
+							relation = fields_def[field[len(prefix)]]['relation']
+							for word in line[i].split(','):
+								res2 = self.pool.get(relation).name_search(cr, uid, word, [])
+								res3 = (res2 and res2[0][0]) or False
+								if res3:
+									res.append(res3)
+							if len(res):
+								res= [(6,0,res)]
+					else:
+						res = line[i] or False
+					row[field[len(prefix)]] = res
+				elif (prefix==field[0:len(prefix)]):
+					if field[0] not in todo:
+						todo.append(field[len(prefix)])
+
+
+			#
+			# Import one2many fields
+			#
+			nbrmax = 0
+			for field in todo:
+				newfd = self.pool.get(fields_def[field]['relation']).fields_get(cr, uid, context=context)
+				(newrow,max2,w2) = process_liness(self, datas, prefix+[field], newfd, position)
+				nbrmax = max(nbrmax, max2)
+				warning = warning+w2
+				reduce(lambda x,y: x and y, newrow)
+				row[field] = (reduce(lambda x,y: x or y, newrow.values()) and [(0,0,newrow)]) or []
+				i = max2
+				while (position+i)<len(datas):
+					ok = True
+					for j in range(len(fields)):
+						field2 = fields[j]
+						if (len(field2)<=(len(prefix)+1)) and datas[position+i][j]:
+							ok = False
+					if not ok:
+						break
+
+					(newrow,max2,w2) = process_liness(self, datas, prefix+[field], newfd, position+i)
+					warning = warning+w2
+					if reduce(lambda x,y: x or y, newrow.values()):
+						row[field].append((0,0,newrow))
+					i+=max2
+					nbrmax = max(nbrmax, i)
+
+			if len(prefix)==0:
+				for i in range(max(nbrmax,1)):
+					#if datas:
+					datas.pop(0)
+			result = [row, nbrmax+1, warning]
+			return result
+
+		fields_def = self.fields_get(cr, uid, context=context)
+		done = 0
+		while len(datas):
+			(res,other,warning) = process_liness(self, datas, [], fields_def)
+			try:
+				self.create(cr, uid, res, context)
+			except Exception, e:
+				cr.rollback()
+				return (-1, res, e[0], warning)
+			done += 1
+		#
+		# TODO: Send a request with the result and multi-thread !
+		#
+		return (done, 0, 0, 0)
+
 	def read(self, cr, user, ids, fields=None, context={}, load='_classic_read'):
 		self.pool.get('ir.model.access').check(cr, user, self._name, 'read')
 		if not fields:
 			fields = self._columns.keys() + self._inherit_fields.keys()
-		
 		result =  self._read_flat(cr, user, ids, fields, context, load)
 		for r in result:
 			for key,v in r.items():
@@ -478,17 +664,17 @@ class orm(object):
 			fields = self._columns.keys()
 
 		# all inherited fields + all non inherited fields for which the attribute whose name is in load is True
-		fields_pre = filter(lambda x: x in self._columns and getattr(self._columns[x],load), fields) + self._inherits.values()
+		fields_pre = filter(lambda x: x in self._columns and getattr(self._columns[x],'_classic_write'), fields) + self._inherits.values()
 
-		cr.execute('select %s from %s where id in (%s) order by %s' % (','.join(fields_pre + ['id']), self._table, ','.join([str(x) for x in ids]), self._order))
+		if len(fields_pre):
+			cr.execute('select %s from %s where id in (%s) order by %s' % (','.join(fields_pre + ['id']), self._table, ','.join([str(x) for x in ids]), self._order))
 		
-		if not cr.rowcount:
-			return []
+			if not cr.rowcount:
+				return []
+			res = cr.dictfetchall()
+		else:
+			res = map(lambda x: {'id':x}, ids)
 
-#TODO: we should probably fetch rows by block, or even one at a time, 
-# because of the memory it takes to store the whole thing, then
-# transform it and store it again in whole
-		res = cr.dictfetchall()
 		if context.get('lang', False):
 			for f in fields_pre:
 				if self._columns[f].translate:
@@ -520,14 +706,13 @@ class orm(object):
 			for r in res:
 				for f in fields_post:
 					r[f] = self.columns[f]._symbol_get(r[f])
-		
 		ids = map(lambda x: x['id'], res)
 
 		# all non inherited fields for which the attribute whose name is in load is False
 		fields_post = filter(lambda x: x in self._columns and not getattr(self._columns[x], load), fields)
 		for f in fields_post:
 			# get the value of that field for all records/ids
-			res2 = self._columns[f].get(cr, self, ids, f, user, context=context)
+			res2 = self._columns[f].get(cr, self, ids, f, user, context=context, values=res)
 			for record in res:
 				record[f] = res2[record['id']]
 
@@ -700,8 +885,8 @@ class orm(object):
 			if self._inherits[v] not in vals:
 				tocreate[v] = {}
 
-		cr.execute('select perm_id from res_users where id=%d', (user,))
-		perm = cr.fetchone()[0]
+		#cr.execute('select perm_id from res_users where id=%d', (user,))
+		perm = False #cr.fetchone()[0]
 		(upd0, upd1, upd2) = ('', '', [])
 		upd_todo = []
 
@@ -731,7 +916,7 @@ class orm(object):
 			upd0 += ',create_uid,create_date'
 			upd1 += ',%d,current_timestamp'
 			upd2.append(user)
-		cr.execute('insert into '+self._table+' (id,perm_id'+upd0+") values ("+str(id_new)+',%d'+upd1+')', tuple([perm]+upd2))
+		cr.execute('insert into '+self._table+' (id,perm_id'+upd0+") values ("+str(id_new)+',NULL'+upd1+')', tuple(upd2))
 		upd_todo.sort(lambda x,y: self._columns[x].priority-self._columns[y].priority)
 		for field in upd_todo:
 			self._columns[field].set(cr, self, id_new, field, vals[field], user, context)
@@ -772,7 +957,7 @@ class orm(object):
 			res.update(self.pool.get(parent).fields_get(cr, user, fields, context))
 		for f in self._columns.keys():
 			res[f] = {'type': self._columns[f]._type}
-			for arg in ('string','readonly','states','size','required','change_default','translate'):
+			for arg in ('string','readonly','states','size','required','change_default','translate', 'help'):
 				if getattr(self._columns[f], arg):
 					res[f][arg] = getattr(self._columns[f], arg)
 			for arg in ('digits', 'invisible'):
@@ -856,6 +1041,31 @@ class orm(object):
 				trans =  tools.translate(cr, user, self._name, 'view', context['lang'], node.getAttribute('string').encode('utf8'))
 				if trans:
 					node.setAttribute('string', trans.decode('utf8'))
+			#
+			# Add view for properties !
+			#
+			if node.localName=='properties':
+				parent = node.parentNode
+
+				doc = node.ownerDocument
+
+				models = map(lambda x: "'"+x+"'", [self._name] + self._inherits.keys())
+				cr.execute('select id,name,group_name from ir_model_fields where model in ('+','.join(models)+') and view_load order by group_name, id')
+				oldgroup = None
+				res = cr.fetchall()
+				for (id,fname,gname) in res:
+					if oldgroup<>gname:
+						child = doc.createElement('separator')
+						child.setAttribute('string',gname)
+						child.setAttribute('colspan',"4")
+						oldgroup=gname
+						parent.insertBefore(child, node)
+
+					child = doc.createElement('field')
+					child.setAttribute('name',fname)
+					parent.insertBefore(child, node)
+				parent.removeChild(node)
+
 		if childs:
 			for f in node.childNodes:
 				fields.update(self.__view_look_dom(cr, user, f,context))
@@ -933,17 +1143,28 @@ class orm(object):
 			return doc_src.toxml()
 
 		result = {'type':view_type, 'model':self._name}
-		if view_id:
-			cr.execute('select arch,name,field_parent,id,type from ir_ui_view where model=%s and id=%d and inherit_id is null', (self._name,view_id))
-		else:
-			cr.execute('select arch,name,field_parent,id,type from ir_ui_view where model=%s and type=%s and inherit_id is null order by priority', (self._name,view_type))
-		sql_res = cr.fetchone()
+
+		ok = True
+		model = True
+		while ok:
+			if view_id:
+				where = (model and (" and model='%s'" % (self._name,))) or ''
+				cr.execute('select arch,name,field_parent,id,type,inherit_id from ir_ui_view where id=%d'+where, (view_id,))
+			else:
+				cr.execute('select arch,name,field_parent,id,type,inherit_id from ir_ui_view where model=%s and type=%s order by priority', (self._name,view_type))
+			sql_res = cr.fetchone()
+			if not sql_res:
+				break
+			ok = sql_res[5]
+			view_id = ok or sql_res[3]
+			model = False
+
 		if sql_res:
 			result['type'] = sql_res[4]
 			result['view_id'] = sql_res[3]
 			result['arch'] = sql_res[0]
 
-			cr.execute('select arch from ir_ui_view where inherit_id=%d order by priority', (sql_res[3],))
+			cr.execute('select arch from ir_ui_view where inherit_id=%d and model=%s order by priority', (sql_res[3],self._name))
 			sql_inherit = cr.fetchall()
 			for (inherit,) in sql_inherit:
 				result['arch'] = _inherit_apply(result['arch'], inherit)
@@ -956,7 +1177,7 @@ class orm(object):
 		else:
 			if view_type=='form':
 				res = self.fields_get(cr, user, context=context)
-				xml = "<?xml version=\"1.0\"?>\n<form string=\"%s\">\n" % (self._name,)
+				xml = "<?xml version=\"1.0\"?>\n<form string=\"%s\">\n" % (self._description,)
 				for x in res:
 					if res[x]['type'] not in ('one2many','many2many'):
 						xml += "\t<field name=\"%s\"/>\n" % (x,)
@@ -965,7 +1186,7 @@ class orm(object):
 				xml += "</form>"
 			elif view_type=='tree':
 				res = self.fields_get(cr, user, [self._rec_name], context)
-				xml = '''<?xml version="1.0"?>\n<tree string="%s">\n\t<field name="%s"/>\n</tree>''' % (self._rec_name,self._rec_name)
+				xml = '''<?xml version="1.0"?>\n<tree string="%s">\n\t<field name="%s"/>\n</tree>''' % (self._description,self._rec_name)
 			else:
 				xml = ''
 			result['arch'] = xml
@@ -982,23 +1203,32 @@ class orm(object):
 		#	if len(fields[f]):
 		#		for typ in fields[f].split(','):
 		#			result['fields'][f]['view_data_'+str(typ)] = self.pool.get(result['fields'][f]['relation']).fields_view_get(cr, user, None, str(typ))
+
 		return result
 
 	# TODO: ameliorer avec NULL
 	def _where_calc(self, args):
 		qu1, qu2 = [], []
 		for x in args:
-			if x[1]!='in':
+			if isinstance(x, WhereClause):
+				qu, val = x.to_query()
+				qu1.append(qu)
+				if val is not None:
+					qu2.append(val)
+				continue
+			if x[1] != 'in':
 #FIXME: this replace all (..., '=', False) values with 'is null' and this is 
 # not what we want for real boolean fields. The problem is, we can't change it
 # easily because we use False everywhere instead of None
+# NOTE FAB: we can't use None because it is not accepted by XML-RPC, that's why
+# boolean (0-1), None -> False
 				if (x[2] is False) and (x[1]=='='):
 					qu1.append(x[0]+' is null')
 				elif (x[2] is False) and (x[1]=='<>' or x[1]=='!='):
 					qu1.append(x[0]+' is not null')
 				else:
 					if x[0]=='id':
-						qu1.append('(%s %s %%s)' % (x[0], x[1]))
+						qu1.append('(%s.%s %s %%s)' % (self._table, x[0], x[1]))
 						qu2.append(x[2])
 					else:
 						if x[1] in ('like', 'ilike'):
@@ -1015,7 +1245,7 @@ class orm(object):
 							x1 = 'like'
 						else:
 							x1 = x[1]
-						qu1.append('(%s %s %s)' % (x[0], x1, self._columns[x[0]]._symbol_set[0]))
+						qu1.append('(%s.%s %s %s)' % (self._table, x[0], x1, self._columns[x[0]]._symbol_set[0]))
 			elif x[1]=='in':
 				if len(x[2])>0:
 					todel = False
@@ -1026,7 +1256,7 @@ class orm(object):
 					if x[0]=='id':
 						qu1.append('(id in (%s))' % (','.join(['%d'] * len(x[2])),))
 					else:
-						qu1.append('(%s in (%s))' % (x[0], ','.join([self._columns[x[0]]._symbol_set[0]]*len(x[2]))))
+						qu1.append('(%s.%s in (%s))' % (self._table, x[0], ','.join([self._columns[x[0]]._symbol_set[0]]*len(x[2]))))
 					if todel:
 						qu1[-1] = '('+qu1[-1]+' or '+x[0]+' is null)'
 					qu2+=x[2]
@@ -1034,7 +1264,7 @@ class orm(object):
 					qu1.append(' (1=0)')
 		return (qu1,qu2)
 
-	def search(self, cr, user, args, offset=0, limit=2000, order=None):
+	def search(self, cr, user, args, offset=0, limit=None, order=None):
 		# if the object has a field named 'active', filter out all inactive 
 		# records unless they were explicitely asked for
 		if 'active' in self._columns:
@@ -1078,12 +1308,13 @@ class orm(object):
 
 				# get the ids of the records of the "distant" resource
 				ids2 = [x[0] for x in field_obj.name_search(cr, user, args[i][2], [], args[i][1])]
+				if not ids2:
+					args[i] = ('id','=','0')
+				else:
+					cr.execute('select '+field._fields_id+' from '+field_obj._table+' where id in ('+','.join(map(str,ids2))+')')
+					ids3 = [x[0] for x in cr.fetchall()]
 
-				# get the ids of the local resource that those distant records point to
-				cr.execute('select '+field._fields_id+' from '+field_obj._table+' where id in ('+','.join(map(str,ids2))+')')
-				ids3 = [x[0] for x in cr.fetchall()]
-
-				args[i] = ('id', 'in', ids3)
+					args[i] = ('id', 'in', ids3)
 				i+=1
 
 			elif field._type=='many2many':
@@ -1106,7 +1337,7 @@ class orm(object):
 					if not len(res_ids):
 						return []
 					cr.execute('select '+field._id1+' from '+field._rel+' where '+field._id2+' in ('+','.join(map(str, res_ids))+')')
-					args[i] = ('id','in',map(lambda x: x[0], cr.fetchall()))
+					args[i] = ('id', 'in', map(lambda x: x[0], cr.fetchall()))
 				i+=1
 
 			elif field._type=='many2one':
@@ -1141,11 +1372,12 @@ class orm(object):
 			else:
 				i+=1
 
+		tables = [self._table]
 		for table in args2:
-			res1 = self.pool.get(table).search(cr, user, args2[table], offset, limit)
-			if not len(res1):
-				return []
-			args.append( (self._inherits[table], 'in', res1) )
+			other = self.pool.get(table)
+			args.extend([WhereClause(other, 'id', '=', '%s.%s' % (self._table, self._inherits[table]))])
+			args.extend([WhereClause(other, col, op, comp) for col, op, comp in args2[table]])
+			tables.append('%s' % other._table)
 			
 		# compute the where, order by, limit and offset clauses
 		(qu1,qu2) = self._where_calc(args)
@@ -1154,20 +1386,19 @@ class orm(object):
 		else:
 			qu1 = ''
 		order_by = order or self._order
-			
+
 		limit_str = limit and ' limit %d' % limit or ''
 		offset_str = offset and ' offset %d' % offset or ''
 		
 		# execute the "main" query to fetch the ids we were searching for
-		cr.execute('select id from '+self._table+qu1+' order by '+order_by+limit_str+offset_str, qu2)
+ 		cr.execute('select %s.id from ' % self._table + ','.join(tables) +qu1+' order by '+order_by+limit_str+offset_str, qu2)
 		res = cr.fetchall()
-		result = [x[0] for x in res]
-		return result
+		return [x[0] for x in res]
 
 	# returns the different values ever entered for one field
 	# this is used, for example, in the client when the user hits enter on 
 	# a char field
-	def distinct_field_get(self, cr, uid, field, value, args=[], offset=0, limit=2000):
+	def distinct_field_get(self, cr, uid, field, value, args=[], offset=0, limit=None):
 		if field in self._inherit_fields:
 			return self.pool.get(self._inherit_fields[field][0]).distinct_field_get(cr, uid,field,value,args,offset,limit)
 		else:
@@ -1178,10 +1409,10 @@ class orm(object):
 			return []
 		return [(r['id'], r[self._rec_name]) for r in self.read(cr, user, ids, [self._rec_name], context, load='_classic_write')]
 
-	def name_search(self, cr, user, name='', args=[], operator='ilike', context={}):
+	def name_search(self, cr, user, name='', args=[], operator='ilike', context={}, limit=80):
 		if name:
 			args += [(self._rec_name,operator,name)]
-		ids = self.search(cr, user, args)
+		ids = self.search(cr, user, args, limit=limit)
 		res = self.name_get(cr, user, ids, context)
 		return res
 
@@ -1189,7 +1420,8 @@ class orm(object):
 		if not default:
 			default = {}
 		if 'state' not in default:
-			default['state'] = 'draft'
+			if 'state' in self._defaults:
+				default['state'] = self._defaults['state'](self, cr, uid, context)
 		data = self.read(cr, uid, [id], context=context)[0]
 		fields = self.fields_get(cr, uid)
 		for f in fields:
@@ -1207,18 +1439,45 @@ class orm(object):
 				res = []
 				rel = self.pool.get(fields[f]['relation'])
 				for rel_id in data[f]:
+					# the lines are first duplicated using the wrong (old) 
+					# parent but then are reassigned to the correct one thanks
+					# to the (4, ...)
 					res.append((4, rel.copy(cr, uid, rel_id, context=context)))
 				data[f] = res
 			elif ftype == 'many2many':
-				res = []
-				rel = self.pool.get(fields[f]['relation'])
-				for rel_id in data[f]:
-					res.append((4, rel_id))
-				data[f] = res
+				data[f] = [(6, 0, data[f])]
 		del data['id']
 		for v in self._inherits:
 			del data[self._inherits[v]]
 		return self.create(cr, uid, data)
 
-# vim:noexpandtab:ts=4
+class WhereClause(object):
 
+	def __init__(self, table, column, operator, condition):
+		self.table = table
+		self.column = column
+		self.operator = operator
+		self.condition = condition
+
+	def clause(self):
+		if not self.column in self.table._columns:
+			val = self.condition
+		else:
+			val = self.table._columns[self.column]._symbol_set[0]
+		return '(%s.%s %s %s)' % (self.table._table, self.column, self.operator, val)
+
+	def value(self):
+		if not self.column in self.table._columns:
+			val = None
+		else:
+			val = self.table._columns[self.column]._symbol_set[1](self.condition)
+			if self.operator in ('like', 'ilike'):
+				val = '%%%s%%' % val
+		return val
+
+	def to_query(self):
+		return self.clause(), self.value()
+
+	def __str__(self):
+		return self.condition() % self.value()
+# vim:noexpandtab:ts=4
