@@ -146,7 +146,11 @@ class inheritor(type):
 				new_dict.update(d.get(s, {}))
 				d[s] = new_dict
 			bases = (parent_obj.__class__,)
-		return type.__new__(cls, name, bases, d)
+		res = type.__new__(cls, name, bases, d)
+		#
+		# update _inherits of others objects
+		#
+		return res
 
 class osv(orm.orm):
 	__metaclass__ = inheritor
