@@ -41,7 +41,7 @@ import print_xml
 import render
 
 #
-# encode to a string in utf8
+# encode a value to a string in utf8 and converts XML entities
 #
 def toxml(val):
 	if isinstance(val, str):
@@ -62,14 +62,6 @@ class report_int(netsvc.Service):
 		self.name2 = '.'.join(name.split('.')[1:])
 		self.joinGroup('report')
 		self.exportMethod(self.create)
-		self.exportMethod(self.result)
-		self.obj = False
-
-	def result(self):
-		if self.obj and self.obj.is_done():
-			return (True, self.obj.get(), self.obj.output_type)
-		else:
-			return (False, False, False)
 
 	def create(self, uid, ids, datas, context={}):
 		return False
@@ -101,7 +93,7 @@ class report_rml(report_int):
 		pdf = create_doc(rml)
 		return (pdf, report_type)
 	
-	def	create_xml(self, uid, ids, datas, context={}):
+	def create_xml(self, uid, ids, datas, context={}):
 		doc = print_xml.document(uid, datas, {})
 		self.bin_datas = doc.bin_datas
 		doc.parse(self.tmpl, ids, self.table, context)
@@ -173,14 +165,14 @@ class report_rml(report_int):
 		return xml
 	
 	def create_pdf(self, xml):
-		self.obj = render.rml(xml, self.bin_datas, os.path.dirname(self.tmpl))
-		self.obj.render()
-		return self.obj.get()
+		obj = render.rml(xml, self.bin_datas, os.path.dirname(self.tmpl))
+		obj.render()
+		return obj.get()
 
 	def create_html(self, xml):
-		self.obj = render.rml2html(xml, self.bin_datas)
-		self.obj.render()
-		return self.obj.get()
+		obj = render.rml2html(xml, self.bin_datas)
+		obj.render()
+		return obj.get()
 
 	def create_raw(self, xml):
 		return xml
