@@ -53,6 +53,15 @@ class InheritDict(dict):
 			else:
 				return self.parent[name]
 
+def tounicode(val):
+	if isinstance(val, str):
+		unicode_val	= unicode(val, 'utf-8')
+	elif isinstance(val, unicode):
+		unicode_val = val
+	else:
+		unicode_val = unicode(val) 
+	return unicode_val
+
 class document(object):
 	def __init__(self, uid, datas, func=False):
 		# create a new document
@@ -62,18 +71,6 @@ class document(object):
 		self.uid = uid
 		self.bin_datas = {}
 
-	# encode value (which is supposed to be encoded in utf8 if it is a string)
-	# to a unicode object and replace invalid chars by their corresponding 
-	# xml entities
-	def toxml(self, val):
-		if isinstance(val, str):
-			unicode_val = val.decode('utf-8')
-		elif isinstance(val, unicode):
-			unicode_val = val
-		else:
-			unicode_val = unicode(val)
-		return unicode_val.replace('&', '&amp;').replace('<','&lt;').replace('>','&gt;')
-
 	def node_attrs_get(self, node):
 		attrs = {}
 		nattr = node.attributes
@@ -155,7 +152,7 @@ class document(object):
 						value = attrs['default']
 					el = self.doc.createElement(node.localName)
 					parent.appendChild(el)
-					el_txt = self.doc.createTextNode(self.toxml(value))
+					el_txt = self.doc.createTextNode(tounicode(value))
 					el.appendChild(el_txt)
 
 #TODO: test this
@@ -198,7 +195,7 @@ class document(object):
 					el = self.doc.createElement(node.localName)
 					parent.appendChild(el)
 					txt = self.datas.get('form', {}).get(attrs['name'], '')
-					el_txt = self.doc.createTextNode(self.toxml(txt))
+					el_txt = self.doc.createTextNode(tounicode(txt))
 					el.appendChild(el_txt)
 
 				elif attrs['type']=='function':
