@@ -39,6 +39,9 @@ import os
 # reportlab use "code page 1252" encoding by default. cfr reportlab user guide p.46
 encoding = 'cp1252'
 
+def str2xml(s):
+	return s.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
+
 def _child_get(node, childs):
 	clds = []
 	for n in node.childNodes:
@@ -114,7 +117,7 @@ class _rml_styles(object):
 					thick = 1
 					if node.hasAttribute('thickness'):
 						thick = float(node.getAttribute('thickness'))
-					styles.append((kind, start, stop, thick,  color.get(node.getAttribute('colorName'))))
+					styles.append((kind, start, stop, thick, color.get(node.getAttribute('colorName'))))
 		return platypus.tables.TableStyle(styles)
 
 	def _para_style_get(self, node):
@@ -190,13 +193,12 @@ class _rml_canvas(object):
 		rc = ''
 		for n in node.childNodes:
 			if n.nodeType == n.ELEMENT_NODE:
-				if n.localName=='pageNumber':
+				if n.localName == 'pageNumber':
 					rc += str(self.canvas.getPageNumber())
-			elif (n.nodeType == node.CDATA_SECTION_NODE):
+			elif n.nodeType in (node.CDATA_SECTION_NODE, node.TEXT_NODE):
+				# this doesn't need to be "entities" encoded like flowables need to
 				rc += n.data
-			elif (n.nodeType == node.TEXT_NODE):
-				rc += n.data
-		return rc.encode(encoding,'replace')
+		return rc.encode(encoding, 'replace')
 
 	def _drawString(self, node):
 		self.canvas.drawString(text=self._textual(node), **utils.attr_get(node, ['x','y']))
@@ -279,8 +281,9 @@ class _rml_canvas(object):
 	def _image(self, node):
 		import urllib
 		from reportlab.lib.utils import ImageReader
-		if str(node.getAttribute('file')) in self.images:
-			s = StringIO.StringIO(self.images[str(node.getAttribute('file'))])
+
+		if not node.hasAttribute('file'):
+			s = self.images[node.getAttribute('name')]
 		else:
 			try:
 				u = urllib.urlopen(str(node.getAttribute('file')))
@@ -384,20 +387,19 @@ class _rml_flowable(object):
 		rc = ''
 		for n in node.childNodes:
 			if n.nodeType == node.ELEMENT_NODE:
-				if n.localName=='getName':
+				if n.localName == 'getName':
 					newNode = self.doc.dom.createTextNode(self.styles.names.get(n.getAttribute('id'),'Unknown name'))
 					node.insertBefore(newNode, n)
 					node.removeChild(n)
-				if n.localName=='pageNumber':
-					rc+='<pageNumber/>'            # TODO: change this !
+				elif n.localName == 'pageNumber':
+					rc += '<pageNumber/>'            # TODO: change this !
 				else:
+					#CHECKME: I wonder if this is useful since we don't stock the result. Maybe for the getName tag?
 					self._textual(n)
 				rc += n.toxml()
-			elif (n.nodeType == node.CDATA_SECTION_NODE):
-				rc += n.data
-			elif (n.nodeType == node.TEXT_NODE):
-				rc += n.data
-		return rc.encode(encoding,'replace')
+			elif n.nodeType in (node.CDATA_SECTION_NODE, node.TEXT_NODE):
+				rc += str2xml(n.data)
+		return rc.encode(encoding, 'replace')
 
 	def _table(self, node):
 		length = 0
