@@ -93,11 +93,18 @@ class rml_parse(object):
 			'user': user,
 			'company': user.company_id,
 			'repeatIn': self.repeatIn,
+			'setTag': self.setTag,
 		}
 		self.localcontext.update(context)
 		self._regex = re.compile('\[\[(.+?)\]\]')
 		self._node = None
 
+	def setTag(self, oldtag, newtag):
+		node = self._find_parent(self._node, [oldtag])
+		if node:
+			node.tagName = newtag
+		return None
+
 	def repeatIn(self, lst, name):
 		node = self._node
 		node.data = ''
@@ -139,7 +146,7 @@ class rml_parse(object):
 			res = 'Error'
 		return res
 
-	def _find_parent(self, node):
+	def _find_parent(self, node, parents):
 		while True:
 			if not node.parentNode:
 				return False
@@ -165,7 +172,7 @@ class rml_parse(object):
 		self._node.data = text
 		if len(todo):
 			for key, newtext in todo:
-				parent_node = self._find_parent(self._node)
+				parent_node = self._find_parent(self._node, parents)
 				assert parents.get(parent_node.localName,False), 'No parent node found !'
 				nodes = [parent_node]
 				for i in range(len(newtext)-1):
@@ -285,9 +292,9 @@ class report_sxw(report_rml):
 			translate(rml_dom.documentElement, context['lang'])
 
 		rml2 = rml_parser._parse(rml_dom, objs, data)
-#		f = file("/tmp/debug.rml", "w")
-#		f.write(rml2)
-#		f.close()
+		#f = file("/tmp/debug.rml", "w")
+		#f.write(rml2)
+		#f.close()
 		cr.close()
 		report_type= data.get('report_type','pdf')
 		create_doc = self.generators[report_type]
