@@ -101,7 +101,7 @@ GNU Public Licence.
 common()
 
 class objects_proxy(netsvc.Service):
-	def __init__(self,name="object"):
+	def __init__(self, name="object"):
 		netsvc.Service.__init__(self,name)
 		self.joinGroup('web-services')
 		self.exportMethod(self.execute)
@@ -157,6 +157,7 @@ class wizard(netsvc.Service):
 
 	def create(self, uid, passwd, wiz_name, datas={}):
 		security.check(uid, passwd)
+#FIXME: this is not thread-safe
 		self.id += 1
 		self.wiz_datas[self.id] = {}
 		self.wiz_name[self.id] = wiz_name
@@ -165,44 +166,45 @@ class wizard(netsvc.Service):
 
 	def execute(self, uid, passwd, wiz_id, datas, action='init', context={}):
 		security.check(uid, passwd)
-		if self.wiz_uid[wiz_id] == uid:
-			return self._execute(uid, wiz_id, datas, action, context)
+
+		if wiz_id in self.wiz_uid:
+			if self.wiz_uid[wiz_id] == uid:
+				return self._execute(uid, wiz_id, datas, action, context)
+			else:
+				raise 'AccessDenied'
 		else:
-			raise 'AccessDenied'
+			raise 'WizardNotFound'
 wizard()
 
 #
 # TODO: set a maximum report number per user to avoid DOS attacks
 #
-# Report Status:
-#     unknown -> wait +-> done
-#                     +-> abort
+# Report state:
+#     False -> True
 #
 class report_spool(netsvc.Service):
 	def __init__(self, name='report'):
-		netsvc.Service.__init__(self,name)
+		netsvc.Service.__init__(self, name)
 		self.joinGroup('web-services')
 		self.exportMethod(self.report)
 		self.exportMethod(self.report_get)
-#CHECKME: report_check n'est pas utilise directement ds le client, on 
-# pourrait donc ne pas l'exporter...
-		self.exportMethod(self.report_check)
 		self._reports = {}
-		self._requests = {}
 		self.id = 0
 		self.id_protect = threading.Semaphore()
 
 	def report(self, uid, passwd, object, ids, datas={}, context={}):
 		security.check(uid, passwd)
+		
 		self.id_protect.acquire()
 		self.id += 1
 		id = self.id
 		self.id_protect.release()
 
 		self._reports[id] = {'uid': uid, 'result': False, 'state': False}
+
 		def go(id, uid, ids, datas, context):
 			obj = netsvc.LocalService('report.'+object)
-			(result,format) = obj.create(uid, ids, datas, context)
+			(result, format) = obj.create(uid, ids, datas, context)
 			self._reports[id]['result'] = result
 			self._reports[id]['format'] = format
 			self._reports[id]['state'] = True
@@ -211,70 +213,36 @@ class report_spool(netsvc.Service):
 		thread.start_new_thread(go, (id, uid, ids, datas, context))
 		return id
 
-#	def _rep_get_ok(self, id, result, *args):
-#		req = self._requests[id]
-#		self._results[req] = result
-#		del self._requests[id]
-#		self.resumeResponse(req)
-#
-#	def _rep_get_fail(self, id, error,description,origin,details):
-#		req = self._requests[id]
-#		del self._requests[id]
-#		#fail = self.currentFailure()
-#		self.cancelResponse(req, error, description, origin, details)
-#
-#	def _rep_sus_check(self, req):
-#		result = self._results[req]
-#		del self._results[req]
-#		return result[0]
-#
-
-	def _rep_sus(self, report_id):
+	def _check_report(self, report_id):
 		result = self._reports[report_id]
-		res={'state':result['state']}
+		res = {'state': result['state']}
 		if res['state']:
 			if tools.config['reportgz']:
 				import zlib
-				res['result']=base64.encodestring(zlib.compress(result['result']))
-				res['code']='zlib'
+				res2 = zlib.compress(result['result'])
+				res['code'] = 'zlib'
 			else:
-				if type(result['result'])==type(u''):
-					res2 = result['result'].encode('latin1','replace')
+				#CHECKME: why is this needed???
+				if isinstance(result['result'], unicode):
+					res2 = result['result'].encode('latin1', 'replace')
 				else:
 					res2 = result['result']
-				res['result'] = base64.encodestring(res2)
+			res['result'] = base64.encodestring(res2)
 			res['format'] = result['format']
+			del self._reports[report_id]
 		return res
 
-	def report_check(self, uid, passwd, report_id, format='normal'):
-#CHECKME: faudrait pas rajouter un security.check(uid, passwd) ici? 
-		value = ''
+	def report_get(self, uid, passwd, report_id):
+		security.check(uid, passwd)
+
 		if report_id in self._reports:
-			if self._reports[report_id]['uid'] != uid:
-				raise 'AccessDenied'
+			if self._reports[report_id]['uid'] == uid:
+				return self._check_report(report_id)
 			else:
-				# returns the whole dictionary describing this report
-#CHECKME: faudrait pas retourner juste la clef state?
-# vérifier ds le client... reponse: dans le client, il attend un dico mais 
-# n'utilise que la clef state
-				return self._reports[report_id]
+				raise 'AccessDenied'
 		else:
 			raise 'ReportNotFound'
 
-
-	def report_get(self, uid, passwd, report_id, format='normal'):
-#CHECKME: faudrait pas rajouter un security.check(uid, passwd) ici?
-# pr le moment il suffit de faire une requete avec le bon uid pour 
-# recuperer (et "manger") un rapport demandé par qqun d'autre
-# juste checker done['state']?
-		done = self.report_check(uid, passwd, report_id, format)
-		if done:
-			result = self._rep_sus(report_id)
-			if result['state']:
-				del self._reports[report_id]
-			return result
-		return done
-
 report_spool()
 
 # vim:noexpandtab
