@@ -37,24 +37,59 @@ import sys,os
 #	from tools import decimal
 
 
+import re
+
 class fake_cursor:
 	nbr = 0
+	_tables = {}
 	def __init__(self,db,con):
 		self.db=db
 		self.obj=db.cursor()
 		self.con=con
-#	def execute(self,*args):
-#		#print ' sql:', fake_cursor.nbr, args
-#		fake_cursor.nbr += 1
-#		return self.obj.execute(*args)
-#	def close(self):
-#		#print "close cursors fno:",[i.fileno() for i in self.db.cursors]
-#		self.obj.close()
-#		del self.obj
-#		print "after close cursors fno:",[i.fileno() for i in self.db.cursors]
+	def execute_dont_run(self,*args):
+		if not fake_cursor.nbr % 100:
+			print 'sql: ',fake_cursor.nbr
+		res = re.match('^select.* from ([a-zA-Z_]+) .*$', args[0], re.I)
+		if res:
+			fake_cursor._tables.setdefault(res.group(1), 0)
+			fake_cursor._tables[res.group(1)] += 1
+		#else:
+		#	if len(args)>1:
+		#		print 'sql: ',fake_cursor.nbr, args[0], args[1]
+		#	else:
+		#		print 'sql: ',fake_cursor.nbr, args[0]
+
+		if not fake_cursor.nbr % 5000:
+			ct = []
+			for t,c in fake_cursor._tables.items():
+				ct.append([c,t])
+			ct.sort()
+			ct.reverse()
+			print 'After %d queries' % (fake_cursor.nbr,)
+			for line in ct[:50]:
+				print '    %s: %d' % (line[1], line[0])
+		#if len(args)>1:
+		#	print 'sql: ',fake_cursor.nbr, args[0], args[1]
+		#else:
+		#	print 'sql: ',fake_cursor.nbr, args[0]
+		fake_cursor.nbr += 1
+		return self.obj.execute(*args)
+
+	def close(self):
+#		print "close cursors fno:", [i.fileno() for i in self.db.cursors]
+		self.obj.close()
+
+		# This force the cursor to be freed, and thus, available again. It is 
+		# important because otherwise we can overload the server very easily 
+		# because of a cursor shortage (because cursors are not garbage
+		# collected as fast as they should). The problem is probably due in 
+		# part because browse records keep a reference to the cursor.
+		del self.obj
+#		print "after close cursors fno:", [i.fileno() for i in self.db.cursors]
+
 	def __getattr__(self, name):
 #		print 'LOOK',name
-		return getattr(self.obj,name)
+		return getattr(self.obj, name)
 
 class fakedb:
 	def __init__(self,truedb):
