@@ -40,7 +40,7 @@ GNU Public Licence.
 """
 
 __author__ = 'Fabien Pinckaers, <fp@tiny.be>'
-__version__ = '3.3.0'
+__version__ = '3.4.2'
 
 import __builtin__
 __builtin__.__dict__['tinyerp_version'] = __version__
@@ -185,6 +185,12 @@ if tools.config["translate_in"]:
 	tools.trans_load(tools.config["translate_in"], tools.config["language"])
 	sys.exit(0)
 
+#----------------------------------------------------------------------------------
+# if we don't want the server to continue to run after initialization, we quit here
+#----------------------------------------------------------------------------------
+if tools.config["stop_after_init"]:
+    sys.exit(0)
+
 #----------------------------------------------------------
 # Launch Server
 #----------------------------------------------------------
@@ -194,8 +200,9 @@ except Exception:
 	logger.notifyChannel("init", netsvc.LOG_ERROR, "invalid port '%s'!" % (tools.config["port"],))
 	sys.exit(1)
 interface = tools.config["interface"]
+secure = tools.config["secure"]
 
-httpd = netsvc.HttpDaemon(interface,port)
+httpd = netsvc.HttpDaemon(interface,port, secure)
 
 if tools.config["xmlrpc"]:
 	xml_gw = netsvc.xmlrpc.RpcGateway('web-services')
@@ -208,9 +215,14 @@ if tools.config["xmlrpc"]:
 #	httpd.attach("/soap", soap_gw )
 #	logger.notifyChannel("web-services", netsvc.LOG_INFO, 'starting SOAP services, port '+str(port))
 #
+if not (netsvc.HAS_SSL and tools.config['secure']):
+	logger.notifyChannel("web-services", netsvc.LOG_INFO, "You are not using the SSL layer")
+else:
+	logger.notifyChannel("web-services", netsvc.LOG_INFO, "You are using the SSL Layer")
 
 logger.notifyChannel("web-services", netsvc.LOG_INFO, 'the server is running, waiting for connections...')
 osv.osv.osv_pools.get('ir.cron')._poolJobs()
 
+
 httpd.start()
 dispatcher.run()
