@@ -48,11 +48,14 @@ class configmanager(object):
 			'translate_in': None,
 			'translate_out': None,
 			'language': None,
-			'addons_path' : None,
-			'root_path' : None,
-			'debug_mode' : False,
-			'pidfile' : None,
-			'logfile' : None,
+			'addons_path': None,
+			'root_path': None,
+			'debug_mode': False,
+			'pidfile': None,
+			'logfile': None,
+			'secure': False,
+			'smtp_server': 'localhost',
+			'stop_after_init': False,   # this will stop the server after initialization
 		}
 
 		parser = optparse.OptionParser(version=tinyerp_version_string)
@@ -69,7 +72,11 @@ class configmanager(object):
 		parser.add_option("-i", "--init", dest="init", help="init a module (use \"all\" for all modules)")
 		parser.add_option("--without-demo", dest="without_demo", help="load demo data for a module (use \"all\" for all modules)", default=False)
 		parser.add_option("-u", "--update", dest="update", help="update a module (use \"all\" for all modules)")
-		parser.add_option('--debug', dest='debug_mode', action='store_true', default=False, help='Enable debug mode')
+		# stops the server from launching after initialization
+		parser.add_option("--stop-after-init", action="store_true", dest="stop_after_init", default=False, help="stop the server after it initializes")
+		parser.add_option('--debug', dest='debug_mode', action='store_true', default=False, help='enable debug mode')
+		parser.add_option("-S", "--secure", dest="secure", action="store_true", help="launch server over https instead of http", default=False)
+		parser.add_option('--smtp', dest='smtp_server', default='', help='specify the SMTP server for sending mail')
 		
 		group = optparse.OptionGroup(parser, "Modules related options")
 		group.add_option("-g", "--upgrade", action="store_true", dest="upgrade", default=False, help="Upgrade/install/uninstall modules")
@@ -83,8 +90,8 @@ class configmanager(object):
 		parser.add_option_group(group)
 
 		group = optparse.OptionGroup(parser, "Internationalisation options",
-			  "Use these options to translate Tiny ERP to another language. "
-			  "See i18n section of the user manual. Option '-l' is mandatory.")
+			"Use these options to translate Tiny ERP to another language."
+			"See i18n section of the user manual. Option '-l' is mandatory.")
 
 		group.add_option('-l', "--language", dest="language", help="specify the language of the translation file. Use it with --i18n-export and --i18n-import")
 		group.add_option("--i18n-export", dest="translate_out", help="export all sentences to be translated to a CSV file and exit")
@@ -98,28 +105,41 @@ class configmanager(object):
 			print "Error: the i18n-import and i18n-export options cannot be used without the language option"
 			sys.exit(2)
 
-		self.rcfile = fname or opt.config or os.environ.get('TERP_SERVER') or os.path.expanduser('~/.terp_serverrc')
+		# place/search the config file on Win32 near the server installation
+		# (../etc from the server)
+		# if the server is run by an unprivileged user, he has to specify location of a config file where he has the rights to write,
+		# else he won't be able to save the configurations, or even to start the server...
+		if os.name == 'nt':
+			rcfilepath = os.path.join(os.path.split(os.path.split(os.path.abspath(sys.argv[0]))[0])[0], 'etc', 'tinyerp-server.conf')
+		else:
+			rcfilepath = os.path.expanduser('~/.terp_serverrc')
+
+		self.rcfile = fname or opt.config or os.environ.get('TERP_SERVER') or rcfilepath
 		self.load()
 		
-		for arg in ('interface', 'port', 'db_name', 'db_user', 'db_password', 'db_host', 'db_port'):
+
+		# Verify that we want to log or not, if not the output will go to stdout
+		if self.options['logfile'] in ('None', 'False'):
+                    self.options['logfile'] = False
+		# the same for the pidfile
+                if self.options['pidfile'] in ('None', 'False'):
+                    self.options['pidfile'] = False
+		
+		for arg in ('interface', 'port', 'db_name', 'db_user', 'db_password', 'db_host',
+				'db_port', 'logfile', 'pidfile', 'secure', 'smtp_server'):
 			if getattr(opt, arg):
 				self.options[arg] = getattr(opt, arg)
 
-		for arg in ('language', 'translate_out', 'translate_in'):
+		for arg in ('language', 'translate_out', 'translate_in', 
+			'upgrade', 'verbose', 'debug_mode', 
+			'stop_after_init', 'without_demo'):
 			self.options[arg] = getattr(opt, arg)
 			
-		self.options['upgrade'] = opt.upgrade
-		self.options['verbose'] = opt.verbose
-		self.options['debug_mode'] = opt.debug_mode
-		self.options['pidfile'] = opt.pidfile
-		self.options['logfile'] = opt.logfile
-
 		init = {}
 		if opt.init:
 			for i in opt.init.split(','):
 				init[i] = 1
 		self.options['init'] = init
-		self.options['without_demo'] = opt.without_demo
 		self.options["demo"] = not opt.without_demo and self.options['init'] or {}
 
 		update = {}
@@ -127,7 +147,7 @@ class configmanager(object):
 			for i in opt.update.split(','):
 				update[i] = 1
 		self.options['update'] = update
-		
+
 		self.options['translate_modules'] = opt.translate_modules and opt.translate_modules.split(',') or ['all']
 
 		if self.options.get('language', False):
@@ -154,8 +174,22 @@ class configmanager(object):
 		p = ConfigParser.ConfigParser()
 		p.add_section('options')
 		for o in [opt for opt in self.options.keys() if opt not in ('version','language','translate_out','translate_in','init','update')]:
-			p.set('options',o, self.options[o])
-		p.write(file(self.rcfile,'wb'))
+			p.set('options', o, self.options[o])
+
+		# try to create the directories and write the file
+		try:
+			if not os.path.exists(os.path.dirname(self.rcfile)):
+				os.makedirs(os.path.dirname(self.rcfile))
+			try:
+				p.write(file(self.rcfile, 'w'))
+			except IOError:
+				sys.stderr.write("ERROR: couldn't write the config file\n")
+
+		except OSError:
+			# what to do if impossible?
+			sys.stderr.write("ERROR: couldn't create the config directory\n")
+
+				
 
 	def get(self, key, default=None):
 		return self.options.get(key, default)
