@@ -17,52 +17,17 @@ class ConvertError(Exception):
 	def __str__(self):
 		return 'Exception:\n\t%s\nUsing file:\n%s' % (self.orig, self.d)
 
-# csv
-#TODO: there is almost the same code on the client. (in modules/gui/window/win_import.py)
-# merge it with this code...
-# maybe we should implement this as a method on objects?
-def convert_csv_import(model, input, default={}):
-	input = StringIO.StringIO(input)
-	pool = osv.osv.osv_pools
-	cr = sql_db.db.cursor()
-	uid = 1
-	reader = csv.reader(input)
-	# read the first line (the title of the columns)
-	for row in reader:
-		f=row
-		break
-	for row in reader:
-		dic={}
-		dic.update(default)
-		for i in range(len(f)):
-			if pool.get(model)._columns[f[i]]._type=='integer':
-				row[i] = (row[i] and int(row[i])) or False
-			elif pool.get(model)._columns[f[i]]._type=='float':
-				row[i] = (row[i] and float(row[i])) or False
-			elif pool.get(model)._columns[f[i]]._type=='many2one':
-				res = pool.get(obj).name_search(cr, uid, row[i], [], '=')
-				dic[f[i]] = False
-				if len(res):
-					dic[f[i]] = res[0][0]
-			dic[f[i]]=row[i]
-		pool.get(model).create(cr,uid,dic)
-	cr.commit()
-	cr.close()
-
-def convert_csv_export(csv):
-	pass
-
-def _eval_xml(node, pool, cr, uid, idref):
+def _eval_xml(self,node, pool, cr, uid, idref):
 	if node.nodeType == node.TEXT_NODE:
 		return node.data.encode("utf8")
 	elif node.nodeType == node.ELEMENT_NODE:
 		if node.nodeName in ('field','value'):
 			t = node.getAttribute('type') or 'char'
 			if len(node.getAttribute('search')):
-				f_search = node.getAttribute("search").encode('ascii')
+				f_search = node.getAttribute("search").encode('utf-8')
 				f_model = node.getAttribute("model").encode('ascii')
 				f_use = node.getAttribute("use").encode('ascii')
-				f_name = node.getAttribute("use").encode('ascii')
+				f_name = node.getAttribute("name").encode('utf-8')
 				if len(f_use)==0:
 					f_use = "id"
 				q = eval(f_search, idref)
@@ -82,6 +47,7 @@ def _eval_xml(node, pool, cr, uid, idref):
 			if len(a_eval):
 				import time
 				idref['time'] = time
+				idref['ref'] = lambda x: self.id_get(cr, False, x)
 				return eval(a_eval, idref)
 			if t == 'xml':
 				txt = '<?xml version="1.0"?>\n'+"".join([i.toxml().encode("utf8") for i in node.childNodes]) % idref
@@ -89,7 +55,7 @@ def _eval_xml(node, pool, cr, uid, idref):
 			if t in ('char', 'int', 'float'):
 				d = ""
 				for n in [i for i in node.childNodes]:
-					d+=str(_eval_xml(n,pool,cr,uid,idref))
+					d+=str(_eval_xml(self,n,pool,cr,uid,idref))
 				if t == 'int':
 					d = d.strip()
 					if d=='None':
@@ -102,13 +68,13 @@ def _eval_xml(node, pool, cr, uid, idref):
 			elif t in ('list','tuple'):
 				res=[]
 				for n in [i for i in node.childNodes if (i.nodeType == i.ELEMENT_NODE and i.nodeName=='value')]:
-					res.append(_eval_xml(n,pool,cr,uid,idref))
+					res.append(_eval_xml(self,n,pool,cr,uid,idref))
 				if t=='tuple':
 					return tuple(res)
 				return res
 		elif node.nodeName=="getitem":
 			for n in [i for i in node.childNodes if (i.nodeType == i.ELEMENT_NODE)]:
-				res=_eval_xml(n,pool,cr,uid,idref)
+				res=_eval_xml(self,n,pool,cr,uid,idref)
 			if not res:
 				raise LookupError
 			elif node.getAttribute('type') in ("int", "list"):
@@ -121,7 +87,7 @@ def _eval_xml(node, pool, cr, uid, idref):
 			if len(a_eval):
 				args = eval(a_eval, idref)
 			for n in [i for i in node.childNodes if (i.nodeType == i.ELEMENT_NODE)]:
-				args.append(_eval_xml(n, pool, cr, uid, idref))
+				args.append(_eval_xml(self,n, pool, cr, uid, idref))
 			model = pool.get(node.getAttribute('model'))
 			method = node.getAttribute('name')
 			res = getattr(model, method)(cr, uid, *args)
@@ -163,7 +129,7 @@ class xml_import(object):
 		return False
 
 	def _tag_function(self, cr, rec, data_node=None):
-		_eval_xml(rec, self.pool, cr, self.uid, self.idref)
+		_eval_xml(self,rec, self.pool, cr, self.uid, self.idref)
 		return False
 
 	def _tag_wizard(self, cr, rec, data_node=None):
@@ -189,8 +155,8 @@ class xml_import(object):
 			return False
 		res = {}
 		for field in [i for i in rec.childNodes if (i.nodeType == i.ELEMENT_NODE and i.nodeName=="field")]:
-			f_name = field.getAttribute("name").encode('ascii')
-			f_val = _eval_xml(field,self.pool, cr, self.uid, self.idref)
+			f_name = field.getAttribute("name").encode('utf-8')
+			f_val = _eval_xml(self,field,self.pool, cr, self.uid, self.idref)
 			res[f_name] = f_val
 		self.pool.get('ir.model.data').ir_set(cr, self.uid, res['key'], res['key2'], res['name'], res['models'], res['value'], replace=res.get('replace',True), isobject=res.get('isobject', False), meta=res.get('meta',None))
 		return False
@@ -210,11 +176,12 @@ class xml_import(object):
 		pid = False
 		for idx, menu_elem in enumerate(m_l):
 			if pid:
-				cr.execute('select * from ir_ui_menu where parent_id=%d and name=%s', (pid, menu_elem))
+				cr.execute('select id from ir_ui_menu where parent_id=%d and name=%s', (pid, menu_elem))
 			else:
-				cr.execute('select * from ir_ui_menu where parent_id is null and name=%s', (menu_elem,))
+				cr.execute('select id from ir_ui_menu where parent_id is null and name=%s', (menu_elem,))
 			res = cr.fetchone()
 			if res:
+				# the menuitem already exists
 				pid = res[0]
 				xml_id = idx==len(m_l)-1 and rec.getAttribute('id').encode('utf8')
 				try:
@@ -222,6 +189,7 @@ class xml_import(object):
 				except:
 					print 'Menu Error', self.module, xml_id, idx==len(m_l)-1
 			elif idx==len(m_l)-1:
+				# if we are at the last menu element
 				values = {'parent_id': pid,'name':menu_elem}
 
 				if rec.hasAttribute('action'):
@@ -258,7 +226,7 @@ class xml_import(object):
 		if rec_id and pid:
 			self.idref[rec_id] = pid
 
-		if rec.hasAttribute('action'):
+		if rec.hasAttribute('action') and pid:
 			a_action = rec.getAttribute('action').encode('utf8')
 			a_type = rec.getAttribute('type').encode('utf8') or 'act_window'
 			a_id = self.id_get(cr, 'ir.actions.%s' % a_type, a_action)
@@ -271,18 +239,38 @@ class xml_import(object):
 		model = self.pool.get(rec_model)
 		assert model, "The model %s does not exist !" % (rec_model,)
 		rec_id = rec.getAttribute("id").encode('ascii')
-		if data_node.getAttribute('noupdate') and not self.mode=='init':
-			id = self.pool.get('ir.model.data')._update_dummy(cr, self.uid, rec_model, self.module, rec_id or False)
-			if rec_id:
-				self.idref[rec_id] = id
-			return None
 
+#		if not rec_id and not data_node.getAttribute('noupdate'):
+#			print "Warning", rec_model
+
+		if data_node.getAttribute('noupdate') and not self.mode == 'init':
+			# check if the xml record has an id string
+			if rec_id:
+				id = self.pool.get('ir.model.data')._update_dummy(cr, self.uid, rec_model, self.module, rec_id)
+				# check if the resource already existed at the last update
+				if id:
+					# if it existed, we don't update the data, but we need to 
+					# know the id of the existing record anyway
+					self.idref[rec_id] = id
+					return None
+				else:
+					# if the resource didn't exist
+					if rec.getAttribute("forcecreate"):
+						# we want to create it, so we let the normal "update" behavior happen
+						pass
+					else:
+						# otherwise do nothing
+						return None
+			else:
+				# otherwise it is skipped
+				return None
+				
 		res = {}
 		for field in [i for i in rec.childNodes if (i.nodeType == i.ELEMENT_NODE and i.nodeName=="field")]:
 #TODO: most of this code is duplicated above (in _eval_xml)...
-			f_name = field.getAttribute("name").encode('ascii')
+			f_name = field.getAttribute("name").encode('utf-8')
 			f_ref = field.getAttribute("ref").encode('ascii')
-			f_search = field.getAttribute("search").encode('ascii')
+			f_search = field.getAttribute("search").encode('utf-8')
 			f_model = field.getAttribute("model").encode('ascii')
 			if not f_model and model._columns.get(f_name,False):
 				f_model = model._columns[f_name]._obj
@@ -296,12 +284,14 @@ class xml_import(object):
 				f_obj = self.pool.get(f_model)
 				# browse the objects searched
 				s = f_obj.browse(cr, self.uid, f_obj.search(cr, self.uid, q))
+				# column definitions of the "local" object
 				_cols = self.pool.get(rec_model)._columns
 				# if the current field is many2many
 				if (f_name in _cols) and _cols[f_name]._type=='many2many':
 					f_val = [(6, 0, map(lambda x: x[f_use], s))]
 				elif len(s):
-					# otherwise take the first element of the search
+					# otherwise (we are probably in a many2one field),
+					# take the first element of the search
 					f_val = s[0][f_use]
 			elif len(f_ref):
 				if f_ref=="null":
@@ -309,7 +299,7 @@ class xml_import(object):
 				else:
 					f_val = self.id_get(cr, f_model, f_ref)
 			else:
-				f_val = _eval_xml(field, self.pool, cr, self.uid, self.idref)
+				f_val = _eval_xml(self,field, self.pool, cr, self.uid, self.idref)
 				if model._columns.has_key(f_name):
 					if isinstance(model._columns[f_name], osv.fields.integer):
 						f_val = int(f_val)
@@ -336,7 +326,11 @@ class xml_import(object):
 			for rec in n.childNodes:
 				if rec.nodeType == rec.ELEMENT_NODE:
 					if rec.nodeName in self._tags:
-						self._tags[rec.nodeName](cr, rec, n)
+						try:
+							self._tags[rec.nodeName](cr, rec, n)
+						except:
+							print rec.toxml()
+							raise
 		cr.commit()
 		cr.close()
 		return True
@@ -358,8 +352,32 @@ class xml_import(object):
 			'workflow': self._tag_workflow,
 		}
 
+#
+# Import a CSV file:
+#     quote: "
+#     delimiter: ,
+#     encoding: UTF8
+#
+def convert_csv_import(fname, csvcontent, idref={}, mode='init'):
+	model = fname.split('.')[0].replace('_', '.')
+	pool = osv.osv.osv_pools
 
+	input=StringIO.StringIO(csvcontent)
+	data = list(csv.reader(input, quotechar='"', delimiter=','))
+
+	datas = []
+	for line in data:
+		datas.append( map(lambda x:x.decode('utf8').encode('utf8'), line))
+
+	uid = 1
+	cr=sql_db.db.cursor()
+	pool.get(model).import_data(cr, uid, data[0], datas[1:])
+	cr.commit()
+	cr.close()
+
+#
 # xml import/export
+#
 def convert_xml_import(module, xmlstr, idref={}, mode='init'):
 	obj = xml_import(module, idref, mode)
 	obj.parse(xmlstr)
@@ -378,3 +396,4 @@ def convert_xml_export(res):
 	de.appendChild(d.createTextNode('Some textual content.'))
 	cr.commit()
 	cr.close()
+
