@@ -31,8 +31,9 @@
 Miscelleanous tools used by tiny ERP.
 """
 
-import os
-import sys
+import os, time, sys
+import inspect
+
 import psycopg
 import netsvc
 import sql_db
@@ -99,22 +100,29 @@ def file_open(name, mode="r", subdir='addons'):
 #----------------------------------------------------------
 # Emails
 #----------------------------------------------------------
-def email_send(email_from, email_to, subject, body, on_error=False):
+def email_send(email_from, email_to, subject, body, email_cc=[], email_bcc=[], on_error=False):
 	"""Send an email."""
 	import smtplib
 	from email.MIMEText import MIMEText
 	from email.Header import Header
+	from email.Utils import formatdate
 	msg = MIMEText(body, _charset='utf-8')
 	msg['Subject'] = Header(subject, 'utf-8')
 	msg['From'] = email_from
-	msg['To'] = email_to
+	msg['To'] = ','.join(email_to)
+	if email_cc:
+		msg['Cc'] = ','.join(email_cc)
+	if email_bcc:
+		msg['Bcc'] = ','.join(email_bcc)
+	msg['Date'] = formatdate(localtime=True)
 	try:
 		s = smtplib.SMTP()
-		s.connect()
-		s.sendmail(email_from, [email_to], msg.as_string())
+		s.connect(config['smtp_server'])
+		s.sendmail(email_from, email_to + email_cc + email_bcc, msg.as_string())
 		s.quit()
-	except:
-		pass
+	except Exception, e:
+		import logging
+		logging.getLogger().info(str(e))
 
 #----------------------------------------------------------
 # SMS
@@ -162,4 +170,46 @@ class currency(float):
 	#	return str(display_value)
 
 
+#
+# Use it as a decorator of the function you plan to cache
+# Timeout: 0 = no timeout, otherwise in seconds
+#
+class cache(object):
+	def __init__(self, timeout=10000, skiparg=2):
+		self.timeout = timeout
+		self.cache = {}
+
+	def __call__(self, fn):
+		arg_names = inspect.getargspec(fn)[0][2:]
+		def cached_result(self2, cr=None, *args, **kwargs):
+			if cr is None:
+				self.cache = {}
+				return True
+			# Update named arguments with positional argument values
+			kwargs.update(dict(zip(arg_names, args)))
+			kwargs = kwargs.items()
+			kwargs.sort()
+			# Work out key as a tuple of ('argname', value) pairs
+			key = tuple(kwargs)
+
+			# Check cache and return cached value if possible
+			if key in self.cache:
+				(value, last_time) = self.cache[key]
+				mintime = time.time() - self.timeout
+				if self.timeout <= 0 or mintime <= last_time:
+					return value
+				else:
+					pass
+					#
+					# Clear Cache at this point ?
+					#
+
+			# Work out new value, cache it and return it
+			# Should copy() this value to avoid futur modf of the cacle ?
+			result = fn(self2,cr,**dict(kwargs))
+
+			self.cache[key] = (result, time.time())
+			return result
+		return cached_result
+
 # vim:noexpandtab
