@@ -256,6 +256,10 @@ def trans_load(filename, lang, strict=False):
 
 		ir.ir_set(cr, uid, 'meta', 'lang', 'lang', [('res.users',False,)], 'en', True, False, meta = {'type':'selection', 'string':'Language', 'selection':ls})
 
+		ids = pool.get('res.users').search(cr, uid, [('login','=','admin')])
+		for id in ids:
+			ir.ir_set(cr, uid, 'meta', 'lang', 'lang', [('res.users',id,)], lang, True, False)
+
 		reader = csv.reader(file(filename,'r').read().split('\n'))
 		
 		# read the first line of the file (it contains columns titles)
@@ -283,12 +287,12 @@ def trans_load(filename, lang, strict=False):
 				if dic['type'] == 'model' and not strict:
 					(model, field) = dic['name'].split(',')
 
-					# get the ids of the ressources of this model which share
+					# get the ids of the resources of this model which share
 					# the same source
 					obj = pool.get(model)
 					ids = osv.orm.orm.search(obj, cr, uid, [(field, '=', dic['src'])])
 
-					# if the ressource id (res_id) is in that list, use it, otherwise use the whole list
+					# if the resource id (res_id) is in that list, use it, otherwise use the whole list
 					ids = (dic['res_id'] in ids) and [dic['res_id']] or ids
 					for id in ids:
 						dic['res_id'] = id
