@@ -40,16 +40,18 @@ class workflow_service(netsvc.Service):
 		self.exportMethod(self.trg_delete)
 		self.exportMethod(self.trg_create)
 		self.exportMethod(self.trg_validate)
-		self.exportMethod(self.trg_merge)
+		self.exportMethod(self.trg_redirect)
+		self.exportMethod(self.trg_trigger)
 
 	def trg_write(self, uid, res_type, res_id, cr):
 		ident = (uid,res_type,res_id)
 		cr.execute('select id from wkf_instance where res_id=%d and res_type=%s and state=%s', (res_id,res_type, 'active'))
 		for (id,) in cr.fetchall():
 			instance.update(cr, id, ident)
+
+	def trg_trigger(self, uid, res_type, res_id, cr):
 		cr.execute('select instance_id from wkf_triggers where res_id=%d and model=%s', (res_id,res_type))
 		res = cr.fetchall()
-		#cr.execute('delete from wkf_triggers where model=%s and res_id=%d', (res_type,res_id))
 		for (instance_id,) in res:
 			cr.execute('select uid,res_type,res_id from wkf_instance where id=%d', (instance_id,))
 			ident = cr.fetchone()
@@ -67,17 +69,30 @@ class workflow_service(netsvc.Service):
 
 	def trg_validate(self, uid, res_type, res_id, signal, cr):
 		ident = (uid,res_type,res_id)
-		cr.execute('select id from wkf_instance where res_id=%d and res_type=%s and state=%s', (res_id,res_type, 'active'))
+		# ids of all active workflow instances for a corresponding resource (id, model_nam)
+		cr.execute('select id from wkf_instance where res_id=%d and res_type=%s and state=%s', (res_id, res_type, 'active'))
 		for (id,) in cr.fetchall():
 			instance.validate(cr, id, ident, signal)
 
-	def trg_merge(self, uid, res_type, res_id, new_rid, cr):
+	# make all workitems which are waiting for a (subflow) workflow instance
+	# for the old resource point to the (first active) workflow instance for
+	# the new resource
+	def trg_redirect(self, uid, res_type, res_id, new_rid, cr):
+		# get ids of wkf instances for the old resource (res_id)
+#CHECKME: shouldn't we get only active instances?
 		cr.execute('select id, wkf_id from wkf_instance where res_id=%d and res_type=%s', (res_id, res_type))
-		for inst_id, wkf_id in cr.fetchall():
-			cr.execute('select id from wkf_instance where res_id=%d and res_type=%s and wkf_id=%d', (new_rid, res_type, wkf_id))
+		for old_inst_id, wkf_id in cr.fetchall():
+			# first active instance for new resource (new_rid), using same wkf
+			cr.execute(
+				'SELECT id '\
+				'FROM wkf_instance '\
+				'WHERE res_id=%d AND res_type=%s AND wkf_id=%d AND state=%s', 
+				(new_rid, res_type, wkf_id, 'active'))
 			new_id = cr.fetchone()
 			if new_id:
-				cr.execute('select id from wkf_workitem where inst_id=%d', (inst_id,))
-				for item_id in cr.fetchall():
-					cr.execute('update wkf_workitem set inst_id=%d where id=%d', (new_id[0], item_id[0]))
+				# select all workitems which "wait" for the old instance
+				cr.execute('select id from wkf_workitem where subflow_id=%d', (old_inst_id,))
+				for (item_id,) in cr.fetchall():
+					# redirect all those workitems to the wkf instance of the new resource
+					cr.execute('update wkf_workitem set subflow_id=%d where id=%d', (new_id[0], item_id))
 workflow_service()
